#include <utility.h>
#include "toolbox.h"
#include "legend.h"

#define LEGSTR_INVALID_PANEL "Invalid Panel"
#define LEGSTR_INVALID_CTRL "Invalid control"
#define LEGSTR_OUT_OF_MEM "Out of memory"
#define LEGSTR_INVALID_POSITION "Invalid position"
#define LEGSTR_INVALID_LEGEND_NUMBER "Invalid legend number"
#define LEGSTR_INVALID_GRAPH_CONTROL "Control does not exist, is invalid or is not a graph control"
#define LEGSTR_MUST_RETAIN_DATA "Legend control data mode must be VAL_RETAIN"
#define LEGSTR_CANNOT_GET_LEGEND_DATA "Cannot get legend data"
#define LEGSTR_UNEXPECTED_NULL_PARAMETER "Unexpected NULL parameter"

#define LEGEND_CTRL_TYPE "Legend Control"
#define LEGEND_NUM_POINTS 7
#define LEGEND_TO_ANCHOR_GAP 3
#define DEFAULT_SAMPLE_WIDTH 30
static double sLegendY[LEGEND_NUM_POINTS] = {0.0,0.5,1.0,0.0,0.5,1.0,0.0};

int LEGENDAPI LGSetLegendItemAttributeFromParmInfo(int pan, int legendCtrl, int legendNumber, int attribute, va_list argList);
int LEGENDAPI LGSetLegendCtrlAttributeFromParmInfo(int pan, int legendCtrl, int attribute, va_list argList);

typedef struct {
    int plotHndl;
    int plotStyle;
    int pointStyle;
    int lineStyle;
    int color;
    char *text;
    int textColor;
    int textPlotHndl;
} *tLegendInfo;

typedef struct {
    int pan;
    int ctrl;
    int anchorCtrl;
    int showSamples;
    int autoSize;
    int relPos;
    int gap;
    int delayUpdate;
    char *textMetaFont;
    ListType legends;
} *tLegendCtrlInfo;

static int LegendCtrlCallback(int pan, int control, int event, void *callbackData, int eventData1, int eventData2);

static int ControlStyle(
    int pan,
    int ctrl)
{
    int status;
    int ctrlStyle;
    status = GetCtrlAttribute(pan, ctrl, ATTR_CTRL_STYLE, &ctrlStyle);
    
    if (status < 0)
        return -1;
    return ctrlStyle;
}

static int ControlExists(
    int pan,
    int anchorCtrl)
{
    int status;
    int dontCare;
    int breakState;

    if (anchorCtrl <= 0)
        return FALSE;
    breakState = GetBreakOnLibraryErrors ();
    if (breakState)
        DisableBreakOnLibraryErrors ();
    status = GetCtrlAttribute(pan, anchorCtrl, ATTR_WIDTH, &dontCare);
    if (breakState)
        EnableBreakOnLibraryErrors ();
    if (status != 0) 
        return FALSE;
    return TRUE;
}
    
static int ValidGraphControl(
    int pan,
    int graphCtrl)
{
    int status;
    int dontCare;
    int breakState;
    int ctrlStyle;
    
    if (graphCtrl <= 0)
        return FALSE;
    breakState = GetBreakOnLibraryErrors ();
    if (breakState)
        DisableBreakOnLibraryErrors ();
    status = GetCtrlAttribute(pan, graphCtrl, ATTR_WIDTH, &dontCare);
    if (breakState)
        EnableBreakOnLibraryErrors ();
    if (status != 0)
        return FALSE;
    ctrlStyle = ControlStyle(pan, graphCtrl);
    if (ctrlStyle != CTRL_GRAPH)
        return FALSE;
    return TRUE;
}

static int CreateLegendCtrlInfo(
    int pan, 
    int ctrl, 
    tLegendCtrlInfo *newLegendCtrlInfo)
{
    int errCode = LG_SUCCESS;
    tLegendCtrlInfo legendCtrlInfo = NULL;
    tLegendInfo legendInfo = NULL;

    legendCtrlInfo = (tLegendCtrlInfo)calloc(1,sizeof(*legendCtrlInfo));
    if (legendCtrlInfo == NULL) {errCode = LG_ERR_OUT_OF_MEM; goto Error;}

    if (ChainCtrlCallback(pan, ctrl, LegendCtrlCallback, 
            (void *)legendCtrlInfo, LEGEND_CTRL_TYPE) < 0) 
        {errCode = LG_ERR_CANNOT_GET_LEGEND_DATA; goto Error;}

    legendCtrlInfo->pan = pan;
    legendCtrlInfo->ctrl = ctrl;
    legendCtrlInfo->textMetaFont = StrDup(VAL_APP_META_FONT);
    legendCtrlInfo->legends = ListCreate(sizeof(legendInfo));
    legendCtrlInfo->delayUpdate = 0;

    if (newLegendCtrlInfo)
        *newLegendCtrlInfo = legendCtrlInfo;

Error:
    if (errCode < 0) {
        if (newLegendCtrlInfo)
            *newLegendCtrlInfo = 0;
        return errCode;
    } else
        return errCode;
}

static void DiscardLegendInfo(tLegendInfo legendInfo)
{
    free(legendInfo->text);
    free(legendInfo);
}

static void DiscardLegendCtrlInfo(tLegendCtrlInfo legendCtrlInfo)
{
    
    static ListType legends;
    int numItems;
    tLegendInfo legendInfo;

    if (legendCtrlInfo) {
        legends = legendCtrlInfo->legends;
        numItems = ListNumItems (legends);
    
        while (ListNumItems(legends)) {
            ListRemoveItem (legends, &legendInfo, END_OF_LIST);
            DiscardLegendInfo(legendInfo);
        }
        ListDispose (legends);
        free(legendCtrlInfo->textMetaFont);
        free(legendCtrlInfo);
    }
}

static int LegendCtrlCallback(
    int pan, 
    int control, 
    int event, 
    void *callbackData, 
    int eventData1, 
    int eventData2)
{
    tLegendCtrlInfo legendCtrlInfo =  (tLegendCtrlInfo)callbackData;

    UNUSED(pan);
    UNUSED(control);
    UNUSED(eventData1);
    UNUSED(eventData2);

    switch (event)
        {
        case EVENT_DISCARD:
            DiscardLegendCtrlInfo(legendCtrlInfo);
            break;
        }

    return 0;
}
static void LGSetAnchoredPosition(
    int pan,
    int legendCtrl,
    int anchorCtrl,
    int position,
    int gap)
{
    int anchorLeft, anchorTop, anchorHeight, anchorWidth;
    int newLeft, newTop, width, height;
   
    if (position == LG_POS_DONT_MOVE  || !ControlExists(pan, anchorCtrl))
        return;
    
    GetCtrlAttribute (pan, anchorCtrl, ATTR_LEFT, &anchorLeft);
    GetCtrlAttribute (pan, anchorCtrl, ATTR_TOP, &anchorTop);
    GetCtrlAttribute (pan, anchorCtrl, ATTR_WIDTH, &anchorWidth);
    GetCtrlAttribute (pan, anchorCtrl, ATTR_HEIGHT, &anchorHeight);
    
    GetCtrlAttribute (pan, legendCtrl, ATTR_WIDTH, &width);
    GetCtrlAttribute (pan, legendCtrl, ATTR_HEIGHT, &height);
    
    switch (position) {
        case LG_POS_DONT_MOVE:
            break;
        case LG_POS_ABOVE_LEFT:
            newTop = anchorTop - gap - height;
            newLeft = anchorLeft;
            break;
        case LG_POS_ABOVE_CENTER:
            newTop = anchorTop - gap - height;
            newLeft = anchorLeft + (anchorWidth / 2) - (width / 2);
            break;
        case LG_POS_ABOVE_RIGHT: 
            newTop = anchorTop - gap - height;
            newLeft = anchorLeft + anchorWidth - width;
            break;
        case LG_POS_RIGHT_TOP:
            newTop = anchorTop;
            newLeft = anchorLeft + anchorWidth + gap;
            break;
        case LG_POS_RIGHT_CENTER:
            newTop = anchorTop + (anchorHeight / 2) - (height / 2);
            newLeft = anchorLeft + anchorWidth + gap;
            break;
        case LG_POS_RIGHT_BOTTOM:
            newTop = anchorTop + anchorHeight - height;
            newLeft = anchorLeft + anchorWidth + gap;
            break;
        case LG_POS_BELOW_RIGHT:
            newTop = anchorTop + anchorHeight + gap;
            newLeft = anchorLeft + anchorWidth - width;
            break;
        case LG_POS_BELOW_CENTER:
            newTop = anchorTop + anchorHeight + gap;
            newLeft = anchorLeft + (anchorWidth / 2) - (width / 2);
            break;
        case LG_POS_BELOW_LEFT:
            newTop = anchorTop + anchorHeight + gap;
            newLeft = anchorLeft;
            break;
        case LG_POS_LEFT_BOTTOM:
            newTop = anchorTop + anchorHeight - height;
            newLeft = anchorLeft - width - gap;
            break;
        case LG_POS_LEFT_CENTER:
            newTop = anchorTop + (anchorHeight / 2) - (height / 2);
            newLeft = anchorLeft - width - gap;
            break;
        case LG_POS_LEFT_TOP:
            newTop = anchorTop;
            newLeft = anchorLeft - width - gap;
            break;
    }

    SetCtrlAttribute(pan, legendCtrl, ATTR_TOP, newTop);
    SetCtrlAttribute(pan, legendCtrl, ATTR_LEFT, newLeft);
}


static int LGNewCtrlAnchored(
    int pan,
    int anchorCtrl,
    int position,
    int gap,
    int xSizePercent,
    int ySizePercent,
    int ctrlStyle,
    char label[])
{
    int anchorLeft, anchorTop, anchorHeight, anchorWidth;
    int newLeft, newTop, newWidth, newHeight;
    int newCtrl;
    
    newCtrl = NewCtrl (pan, ctrlStyle, label, 0, 0);
    if (ControlExists(pan, anchorCtrl)) {
        GetCtrlAttribute (pan, anchorCtrl, ATTR_LEFT, &anchorLeft);
        GetCtrlAttribute (pan, anchorCtrl, ATTR_TOP, &anchorTop);
        GetCtrlAttribute (pan, anchorCtrl, ATTR_WIDTH, &anchorWidth);
        GetCtrlAttribute (pan, anchorCtrl, ATTR_HEIGHT, &anchorHeight);
    
        newWidth = (anchorWidth * xSizePercent) / 100;
        newHeight = (anchorHeight * ySizePercent) / 100;
    
        SetCtrlAttribute(pan, newCtrl, ATTR_WIDTH, newWidth);
        SetCtrlAttribute(pan, newCtrl, ATTR_HEIGHT, newHeight);
    
        switch (position) {
            case LG_POS_DONT_MOVE:
                break;
            case LG_POS_ABOVE_LEFT:
                newTop = anchorTop - gap - newHeight;
                newLeft = anchorLeft;
                break;
            case LG_POS_ABOVE_CENTER:
                newTop = anchorTop - gap - newHeight;
                newLeft = anchorLeft + (anchorWidth / 2) - (newWidth / 2);
                break;
            case LG_POS_ABOVE_RIGHT: 
                newTop = anchorTop - gap - newHeight;
                newLeft = anchorLeft + anchorWidth - newWidth;
                break;
            case LG_POS_RIGHT_TOP:
                newTop = anchorTop;
                newLeft = anchorLeft + anchorWidth + gap;
                break;
            case LG_POS_RIGHT_CENTER:
                newTop = anchorTop + (anchorHeight / 2) - (newHeight / 2);
                newLeft = anchorLeft + anchorWidth + gap;
                break;
            case LG_POS_RIGHT_BOTTOM:
                newTop = anchorTop + anchorHeight - newHeight;
                newLeft = anchorLeft + anchorWidth + gap;
                break;
            case LG_POS_BELOW_RIGHT:
                newTop = anchorTop + anchorHeight + gap;
                newLeft = anchorLeft + anchorWidth - newWidth;
                break;
            case LG_POS_BELOW_CENTER:
                newTop = anchorTop + anchorHeight + gap;
                newLeft = anchorLeft + (anchorWidth / 2) - (newWidth / 2);
                break;
            case LG_POS_BELOW_LEFT:
                newTop = anchorTop + anchorHeight + gap;
                newLeft = anchorLeft;
                break;
            case LG_POS_LEFT_BOTTOM:
                newTop = anchorTop + anchorHeight - newHeight;
                newLeft = anchorLeft - newWidth - gap;
                break;
            case LG_POS_LEFT_CENTER:
                newTop = anchorTop + (anchorHeight / 2) - (newHeight / 2);
                newLeft = anchorLeft - newWidth - gap;
                break;
            case LG_POS_LEFT_TOP:
                newTop = anchorTop;
                newLeft = anchorLeft - newWidth - gap;
                break;
        }
    
        SetCtrlAttribute(pan, newCtrl, ATTR_TOP, newTop);
        SetCtrlAttribute(pan, newCtrl, ATTR_LEFT, newLeft);
    }
    SetCtrlAttribute(pan, newCtrl, ATTR_BORDER_VISIBLE, 0);

    return newCtrl;

}
    
int LEGENDAPI LGCreateLegendControl (
    int pan, 
    int anchorCtrl, 
    int relPos,
    int autoSize, 
    int showSamples, 
    int *legendCtrl)
{
    int retCode = LG_SUCCESS;
    int newLegendCtrl = 0;
    int status = 0;
    int anchorStyle;
    int graphBG, plotBG;
    tLegendCtrlInfo legendCtrlInfo;
    
    if (pan <= 0) {retCode = LG_ERR_INVALID_PANEL; goto Error;}

    newLegendCtrl = LGNewCtrlAnchored(pan, anchorCtrl, relPos, 
            LEGEND_TO_ANCHOR_GAP, 20, 50, CTRL_GRAPH, "");
    if (newLegendCtrl <= 0) {retCode = newLegendCtrl; goto Error;}

    CreateLegendCtrlInfo(pan, newLegendCtrl, &legendCtrlInfo);
    if (legendCtrlInfo == NULL) {retCode = LG_ERR_OUT_OF_MEM; goto Error;}
    legendCtrlInfo->gap = LEGEND_TO_ANCHOR_GAP;
    legendCtrlInfo->anchorCtrl = anchorCtrl;
    legendCtrlInfo->showSamples = showSamples;
    legendCtrlInfo->autoSize = autoSize;
    legendCtrlInfo->relPos = relPos;

    SetCtrlAttribute(pan, newLegendCtrl, ATTR_XGRID_VISIBLE, 0);
    SetCtrlAttribute(pan, newLegendCtrl, ATTR_YGRID_VISIBLE, 0);
    SetCtrlAttribute(pan, newLegendCtrl, ATTR_XLABEL_VISIBLE, 0);
    SetCtrlAttribute(pan, newLegendCtrl, ATTR_YLABEL_VISIBLE, 0);
    
    if (ControlExists(pan, anchorCtrl)) {
        status = GetCtrlAttribute(pan, anchorCtrl, ATTR_CTRL_STYLE, &anchorStyle);
    
        /* match anchor control's graph and plot background colors */
        if (status >= 0 && (anchorStyle == CTRL_GRAPH || anchorStyle == CTRL_STRIP_CHART)) {
            status = GetCtrlAttribute(pan, anchorCtrl, ATTR_GRAPH_BGCOLOR, 
                    &graphBG);
            if (status >= 0)
                status = SetCtrlAttribute(pan, newLegendCtrl, ATTR_GRAPH_BGCOLOR,
                        graphBG);
            status = GetCtrlAttribute(pan, anchorCtrl, ATTR_PLOT_BGCOLOR, 
                    &plotBG);
            if (status >= 0)
                status = SetCtrlAttribute(pan, newLegendCtrl, ATTR_PLOT_BGCOLOR, 
                        plotBG);
        }
    }
    
    if (legendCtrl)
        *legendCtrl = newLegendCtrl;
    return retCode;
Error:
    return retCode;
}
int LEGENDAPI LGConvertGraphToLegend (
    int pan, 
    int legendCtrl, 
    int anchorCtrl,
    int relPos, 
    int autoSize, 
    int showSamples)
{
    int retCode = LG_SUCCESS;
    tLegendCtrlInfo legendCtrlInfo;
    int dataMode;

    if (pan <= 0) {retCode = LG_ERR_INVALID_PANEL; goto Error;}
    if (!ValidGraphControl(pan, legendCtrl))
        {retCode = LG_ERR_INVALID_GRAPH_CONTROL; goto Error;}
    if (legendCtrl <=0) {retCode = LG_ERR_INVALID_CTRL; goto Error;}
    GetCtrlAttribute (pan, legendCtrl, ATTR_DATA_MODE, &dataMode);
    if (dataMode != VAL_RETAIN)
        {retCode = LG_ERR_MUST_RETAIN_DATA; goto Error;}

    CreateLegendCtrlInfo(pan, legendCtrl, &legendCtrlInfo);
    if (legendCtrlInfo == NULL) {retCode = LG_ERR_OUT_OF_MEM; goto Error;}
    legendCtrlInfo->gap = LEGEND_TO_ANCHOR_GAP;
    legendCtrlInfo->anchorCtrl = anchorCtrl;
    legendCtrlInfo->showSamples = showSamples;
    legendCtrlInfo->autoSize = autoSize;
    legendCtrlInfo->relPos = relPos;

    SetCtrlAttribute(pan, legendCtrl, ATTR_XGRID_VISIBLE, 0);
    SetCtrlAttribute(pan, legendCtrl, ATTR_YGRID_VISIBLE, 0);
    SetCtrlAttribute(pan, legendCtrl, ATTR_XLABEL_VISIBLE, 0);
    SetCtrlAttribute(pan, legendCtrl, ATTR_YLABEL_VISIBLE, 0);

    
    LGSetAnchoredPosition(pan, legendCtrl, legendCtrlInfo->anchorCtrl,
                legendCtrlInfo->relPos, legendCtrlInfo->gap);
    
    return retCode;
Error:
    return retCode;
}

int DisplaySample(
    int pan,
    int legendCtrl,
    int i,
    tLegendCtrlInfo legendCtrlInfo,
    tLegendInfo legendInfo,
    double firstX,
    double sampXInc)
{
    int plotHndl;
    int j;
    double y2;
    double sampYInc;
    int xMode, yMode;
    double xMin, xMax, yMin, yMax;
    int numItems;
    
    
    if (!legendCtrlInfo->showSamples) {
        legendInfo->plotHndl = -1;
        return -1;
    }
    switch (legendInfo->plotStyle) {
        case VAL_THIN_LINE:
        case VAL_CONNECTED_POINTS:
        case VAL_SCATTER:                 
        case VAL_FAT_LINE:                
        case VAL_THIN_STEP:               
        case VAL_FAT_STEP:
            plotHndl = PlotWaveform (pan, legendCtrl, sLegendY, LEGEND_NUM_POINTS,
                          VAL_DOUBLE, 5, (i * 10) + 2, firstX, sampXInc,
                          legendInfo->plotStyle,legendInfo->pointStyle, 
                          legendInfo->lineStyle, 1, legendInfo->color);
            break;

        case VAL_VERTICAL_BAR:            
            for (j = 0; j < 6; j++) {
                plotHndl = PlotRectangle (pan, legendCtrl,
                                          (firstX - 1.0) + (j * sampXInc),
                                          (i * 10.0) + 2.0, 
                                          firstX + (((j + 1) * sampXInc) - 2.0), 
                                          (i * 10.0) + 3.0 + (sLegendY[j] * 6.0),
                                          legendInfo->color, legendInfo->color);
                    
            }
            break;
        case VAL_HORIZONTAL_BAR:
            GetAxisRange (pan, legendCtrl, &xMode, &xMin, &xMax, &yMode,
                          &yMin, &yMax);
            numItems = ListNumItems (legendCtrlInfo->legends);
            sampYInc = ((yMax - yMin) / 3.5 / numItems);
            for (j = 0; j < 3; j++) {
                plotHndl = PlotRectangle (pan, legendCtrl,
                                          (firstX - 1.0),
                                          (i * 10.0) + (j * sampYInc), 
                                          firstX + (( (sLegendY[j] + 0.2) * ((xMax - firstX) - 1.0) )), 
                                          (i * 10.0) + ((j + 1) * sampYInc - 1.0),
                                          legendInfo->color, legendInfo->color);
                    
            }
            break;
        case VAL_BASE_ZERO_VERTICAL_BAR:  
            for (j = 0; j < 6; j++) {
                if ((j + 1) % 3 == 0)
                    y2 = -1.0;
                else 
                    y2 = sLegendY[j];
                plotHndl = PlotRectangle (pan, legendCtrl,
                                          (firstX - 1.0) + (j * sampXInc),
                                          (i * 10.0) + 5.0, 
                                          firstX + (((j + 1) * sampXInc) - 2.0), 
                                          (i * 10.0) + 6.0 + (y2 * 6.0),
                                          legendInfo->color, legendInfo->color);
            }                   
            break;
        case VAL_BASE_ZERO_HORIZONTAL_BAR:
            GetAxisRange (pan, legendCtrl, &xMode, &xMin, &xMax, &yMode,
                          &yMin, &yMax);
            numItems = ListNumItems (legendCtrlInfo->legends);
            sampYInc = ((yMax - yMin) / 3.5 / numItems);
            for (j = 0; j < 3; j++) {
                if (j == 1) {
                    plotHndl = PlotRectangle (pan, legendCtrl,
                            (firstX + 5.0), (i * 10.0) + (j * sampYInc), 
                            firstX + (( (1.0 + 0.2) * ((xMax - firstX) - 7.0) )), 
                            (i * 10.0) + ((j + 1) * sampYInc - 1.0),
                            legendInfo->color, legendInfo->color);
                } else {
                    plotHndl = PlotRectangle (pan, legendCtrl,
                            (firstX + 5.0), (i * 10.0) + (j * sampYInc), 
                            firstX - (( (0.05) * ((xMax - firstX) - 7.0) )), 
                            (i * 10.0) + ((j + 1) * sampYInc - 1.0),
                            legendInfo->color, legendInfo->color);
                }
                    
            }
            break;
    }
    legendInfo->plotHndl = plotHndl;
    return plotHndl;
}
    

int LEGENDAPI LGDisplayLegendItems (
    int pan, 
    int legendCtrl)
{
    int plotHndl, textPlotHndl;
    tLegendCtrlInfo legendCtrlInfo = NULL;
    tLegendInfo legendInfo = NULL;
    int errCode = LG_SUCCESS;
    int i;
    ListType legends;
    int numItems;
    int height, width;
    int maxWidth = 0;
    int plotWidth, plotHeight;
    double spWidth;  /* the text width in terms of the axis range */
    double sampXInc;
    double plotXMax, plotYMax;
    double firstX;
    int borderVisible;
    int borderSlopHeight, borderSlopWidth;
    int borderHeight, borderWidth;

    SetCtrlAttribute(pan, legendCtrl, ATTR_REFRESH_GRAPH, 0);
    if (GetChainedCallbackData(pan, legendCtrl, LEGEND_CTRL_TYPE, 
            (void **)&legendCtrlInfo) < 0) 
        {errCode = LG_ERR_CANNOT_GET_LEGEND_DATA; goto Error;}
    DeleteGraphPlot (pan, legendCtrl, -1, VAL_DELAYED_DRAW);
    
    legends = legendCtrlInfo->legends;    
    numItems = ListNumItems (legends);
    
    for (i = 0; i < numItems; i++) {
        ListGetItem (legends, &legendInfo, i + 1);
        if (GetTextDisplaySize (legendInfo->text, legendCtrlInfo->textMetaFont, 
                &height, &width) != 0)
            continue;
        
        if (width > maxWidth)
            maxWidth = width;
    }
    GetCtrlAttribute (pan, legendCtrl, ATTR_PLOT_AREA_WIDTH, &plotWidth);
    GetCtrlAttribute (pan, legendCtrl, ATTR_PLOT_AREA_HEIGHT, &plotHeight);
    if (legendCtrlInfo->autoSize) {
        GetCtrlAttribute(pan, legendCtrl, ATTR_BORDER_VISIBLE, &borderVisible);
        if (borderVisible) {
            GetCtrlAttribute (pan, legendCtrl, ATTR_WIDTH, &borderWidth);
            GetCtrlAttribute (pan, legendCtrl, ATTR_HEIGHT, &borderHeight);
            borderSlopWidth = borderWidth - plotWidth;
            borderSlopHeight = borderHeight - plotHeight;
            
        } else {
            borderSlopHeight = 0;
            borderSlopWidth = 0;
        }
        SetCtrlAttribute(pan, legendCtrl, ATTR_HEIGHT, 
            (int)(borderSlopHeight + (numItems * (height + 1))) );
        if (legendCtrlInfo->showSamples) {
            SetCtrlAttribute(pan, legendCtrl, ATTR_WIDTH, 
                    (int)(borderSlopWidth + (maxWidth * 1.2) 
                          + DEFAULT_SAMPLE_WIDTH) );
        } else {
            SetCtrlAttribute(pan, legendCtrl, ATTR_WIDTH, (int) (maxWidth * 1.1));
        }
        LGSetAnchoredPosition(pan, legendCtrl, legendCtrlInfo->anchorCtrl,
                legendCtrlInfo->relPos, legendCtrlInfo->gap);
        GetCtrlAttribute (pan, legendCtrl, ATTR_PLOT_AREA_WIDTH, &plotWidth);
    }

    plotXMax = LEGEND_NUM_POINTS * 10;
    plotYMax = numItems * 10;
    SetAxisRange (pan, legendCtrl, VAL_MANUAL, 0, plotXMax, 
                                   VAL_MANUAL, -1, plotYMax + 2);
    spWidth = (maxWidth * LEGEND_NUM_POINTS * 10) / plotWidth;
    sampXInc = (plotXMax - spWidth) / LEGEND_NUM_POINTS;
        
    for (i = numItems; i > 0; i--) {
        ListGetItem (legends, &legendInfo, i);

        firstX = spWidth * 1.15;
        plotHndl = DisplaySample(pan, legendCtrl, numItems - i, legendCtrlInfo,
                legendInfo, firstX, sampXInc);
            
        textPlotHndl = PlotText(pan, legendCtrl, 0.0, (numItems - i) * 10.0,
                legendInfo->text, legendCtrlInfo->textMetaFont, 
                legendInfo->textColor, VAL_TRANSPARENT);

        legendInfo->textPlotHndl = textPlotHndl;
    }
    SetCtrlAttribute(pan, legendCtrl, ATTR_REFRESH_GRAPH, 1);
    return LG_SUCCESS;
Error:
    SetCtrlAttribute(pan, legendCtrl, ATTR_REFRESH_GRAPH, 1);
    return errCode;
}

int LEGENDAPI LGInsertLegendItem (
    int pan, 
    int legendCtrl, 
    int insertPosition,
    char legendText[], 
    int textColor,
    int plotStyle, 
    int pointStyle,
    int lineStyle, 
    int plotColor)
{
    tLegendCtrlInfo legendCtrlInfo = NULL;
    tLegendInfo legendInfo = NULL;
    int errCode = LG_SUCCESS;
    
    legendInfo = (tLegendInfo)calloc(1,sizeof(*legendInfo));
    if (legendInfo == NULL) {errCode = LG_ERR_OUT_OF_MEM; goto Error;}

    if (GetChainedCallbackData(pan, legendCtrl, LEGEND_CTRL_TYPE, 
        (void **)&legendCtrlInfo) < 0)
        {errCode = LG_ERR_CANNOT_GET_LEGEND_DATA; goto Error;}

    legendInfo->plotStyle = plotStyle;
    legendInfo->pointStyle = pointStyle;
    legendInfo->lineStyle = lineStyle;
    legendInfo->color = plotColor;
    legendInfo->text = StrDup(legendText);
    legendInfo->textColor = textColor;
           
    if (ListInsertItem (legendCtrlInfo->legends, &legendInfo, insertPosition) == 0) {
        errCode = LG_ERR_OUT_OF_MEM;
        goto Error;
    }

    if (!legendCtrlInfo->delayUpdate)
        LGDisplayLegendItems(pan, legendCtrl);
    return LG_SUCCESS;
Error:
    return errCode;
}

int LEGENDAPI LGInsertLegendItemForPlot (
    int pan, 
    int legendCtrl, 
    int insertPosition,
    char legendText[], 
    int textColor,
    int plotHndl)
{
    tLegendCtrlInfo legendCtrlInfo = NULL;
    tLegendInfo legendInfo = NULL;
    int errCode = LG_SUCCESS;
    int anchorCtrl;
    int status = 0;
    int anchorStyle;
    
    legendInfo = (tLegendInfo)calloc(1,sizeof(*legendInfo));
    if (legendInfo == NULL) {errCode = LG_ERR_OUT_OF_MEM; goto Error;}

    if (GetChainedCallbackData(pan, legendCtrl, LEGEND_CTRL_TYPE, 
        (void **)&legendCtrlInfo) < 0)
        {errCode = LG_ERR_CANNOT_GET_LEGEND_DATA; goto Error;}

    anchorCtrl = legendCtrlInfo->anchorCtrl;

    legendInfo->text = StrDup(legendText);
    legendInfo->textColor = textColor;

    if (ControlExists(pan,anchorCtrl)) {
        status = GetCtrlAttribute(pan, anchorCtrl, ATTR_CTRL_STYLE, &anchorStyle);
    }
                         
    if (status >= 0 && ControlExists(pan,anchorCtrl) && anchorStyle == CTRL_GRAPH) {
        status = GetPlotAttribute (pan, anchorCtrl, plotHndl,
                ATTR_TRACE_POINT_STYLE, &(legendInfo->pointStyle));
        if (status != 0) {
            errCode = status;
            legendInfo->pointStyle = VAL_EMPTY_SQUARE;
        }
        status = GetPlotAttribute (pan, anchorCtrl, plotHndl, ATTR_LINE_STYLE,
                &(legendInfo->lineStyle));
        if (status != 0) {
            errCode = status;
            legendInfo->lineStyle = VAL_SOLID;
        }
        status = GetPlotAttribute (pan, anchorCtrl, plotHndl, ATTR_PLOT_STYLE,
                &(legendInfo->plotStyle));
        if (status != 0) {
            errCode = status;
            legendInfo->plotStyle = VAL_THIN_LINE;
        }
        status = GetPlotAttribute (pan, anchorCtrl, plotHndl, ATTR_TRACE_COLOR,
                &(legendInfo->color));
        if (status != 0) {
            errCode = status;
            legendInfo->color = VAL_RED;
        }
        
    } else {
        legendInfo->plotStyle = VAL_THIN_LINE;
        legendInfo->pointStyle = VAL_EMPTY_SQUARE;
        legendInfo->lineStyle = VAL_SOLID;
        legendInfo->color = VAL_RED;
    }
    if (ListInsertItem (legendCtrlInfo->legends, &legendInfo, insertPosition) == 0) {
        errCode = LG_ERR_OUT_OF_MEM;
        goto Error;
    }

    if (!legendCtrlInfo->delayUpdate)
        LGDisplayLegendItems(pan, legendCtrl);
    return LG_SUCCESS;
Error:
    return errCode;
}

int LEGENDAPI LGDeleteLegendItem (
    int pan, 
    int legendCtrl, 
    int legendNumber)
{
    tLegendCtrlInfo legendCtrlInfo = NULL;
    tLegendInfo legendInfo = NULL;
    int errCode = LG_SUCCESS;
    ListType legends;
    int numItems;
    
    if (GetChainedCallbackData(pan, legendCtrl, LEGEND_CTRL_TYPE, 
        (void **)&legendCtrlInfo) < 0)
        {errCode = LG_ERR_CANNOT_GET_LEGEND_DATA; goto Error;}

    legends = legendCtrlInfo->legends;    
    numItems = ListNumItems (legends);
    if (legendNumber < FRONT_OF_LIST || legendNumber > numItems) {
        errCode = LG_ERR_INVALID_LEGEND_NUMBER;
        goto Error;
    }

    ListRemoveItem (legends, &legendInfo, legendNumber);
    if (legendInfo)
        DiscardLegendInfo(legendInfo);
    if (!legendCtrlInfo->delayUpdate)
        LGDisplayLegendItems(pan, legendCtrl);
    return LG_SUCCESS;
Error:
    return errCode;
}

int LEGENDAPI LGClearLegendCtrl (
    int pan, 
    int legendCtrl) 
{
    int i;
    tLegendCtrlInfo legendCtrlInfo = NULL;
    tLegendInfo legendInfo = NULL;
    int errCode = LG_SUCCESS;
    ListType legends;
    int numItems;
    
    if (GetChainedCallbackData(pan, legendCtrl, LEGEND_CTRL_TYPE, 
        (void **)&legendCtrlInfo) < 0)
        {errCode = LG_ERR_CANNOT_GET_LEGEND_DATA; goto Error;}

    legends = legendCtrlInfo->legends;    
    numItems = ListNumItems (legends);
    for (i = numItems; i > 0; i--) {
        ListRemoveItem (legends, &legendInfo, i);
        if (legendInfo)
            DiscardLegendInfo(legendInfo);
    }

    if (!legendCtrlInfo->delayUpdate)
        LGDisplayLegendItems(pan, legendCtrl);
    return LG_SUCCESS;
Error:
    return errCode;
}

int LEGENDAPI LGNumberOfLegendItems (
    int pan, 
    int legendCtrl,
    int *numItems) 
{
    tLegendCtrlInfo legendCtrlInfo = NULL;
    int errCode = LG_SUCCESS;
    ListType legends;
    
    if (GetChainedCallbackData(pan, legendCtrl, LEGEND_CTRL_TYPE, 
        (void **)&legendCtrlInfo) < 0)
        {errCode = LG_ERR_CANNOT_GET_LEGEND_DATA; goto Error;}

    legends = legendCtrlInfo->legends;    
    *numItems = ListNumItems (legends);
    return LG_SUCCESS;
Error:
    *numItems = -1;
    return errCode;
}

int LEGENDAPI LGGetLegendCtrlAttribute(
    int pan, 
    int legendCtrl, 
    int attribute, 
    void *value)
{
    int errCode = LG_SUCCESS;
    tLegendCtrlInfo legendCtrlInfo = NULL;

    if (!value) {errCode = LG_ERR_UNEXPECTED_NULL_PARAMETER; goto Error;}

    if (GetChainedCallbackData(pan, legendCtrl, LEGEND_CTRL_TYPE, 
        (void **)&legendCtrlInfo) < 0)
        {errCode = LG_ERR_CANNOT_GET_LEGEND_DATA; goto Error;}

    switch (attribute) {
        case LG_ATTR_META_FONT_LEN:
            *(int *)value = (legendCtrlInfo->textMetaFont == NULL) ? 0: strlen(legendCtrlInfo->textMetaFont);
            break;
        case LG_ATTR_META_FONT:
            strcpy(value, legendCtrlInfo->textMetaFont);
            break;
        case LG_ATTR_SHOW_SAMPLES:
            *(int *)value = legendCtrlInfo->showSamples;
            break;
        case LG_ATTR_REL_POS:
            *(int *)value = legendCtrlInfo->relPos;
            break;
        case LG_ATTR_GRAPH_BG_COLOR:
            GetCtrlAttribute(pan, legendCtrl, ATTR_GRAPH_BGCOLOR, value); 
            break;
        case LG_ATTR_PLOT_BG_COLOR:
            GetCtrlAttribute(pan, legendCtrl, ATTR_PLOT_BGCOLOR, value); 
            break;
        case LG_ATTR_AUTO_SIZE:
            *(int *)value = legendCtrlInfo->autoSize;
            break;
        case LG_ATTR_OFFSET_FROM_ANCHOR:
            *(int *)value = legendCtrlInfo->gap;
            break;
        case LG_ATTR_ANCHOR_CTRL:
            *(int *)value = legendCtrlInfo->anchorCtrl;
            break;
        case LG_ATTR_DELAY_UPDATE:
            *(int *)value = legendCtrlInfo->delayUpdate;
            break;
    }

Error:
    return errCode;
}

int LEGENDAPI_CDECL LGSetLegendCtrlAttribute(
    int pan, 
    int legendCtrl, 
    int attribute, 
    ...)
{
    int error;
    va_list parmInfo;
    
    va_start(parmInfo, attribute);
    error = LGSetLegendCtrlAttributeFromParmInfo(pan, legendCtrl, attribute, parmInfo);
    va_end(parmInfo);
    
    return error;
}

int LEGENDAPI LGSetLegendCtrlAttributeFromParmInfo(
    int pan, 
    int legendCtrl, 
    int attribute, 
    va_list parmInfo)
{
    int errCode = LG_SUCCESS;
    tLegendCtrlInfo legendCtrlInfo = NULL;
    int tempInt;

    if (GetChainedCallbackData(pan, legendCtrl, LEGEND_CTRL_TYPE, 
            (void **)&legendCtrlInfo) < 0)
        {errCode = LG_ERR_CANNOT_GET_LEGEND_DATA; goto Error;}

    /* va_start(parmInfo, attribute);*/

    switch (attribute) {
        case LG_ATTR_META_FONT:
            free(legendCtrlInfo->textMetaFont);
            legendCtrlInfo->textMetaFont = StrDup((char *)va_arg(parmInfo, char *));
            break;
        case LG_ATTR_SHOW_SAMPLES:
            legendCtrlInfo->showSamples = va_arg(parmInfo, int);
            break;
        case LG_ATTR_REL_POS:
            tempInt = va_arg(parmInfo, int);
            if (tempInt < LG_POS_DONT_MOVE || tempInt > LG_POS_LEFT_TOP) {
                errCode = LG_ERR_INVALID_POSITION;
                goto Error;
            }
            legendCtrlInfo->relPos = tempInt;
            LGSetAnchoredPosition(pan, legendCtrl, legendCtrlInfo->anchorCtrl,
                    tempInt, legendCtrlInfo->gap);
            break;
        case LG_ATTR_GRAPH_BG_COLOR:
            tempInt = va_arg(parmInfo, int);
            SetCtrlAttribute(pan, legendCtrl, ATTR_GRAPH_BGCOLOR, tempInt); 
            break;
        case LG_ATTR_PLOT_BG_COLOR:
            tempInt = va_arg(parmInfo, int);
            SetCtrlAttribute(pan, legendCtrl, ATTR_PLOT_BGCOLOR, tempInt); 
            break;
        case LG_ATTR_AUTO_SIZE:
            legendCtrlInfo->autoSize = va_arg(parmInfo, int);
            break;
        case LG_ATTR_OFFSET_FROM_ANCHOR:
            tempInt = va_arg(parmInfo, int);
            legendCtrlInfo->gap = tempInt;
            LGSetAnchoredPosition(pan, legendCtrl, legendCtrlInfo->anchorCtrl,
                    legendCtrlInfo->relPos, tempInt);
            break;
        case LG_ATTR_ANCHOR_CTRL:
            tempInt = va_arg(parmInfo, int);
            if (tempInt <= 0) {
                errCode = LG_ERR_INVALID_CTRL;
                goto Error;
            }
            legendCtrlInfo->anchorCtrl = tempInt;
            LGSetAnchoredPosition(pan, legendCtrl, legendCtrlInfo->anchorCtrl,
                    legendCtrlInfo->relPos, legendCtrlInfo->gap);
            break;
        case LG_ATTR_DELAY_UPDATE:
            tempInt = va_arg(parmInfo, int);
            legendCtrlInfo->delayUpdate = tempInt;
            break;
    }
    if (!legendCtrlInfo->delayUpdate)
        LGDisplayLegendItems(pan, legendCtrl);
        
    /* va_end(parmInfo); */

Error:
    return errCode;
}


int LEGENDAPI LGGetLegendItemAttribute(
    int pan, 
    int legendCtrl,
    int legendNumber,
    int attribute, 
    void *value)
{
    int errCode = LG_SUCCESS;
    tLegendCtrlInfo legendCtrlInfo = NULL;
    ListType legends;
    tLegendInfo legendInfo = NULL;
    int numItems;

    if (!value) {errCode = LG_ERR_UNEXPECTED_NULL_PARAMETER; goto Error;}

    if (GetChainedCallbackData(pan, legendCtrl, LEGEND_CTRL_TYPE, 
            (void **)&legendCtrlInfo) < 0)
        {errCode = LG_ERR_CANNOT_GET_LEGEND_DATA; goto Error;}

    legends = legendCtrlInfo->legends;    
    numItems = ListNumItems (legends);
    if (legendNumber < FRONT_OF_LIST || legendNumber > numItems) 
        {errCode = LG_ERR_INVALID_LEGEND_NUMBER; goto Error;}

    ListGetItem (legends, &legendInfo, legendNumber);

    switch (attribute) {
        case LG_ATTR_LEGEND_TEXT_LEN:
            *(int *)value =  (legendInfo->text == NULL) ? 0: strlen(legendInfo->text);
            break;
        case LG_ATTR_LEGEND_TEXT:
            strcpy(value, legendInfo->text);
            break;
        case LG_ATTR_TEXT_COLOR:
            *(int *)value = legendInfo->textColor;
            break;
        case LG_ATTR_PLOT_STYLE: 
            *(int *)value = legendInfo->plotStyle;
            break;
        case LG_ATTR_POINT_STYLE:
            *(int *)value = legendInfo->pointStyle;
            break;
        case LG_ATTR_LINE_STYLE:
            *(int *)value = legendInfo->lineStyle;
            break;
        case LG_ATTR_PLOT_COLOR:   
            *(int *)value = legendInfo->color;
            break;
    }

Error:
    return errCode;
}

int LEGENDAPI_CDECL LGSetLegendItemAttribute(
    int pan, 
    int legendCtrl, 
    int legendNumber,
    int attribute, 
    ...)
{
    int error;
    va_list parmInfo;
    
    va_start(parmInfo, attribute);
    error = LGSetLegendItemAttributeFromParmInfo(pan, legendCtrl, legendNumber, attribute, parmInfo);
    va_end(parmInfo);
    
    return error;
}

int LEGENDAPI LGSetLegendItemAttributeFromParmInfo(
    int pan, 
    int legendCtrl, 
    int legendNumber,
    int attribute, 
    va_list parmInfo)
{
    int errCode = LG_SUCCESS;
    tLegendCtrlInfo legendCtrlInfo = NULL;
    ListType legends;
    tLegendInfo legendInfo = NULL;
    int numItems;

    SetCtrlAttribute(pan, legendCtrl, ATTR_REFRESH_GRAPH, 0);
    if (GetChainedCallbackData(pan, legendCtrl, LEGEND_CTRL_TYPE, 
            (void **)&legendCtrlInfo) < 0) 
        {errCode = LG_ERR_CANNOT_GET_LEGEND_DATA; goto Error;}

    legends = legendCtrlInfo->legends;    
    numItems = ListNumItems (legends);
    if (legendNumber < FRONT_OF_LIST || legendNumber > numItems) 
        {errCode = LG_ERR_INVALID_LEGEND_NUMBER; goto Error;}

    ListGetItem (legends, &legendInfo, legendNumber);
    switch (attribute) {
        case LG_ATTR_LEGEND_TEXT:
            if (legendInfo->text != NULL)
                free(legendInfo->text);
            legendInfo->text= StrDup((char *)va_arg(parmInfo, char *));
            break;
        case LG_ATTR_TEXT_COLOR:
            legendInfo->textColor = va_arg(parmInfo, int);
            break;
        case LG_ATTR_PLOT_STYLE: 
            legendInfo->plotStyle = va_arg(parmInfo, int);
            break;
        case LG_ATTR_POINT_STYLE:
            legendInfo->pointStyle = va_arg(parmInfo, int);
            break;
        case LG_ATTR_LINE_STYLE:
            legendInfo->lineStyle = va_arg(parmInfo, int);
            break;
        case LG_ATTR_PLOT_COLOR:   
            legendInfo->color = va_arg(parmInfo, int);
            break;
    }
    if (!legendCtrlInfo->delayUpdate)
        LGDisplayLegendItems(pan, legendCtrl);

    return LG_SUCCESS;
Error:
    va_end(parmInfo);
    return errCode;
}

char * LEGENDAPI LGGetErrorText(int errCode)
{
    switch (errCode) {
        case LG_ERR_INVALID_PANEL:
            return LEGSTR_INVALID_PANEL;
        case LG_ERR_INVALID_CTRL:
            return LEGSTR_INVALID_CTRL;
        case LG_ERR_OUT_OF_MEM:
            return LEGSTR_OUT_OF_MEM;
        case LG_ERR_INVALID_POSITION:
            return LEGSTR_INVALID_POSITION;
        case LG_ERR_INVALID_LEGEND_NUMBER:
            return LEGSTR_INVALID_LEGEND_NUMBER;
        case LG_ERR_INVALID_GRAPH_CONTROL:
            return LEGSTR_INVALID_GRAPH_CONTROL;
        case LG_ERR_MUST_RETAIN_DATA:
            return LEGSTR_MUST_RETAIN_DATA;
        case LG_ERR_CANNOT_GET_LEGEND_DATA:
            return LEGSTR_CANNOT_GET_LEGEND_DATA;
        case LG_ERR_UNEXPECTED_NULL_PARAMETER:
            return LEGSTR_UNEXPECTED_NULL_PARAMETER;
    }
    return NULL;
}
