/************************************************************************
   Purpose:  Configures the HP (Agilent Technologies) 33120A 15 Mhz 
   			 Function/Arbitrary Waveform Generator VXI device using VISA.
   			 
   Project Name: project5.prj 

*************************************************************************/

#include "hp33120a.h"
#include <utility.h>
#include <formatio.h>
#include <userint.h>
#include <ansi_c.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <gpib.h> 
#include <visa.h>
#include "project5.h" 
#define BUFFER_LEN 256		//Buffer Length

static int AWGPanelHandle;
int  DriverIndex;
short int  ImpedanceIndex; 

ViStatus status = VI_SUCCESS; 
ViSession  InstrHandle, ResourceManager; 
ViChar buffer[BUFFER_LEN], bytes[BUFFER_LEN], errStr[BUFFER_LEN];  
unsigned char read_buf[BUFFER_LEN], cmdString[BUFFER_LEN];
ViReal64 Impedance;

int SetupCommunication(void);

int main()
{
	    ibonl(0,1);//put GPIB board on line
		AWGPanelHandle =LoadPanel(0, "project5.uir", AWG_PNL);
		DisplayPanel(AWGPanelHandle); 
		//Open VISA Default Resource Manager
		status = viOpenDefaultRM(&ResourceManager);
		if (status < VI_SUCCESS) 
		{   
			sprintf (buffer, "Error Opening Resource Manager...Exiting Program");
			InsertTextBoxLine (AWGPanelHandle, AWG_PNL_DISP, -1, buffer);
			viClose (ResourceManager);
			return -1;  
		}   
		RunUserInterface();
} //main

//Find VISA Resources
int CVICALLBACK FindResourcesCB (int panel, int control, int event,
		void *callbackData, int eventData1, int eventData2)
{
	int counter=0;
	char InstrumentDesc[BUFFER_LEN];
	ViFindList ResourceList;
	ViUInt32 Matches;
	
	switch (event)
	{
	 case EVENT_COMMIT:
		   
	//Find all VISA "INSTR"  resources on the system 
	status = viFindRsrc (ResourceManager, "?*INSTR", &ResourceList, &Matches,
						 									InstrumentDesc);
	if (status < VI_SUCCESS)  
	{
		sprintf (buffer, "Error in finding resources.... Exititng Program"); 
		InsertTextBoxLine (AWGPanelHandle, AWG_PNL_DISP, -1, buffer);
		viClose (ResourceManager);  //Close Resource Manager
		return -1;  
	}
	
	//Find all instruments on the system 
	while (counter < Matches)
	{	 
		if (counter != 0) 
		{	//Find next device   
			status = viFindNext (ResourceList, InstrumentDesc);  
			if (status < VI_SUCCESS) 
			{   
				sprintf (buffer, "An error occurred finding the next resource");
				InsertTextBoxLine (AWGPanelHandle, AWG_PNL_DISP, -1, buffer);
				viClose (ResourceManager);
				return -1;  
			}   
		}//end if	 
	
		//Display instrument found
		sprintf(buffer, "Found device:%s", InstrumentDesc);
		InsertTextBoxLine (AWGPanelHandle, AWG_PNL_DISP, -1, buffer);
	
		counter++;   //instruments found counter
	}// end while
	viClose(ResourceList);	
	break;
	}
return 0;
} //FindResourcesCB

//Open Resources and setup the device
int SetupCommunication(void)
{
	//Open a session to the device 
	status = viOpen(ResourceManager,"VXI0::10::INSTR",VI_NULL, 
											VI_NULL,&InstrHandle);
	if (status < VI_SUCCESS)
	{
	   viStatusDesc (InstrHandle, status, errStr);
	   MessagePopup ("Error!", errStr);
	}

	//Set time-out to 5 seconds
	status = viSetAttribute (InstrHandle, VI_ATTR_TMO_VALUE, 5000);
	if (status < VI_SUCCESS)
	{
	   viStatusDesc (InstrHandle, status, errStr);
	   MessagePopup ("Error!", errStr);
	}
	
	//Initialize the device
	status = viWrite (InstrHandle, (ViBuf)"*RST", 5, VI_NULL);
	status = viWrite (InstrHandle, (ViBuf)"*IDN?", 6, VI_NULL); 
	status = viRead (InstrHandle, read_buf,BUFFER_LEN, VI_NULL); 
	InsertTextBoxLine (AWGPanelHandle, AWG_PNL_DISP, -1, (char *)read_buf);   
		
 return 0;
} //SetupCommunication

//Invoked from CONFIG. AWG to configure the AWG 
int CVICALLBACK ConfigAWGCB (int panel, int control, int event,
		void *callbackData, int eventData1, int eventData2)
{
	int WaveIndex;
    ViReal64 frequency, Amp, DC_Offset;
	ViChar impedance[BUFFER_LEN], Waveform[BUFFER_LEN]; 
	unsigned char imp_buf[BUFFER_LEN];
	
	switch (event)
	{
	  case EVENT_COMMIT:
    	SetupCommunication();  //Open resources 
		GetCtrlVal(AWGPanelHandle,AWG_PNL_FREQ, &frequency);
		//Get impedance
		GetCtrlVal(AWGPanelHandle,AWG_PNL_IMP, &ImpedanceIndex);
		if (ImpedanceIndex)   //50 ohms impedance
		{
			GetCtrlVal(AWGPanelHandle,AWG_PNL_AMP_50_OHMS, &Amp);
			GetCtrlVal(AWGPanelHandle,AWG_PNL_DC_50HMS, &DC_Offset);
		
			Fmt(cmdString, "%s<OUTP:LOAD 50"); //Set impedance to 50 ohms
			status = viWrite (InstrHandle, cmdString, NumFmtdBytes(), VI_NULL);
			//read back configured impedance
			Fmt(cmdString, "%s<OUTP:LOAD?");   
			status = viWrite (InstrHandle, cmdString, NumFmtdBytes(), VI_NULL);
			status = viRead (InstrHandle, imp_buf,BUFFER_LEN, VI_NULL);
			Fmt(impedance, "%s<Impedance = %s\n", imp_buf);
			InsertTextBoxLine (AWGPanelHandle, AWG_PNL_DISP, -1, impedance);  
			
		}
		else    //Open circuit
		{
			GetCtrlVal(AWGPanelHandle,AWG_PNL_AMP_OPEN_OHMS, &Amp);
			GetCtrlVal(AWGPanelHandle,AWG_PNL_OPEN_DC, &DC_Offset);
			
			Fmt(cmdString, "%s<OUTP:LOAD INF"); //Set impedance to OPEN
			status = viWrite (InstrHandle, cmdString, NumFmtdBytes(), VI_NULL);
			Fmt(cmdString, "%s<OUTP:LOAD?");  //read back configured impedance 
			status = viWrite (InstrHandle, cmdString, NumFmtdBytes(), VI_NULL);
			status = viRead (InstrHandle, imp_buf,BUFFER_LEN, VI_NULL);
			Fmt(impedance, "%s<Impedance = %s\n", imp_buf);
			InsertTextBoxLine (AWGPanelHandle, AWG_PNL_DISP, -1, impedance);
			
		} 	
	
	    //Select waveform
	    GetCtrlIndex(AWGPanelHandle,AWG_PNL_WAVE_SHAPE, &WaveIndex);
	    switch(WaveIndex)
	    {
	     	   case 0:
						Fmt(cmdString, "%s<APPL:SIN %f, %f VPP, %f V",
															frequency, Amp, DC_Offset);
						status = viWrite (InstrHandle, cmdString, NumFmtdBytes(), 
																			VI_NULL);
						Fmt(cmdString, "%s<APPL?");  //read back configuration
						status = viWrite (InstrHandle, cmdString, NumFmtdBytes(),
																				VI_NULL);
						status = viRead (InstrHandle, read_buf,BUFFER_LEN, VI_NULL);
						CopyString(Waveform, 0, "Sine",0, -1); 
	     	   		 break;
	     	   case 1:
						Fmt(cmdString, "%s<APPL:SQU %f, %f, %f ",
															frequency, Amp, DC_Offset);
						status = viWrite (InstrHandle, cmdString, NumFmtdBytes(), 
																				VI_NULL);
						Fmt(cmdString, "%s<APPL?");  //read back configuration  
						status = viWrite (InstrHandle, cmdString, NumFmtdBytes(), 
																				VI_NULL);
						status = viRead (InstrHandle, read_buf,BUFFER_LEN, VI_NULL);
						CopyString(Waveform, 0, "Square",0, -1); 
	     	   		 break;
	     	   case 2:
					 	//Set Waveform shape, amplitude, offset and frequency
						Fmt(cmdString, "%s<APPL:TRI %f, %f VPP, %f V",
															frequency, Amp, DC_Offset);
						status = viWrite (InstrHandle, cmdString, NumFmtdBytes(), 
																				VI_NULL);
						Fmt(cmdString, "%s<APPL?");  //read back configuration
						status = viWrite (InstrHandle, cmdString, NumFmtdBytes(), 
																				VI_NULL);
						status = viRead (InstrHandle, read_buf,BUFFER_LEN, VI_NULL);
					
						CopyString(Waveform, 0, "Triangle",0, -1); 
	     	   		break;
     	   
	     	   case 3:
					 	//Set Waveform shape, amplitude, offset and frequency
						Fmt(cmdString, "%s<APPL:RAMP %f, %f VPP, %f V",
															frequency, Amp, DC_Offset );
						status = viWrite (InstrHandle, cmdString, NumFmtdBytes(), 
																			VI_NULL);
						Fmt(cmdString, "%s<APPL?");  //read back configuration
						status = viWrite (InstrHandle, cmdString, NumFmtdBytes(),
																				VI_NULL);
						status = viRead (InstrHandle, read_buf,BUFFER_LEN, VI_NULL);
						CopyString(Waveform, 0, "Ramp",0, -1); 
	                break;
     	   		
	     	   case 4:
					 	//Set Waveform shape, amplitude, offset and frequency
						Fmt(cmdString, "%s<APPL:DC %f, %f VPP, %f V",
															frequency, Amp, DC_Offset );
						status = viWrite (InstrHandle, cmdString, NumFmtdBytes(), 
																			VI_NULL);
						Fmt(cmdString, "%s<APPL?");  //read back configuration  
						status = viWrite (InstrHandle, cmdString, NumFmtdBytes(), 
																			VI_NULL);
						status = viRead (InstrHandle, read_buf,BUFFER_LEN, 
																			VI_NULL);
						CopyString(Waveform, 0, "DC",0, -1); 
	     	   		break;
     	   
	     	   case 5:
					 	//Set Waveform shape, amplitude, offset and frequency
						Fmt(cmdString, "%s<APPL:NOIS %f, %f VPP, %f V",
															frequency, Amp, DC_Offset );
						status = viWrite (InstrHandle, cmdString, NumFmtdBytes(), 
																				VI_NULL);
						Fmt(cmdString, "%s<APPL?");  //read back configuration    
						status = viWrite (InstrHandle, cmdString, NumFmtdBytes(),
																				VI_NULL);
						status = viRead (InstrHandle, read_buf,BUFFER_LEN, VI_NULL);
						CopyString(Waveform, 0, "Noise",0, -1); 
	     	   		break;
	     	   	default:
	     	   		Fmt(buffer, "%s<Error in Wavetype selection");
		 			InsertTextBoxLine (AWGPanelHandle, AWG_PNL_DISP, -1, buffer);
	     	   		break;
	     	   		
	     	}
		   
		   	Fmt(buffer, "%s<WAVE = %s FREQ= %f  AMPLITUDE= %f  OFFSET = %f",
		   	 									Waveform,frequency,Amp,DC_Offset);
		   	InsertTextBoxLine (AWGPanelHandle, AWG_PNL_DISP, -1, buffer); 
	   	
    break;
  }
 
   return 0;
} //ConfigAWGCB

//Obtain selected impedance from impedance switch and dim, un-dim controls
int CVICALLBACK ImpedanceSelectorCB (int panel, int control, int event,
		void *callbackData, int eventData1, int eventData2)
{
	switch (event)
		{
		case EVENT_COMMIT:
			
			GetCtrlVal(AWGPanelHandle,AWG_PNL_IMP, &ImpedanceIndex);
			if (ImpedanceIndex)   //50 ohms setup
			{   //Dim and un-dim the controls
				SetCtrlAttribute(AWGPanelHandle,AWG_PNL_DC_50HMS,ATTR_DIMMED,0);
				SetCtrlAttribute(AWGPanelHandle,AWG_PNL_AMP_50_OHMS,ATTR_DIMMED,0);
				SetCtrlAttribute (AWGPanelHandle, AWG_PNL_OPEN_DC, ATTR_DIMMED, 1);   
				SetCtrlAttribute(AWGPanelHandle,AWG_PNL_AMP_OPEN_OHMS,ATTR_DIMMED,1);
			}
			else  //open-circuit
			{
				SetCtrlAttribute(AWGPanelHandle,AWG_PNL_DC_50HMS,ATTR_DIMMED,1);
				SetCtrlAttribute(AWGPanelHandle,AWG_PNL_AMP_50_OHMS,ATTR_DIMMED,1);
				SetCtrlAttribute (AWGPanelHandle, AWG_PNL_OPEN_DC, ATTR_DIMMED, 0);   
				SetCtrlAttribute(AWGPanelHandle,AWG_PNL_AMP_OPEN_OHMS,ATTR_DIMMED,0);
				} 	
			
			break;
		}
	return 0;
} //ImpedanceSelectorCB

//Invoked from CLEAR DISPLAY button
int CVICALLBACK ClearTextBoxCB (int panel, int control, int event,
		void *callbackData, int eventData1, int eventData2)
{
	switch (event)
	{
		case EVENT_COMMIT:
			DefaultCtrl(AWGPanelHandle,AWG_PNL_DISP);
			break;
	}
	return 0;
}//ClearTextBoxCB

//Exit device
int CVICALLBACK ExitAWGCB (int panel, int control, int event,
		void *callbackData, int eventData1, int eventData2)
{
	switch (event)
		{
		case EVENT_COMMIT:
			DiscardPanel(AWGPanelHandle);
			ibonl (0, 0);//Take GPIB board off line
			viClose(InstrHandle);
			viClose(ResourceManager); 
			QuitUserInterface(0);
			break;
		}
	return 0;
}//ExitAWGCB

//Exit when panel is closed
int CVICALLBACK ClosePanel (int panel, int event, void *callbackData,
										int eventData1, int eventData2)
{
	switch (event)
		{
		case EVENT_CLOSE:
			ibonl (0, 0);//Take GPIB board off line
			viClose(InstrHandle); 
			viClose(ResourceManager); 
			QuitUserInterface (0);
			break;
		}
	return 0;
} //ClosePanel

