/***************************************************************************** 
 *  Copyright 1998 National Instruments Corporation.  All Rights Reserved.   * 
 *****************************************************************************/

/*****************************************************************************
 *  Hewlett Packard 33120A Instrument Driver                               
 *  LabWindows/CVI 5.0 Instrument Driver                                     
 *  Original Release: 3/16/98                                  
 *  By: GGB, NI                              
 *      PH. 1-800-433-3488   Fax                               
 *                                                                           
 *  Modification History:                                                    
 *                                                                           
 *       3/16/98 - Instrument Driver Created.                  
 *       
 *       7/22/98 - Added calls to Ivi_UnlockSession to hp33120a_EnableOutput
 *                 and hp33120a_DisableOutput.
 *               - Modified documentation of hp33120a.fp to reflect correct 
 *                 value information.
 *               - Corrected behavior of ClearArbWaveform and
 *                 HP33120A_ATTR_ARB_SAMPLE_RATE attribute to behave correctly
 *                 for simulation mode.
 *****************************************************************************/

#include <string.h>
#include <stdio.h>  
#include <formatio.h>
#include <math.h>
#include "hp33120a.h"

/*****************************************************************************
 *--------------------- Hidden Attribute Declarations -----------------------*
 *****************************************************************************/

#define HP33120A_ATTR_OPC_TIMEOUT            (IVI_SPECIFIC_PRIVATE_ATTR_BASE + 1L)     /* ViInt32 */

#define HP33120A_ATTR_WFM_STRUCTURE          (IVI_SPECIFIC_PRIVATE_ATTR_BASE + 102L)   /* ViAddr */

/*****************************************************************************
 *---------------------------- Useful Macros --------------------------------*
 *****************************************************************************/

    /*- Modulation Modes -------------------------*/
#define HP33120A_VAL_AM                      (1L)
#define HP33120A_VAL_FM                      (2L)
#define HP33120A_VAL_FSK                     (3L)
#define HP33120A_VAL_BURST                   (4L)
#define HP33120A_VAL_SWEEP                   (5L)

    /*- Length of Arbitrary Waveform Names -------*/
#define HP33120A_VAL_NAME_LENGTH             (9L)

    /*- I/O buffer size --------------------------*/
#define BUFFER_SIZE                             512L        

    /*- 488.2 Event Status Register (ESR) Bits ---*/
#define IEEE_488_2_QUERY_ERROR_BIT              0x04
#define IEEE_488_2_DEVICE_DEPENDENT_ERROR_BIT   0x08
#define IEEE_488_2_EXECUTION_ERROR_BIT          0x10
#define IEEE_488_2_COMMAND_ERROR_BIT            0x20

/* The driver uses this structure in its utility functions for creating and */
/* removing arbitray waveforms. */
struct wfmNode
    {
    ViInt32 wfmSize;
    ViChar  wfmName[HP33120A_VAL_NAME_LENGTH];
    };
typedef struct wfmNode wfmNode;

typedef wfmNode *wfmNodePtr;

/*****************************************************************************
 *-------------- Utility Function Declarations (Non-Exported) ---------------*
 *****************************************************************************/
static ViStatus hp33120a_InitAttributes (ViSession vi);
static ViStatus hp33120a_DefaultInstrSetup (ViSession openInstrSession);
static ViStatus hp33120a_CheckStatus (ViSession vi);
static ViStatus hp33120a_WaitForOPC (ViSession vi, ViInt32 maxTime);

static ViStatus hp33120a_VerifyOutputMode   (ViSession vi, ViInt32 outputMode);
static ViStatus hp33120a_GetEquivalentFrequency (ViSession vi, ViConstString channelName,
                         ViReal64 *frequency, ViReal64 *maxFreq);
                         
static ViStatus hp33120a_AnyModulationEnabled (ViSession vi, ViConstString channelName, 
                           ViInt32 modType, ViBoolean *modOn);

static ViStatus hp33120a_CreateDriverWfmRecord (ViSession vi);
static ViStatus hp33120a_VerifyWfmCreatable (ViSession vi, ViInt32 wfmSize, ViReal64 wfmData[]);
static ViStatus hp33120a_GetNewWfmInfo (ViSession vi, ViInt32 *wfmHandle, ViChar wfmName[]);
static ViStatus hp33120a_GetNewWfmName (ViSession vi, wfmNodePtr wfmRecord, ViInt32 maxWfms, ViInt32 wfmHandle, ViChar wfmName[]);
static ViStatus hp33120a_GetNumDefinedWaveforms (ViSession vi, ViInt32 maxNumWfms, ViInt32 *numWfms);
static ViStatus hp33120a_UpdateDriverWfmRecord (ViSession vi, ViInt32 newWfmHandle, ViInt32 wfmSize, ViChar wfmName[]);
static ViStatus hp33120a_WfmExists (ViSession vi, ViInt32 wfmHandle, ViBoolean *wfmExists);
static ViStatus hp33120a_ClearDriverWfmRecord (ViSession vi, ViInt32 wfmHandle);
static ViStatus hp33120a_GetWfmNameFromHandleUtil (ViSession vi, ViInt32 wfmHandle, ViInt32 arraySize, ViChar wfmName[]);
static ViStatus hp33120a_GetWfmHandleFromNameUtil (ViSession vi, ViConstString wfmName, ViInt32 *wfmHandle);

static ViStatus hp33120a_GetWfmSize (ViSession vi, ViInt32 wfmHandle, ViInt32 *wfmSize);
    /*- File I/O Utility Functions -*/
static ViStatus hp33120a_ReadToFile (ViSession vi, ViConstString filename, 
                     ViInt32 maxBytesToRead, ViInt32 fileAction, ViInt32 *totalBytesWritten);
static ViStatus hp33120a_WriteFromFile (ViSession vi, ViConstString filename, 
                    ViInt32 maxBytesToWrite, ViInt32 byteOffset, 
                    ViInt32 *totalBytesWritten);

/*****************************************************************************
 *----------------- Callback Declarations (Non-Exported) --------------------*
 *****************************************************************************/

    /*- Global Session Callbacks --------------------------------------------*/
    
static ViStatus _VI_FUNC hp33120a_CheckStatusCallback (ViSession vi, ViSession io);
static ViStatus _VI_FUNC hp33120a_WaitForOPCCallback (ViSession vi, ViSession io);

    /*- Attribute callbacks -------------------------------------------------*/

static ViStatus _VI_FUNC hp33120aAttrDriverRevision_ReadCallback (ViSession vi,
                                  ViSession io, 
                                  ViConstString channelName,
                                  ViAttr attributeId, 
                                  const ViConstString cacheValue);
static ViStatus _VI_FUNC hp33120aAttrIdQueryResponse_ReadCallback (ViSession vi,
                                   ViSession io, 
                                   ViConstString channelName, 
                                   ViAttr attributeId, 
                                   const ViConstString cacheValue);
static ViStatus _VI_FUNC hp33120aAttrOutputMode_WriteCallback (ViSession vi,
                                   ViSession io,
                                   ViConstString channelName,
                                   ViAttr attributeId,
                                   ViInt32 value);
static ViStatus _VI_FUNC hp33120aAttrOutputEnabled_CheckCallback (ViSession vi,
                                  ViConstString channelName,
                                  ViAttr attributeId,
                                  ViBoolean value);
static ViStatus _VI_FUNC hp33120aAttrOutputImpedance_WriteCallback (ViSession vi,
                                    ViSession io,
                                    ViConstString channelName,
                                    ViAttr attributeId,
                                    ViReal64 value);
static ViStatus _VI_FUNC hp33120aAttrOutputImpedance_ReadCallback (ViSession vi,
                                   ViSession io,
                                   ViConstString channelName,
                                   ViAttr attributeId,
                                   ViReal64 *value);
static ViStatus _VI_FUNC hp33120aAttrFuncWaveform_WriteCallback (ViSession vi,
                                 ViSession io,
                                 ViConstString channelName,
                                 ViAttr attributeId,
                                 ViInt32 value);
static ViStatus _VI_FUNC hp33120aAttrFuncWaveform_ReadCallback (ViSession vi,
                                ViSession io,
                                ViConstString channelName,
                                ViAttr attributeId,
                                ViInt32 *value);
static ViStatus _VI_FUNC hp33120aAttrFuncWaveform_CheckCallback (ViSession vi,
                                 ViConstString channelName,
                                 ViAttr attributeId,
                                 ViInt32 value);
static ViStatus _VI_FUNC hp33120aAttrFuncAmplitude_WriteCallback (ViSession vi,
                                  ViSession io,
                                  ViConstString channelName,
                                  ViAttr attributeId,
                                  ViReal64 value);
static ViStatus _VI_FUNC hp33120aAttrFuncAmplitude_ReadCallback (ViSession vi,
                                 ViSession io,
                                 ViConstString channelName,
                                 ViAttr attributeId,
                                 ViReal64 *value);
static ViStatus _VI_FUNC hp33120aAttrFuncAmplitude_CheckCallback (ViSession vi,
                                  ViConstString channelName,
                                  ViAttr attributeId,
                                  ViReal64 value);
static ViStatus _VI_FUNC hp33120aAttrFuncDCOffset_WriteCallback (ViSession vi,
                                 ViSession io,
                                 ViConstString channelName,
                                 ViAttr attributeId,
                                 ViReal64 value);
static ViStatus _VI_FUNC hp33120aAttrFuncDCOffset_ReadCallback (ViSession vi,
                                ViSession io,
                                ViConstString channelName,
                                ViAttr attributeId,
                                ViReal64 *value);
static ViStatus _VI_FUNC hp33120aAttrFuncDCOffset_CheckCallback (ViSession vi,
                                 ViConstString channelName,
                                 ViAttr attributeId,
                                 ViReal64 value);
static ViStatus _VI_FUNC hp33120aAttrFuncFrequency_WriteCallback (ViSession vi,
                                  ViSession io,
                                  ViConstString channelName,
                                  ViAttr attributeId,
                                  ViReal64 value);
static ViStatus _VI_FUNC hp33120aAttrFuncFrequency_ReadCallback (ViSession vi,
                                 ViSession io,
                                 ViConstString channelName,
                                 ViAttr attributeId,
                                 ViReal64 *value);
static ViStatus _VI_FUNC hp33120aAttrFuncFrequency_CheckCallback (ViSession vi,
                                  ViConstString channelName,
                                  ViAttr attributeId,
                                  ViReal64 value);
static ViStatus _VI_FUNC hp33120aAttrFuncFrequency_RangeTableCallback (ViSession vi,
                                       ViConstString channelName,
                                       ViAttr attributeId,
                                       IviRangeTablePtr *rangeTablePtr);
static ViStatus _VI_FUNC hp33120aAttrFuncDutyCycleHigh_WriteCallback (ViSession vi,
                                      ViSession io,
                                      ViConstString channelName,
                                      ViAttr attributeId,
                                      ViReal64 value);
static ViStatus _VI_FUNC hp33120aAttrFuncDutyCycleHigh_ReadCallback (ViSession vi,
                                     ViSession io,
                                     ViConstString channelName,
                                     ViAttr attributeId,
                                     ViReal64 *value);
static ViStatus _VI_FUNC hp33120aAttrFuncDutyCycleHigh_CheckCallback (ViSession vi,
                                      ViConstString channelName,
                                      ViAttr attributeId,
                                      ViReal64 value);
static ViStatus _VI_FUNC hp33120aAttrArbWaveformHandle_WriteCallback (ViSession vi,
                                      ViSession io,
                                      ViConstString channelName,
                                      ViAttr attributeId,
                                      ViInt32 value);
static ViStatus _VI_FUNC hp33120aAttrArbWaveformHandle_ReadCallback (ViSession vi,
                                     ViSession io,
                                     ViConstString channelName,
                                     ViAttr attributeId,
                                     ViInt32 *value);
static ViStatus _VI_FUNC hp33120aAttrArbWaveformHandle_CheckCallback (ViSession vi,
                                      ViConstString channelName,
                                      ViAttr attributeId,
                                      ViInt32 value);
static ViStatus _VI_FUNC hp33120aAttrArbGain_WriteCallback (ViSession vi,
                                ViSession io,
                                ViConstString channelName,
                                ViAttr attributeId,
                                ViReal64 value);
static ViStatus _VI_FUNC hp33120aAttrArbGain_ReadCallback (ViSession vi,
                               ViSession io,
                               ViConstString channelName,
                               ViAttr attributeId,
                               ViReal64 *value);
static ViStatus _VI_FUNC hp33120aAttrArbGain_CheckCallback (ViSession vi,
                                ViConstString channelName,
                                ViAttr attributeId,
                                ViReal64 value);
static ViStatus _VI_FUNC hp33120aAttrArbOffset_WriteCallback (ViSession vi,
                                  ViSession io,
                                  ViConstString channelName,
                                  ViAttr attributeId,
                                  ViReal64 value);
static ViStatus _VI_FUNC hp33120aAttrArbOffset_ReadCallback (ViSession vi,
                                 ViSession io,
                                 ViConstString channelName,
                                 ViAttr attributeId,
                                 ViReal64 *value);
static ViStatus _VI_FUNC hp33120aAttrArbOffset_CheckCallback (ViSession vi,
                                  ViConstString channelName,
                                  ViAttr attributeId,
                                  ViReal64 value);
static ViStatus _VI_FUNC hp33120aAttrArbSampleRate_WriteCallback (ViSession vi,
                                  ViSession io,
                                  ViConstString channelName,
                                  ViAttr attributeId,
                                  ViReal64 value);
static ViStatus _VI_FUNC hp33120aAttrArbSampleRate_ReadCallback (ViSession vi,
                                 ViSession io,
                                 ViConstString channelName,
                                 ViAttr attributeId,
                                 ViReal64 *value);
static ViStatus _VI_FUNC hp33120aAttrArbSampleRate_CheckCallback (ViSession vi,
                                  ViConstString channelName,
                                  ViAttr attributeId,
                                  ViReal64 value);
static ViStatus _VI_FUNC hp33120aAttrAMEnabled_WriteCallback (ViSession vi,
                                  ViSession io,
                                  ViConstString channelName,
                                  ViAttr attributeId,
                                  ViBoolean value);
static ViStatus _VI_FUNC hp33120aAttrAMEnabled_ReadCallback (ViSession vi,
                                 ViSession io,
                                 ViConstString channelName,
                                 ViAttr attributeId,
                                 ViBoolean *value);
static ViStatus _VI_FUNC hp33120aAttrAMEnabled_CheckCallback (ViSession vi,
                                  ViConstString channelName,
                                  ViAttr attributeId,
                                  ViBoolean value);
static ViStatus _VI_FUNC hp33120aAttrAMSource_WriteCallback (ViSession vi,
                                 ViSession io,
                                 ViConstString channelName,
                                 ViAttr attributeId,
                                 ViInt32 value);
static ViStatus _VI_FUNC hp33120aAttrAMSource_ReadCallback (ViSession vi,
                                ViSession io,
                                ViConstString channelName,
                                ViAttr attributeId,
                                ViInt32 *value);
static ViStatus _VI_FUNC hp33120aAttrAMInternalDepth_WriteCallback (ViSession vi,
                                    ViSession io,
                                    ViConstString channelName,
                                    ViAttr attributeId,
                                    ViReal64 value);
static ViStatus _VI_FUNC hp33120aAttrAMInternalDepth_ReadCallback (ViSession vi,
                                   ViSession io,
                                   ViConstString channelName,
                                   ViAttr attributeId,
                                   ViReal64 *value);
static ViStatus _VI_FUNC hp33120aAttrAMInternalWaveform_WriteCallback (ViSession vi,
                                       ViSession io,
                                       ViConstString channelName,
                                       ViAttr attributeId,
                                       ViInt32 value);
static ViStatus _VI_FUNC hp33120aAttrAMInternalWaveform_ReadCallback (ViSession vi,
                                      ViSession io,
                                      ViConstString channelName,
                                      ViAttr attributeId,
                                      ViInt32 *value);
static ViStatus _VI_FUNC hp33120aAttrAMInternalFrequency_WriteCallback (ViSession vi,
                                    ViSession io,
                                    ViConstString channelName,
                                    ViAttr attributeId,
                                    ViReal64 value);
static ViStatus _VI_FUNC hp33120aAttrAMInternalFrequency_ReadCallback (ViSession vi,
                                       ViSession io,
                                       ViConstString channelName,
                                       ViAttr attributeId,
                                       ViReal64 *value);
static ViStatus _VI_FUNC hp33120aAttrFMEnabled_WriteCallback (ViSession vi,
                                  ViSession io,
                                  ViConstString channelName,
                                  ViAttr attributeId,
                                  ViBoolean value);
static ViStatus _VI_FUNC hp33120aAttrFMEnabled_ReadCallback (ViSession vi,
                                 ViSession io,
                                 ViConstString channelName,
                                 ViAttr attributeId,
                                 ViBoolean *value);
static ViStatus _VI_FUNC hp33120aAttrFMEnabled_CheckCallback (ViSession vi,
                                  ViConstString channelName,
                                  ViAttr attributeId,
                                  ViBoolean value);
static ViStatus _VI_FUNC hp33120aAttrFMInternalDeviation_WriteCallback (ViSession vi,
                                    ViSession io,
                                    ViConstString channelName,
                                    ViAttr attributeId,
                                    ViReal64 value);
static ViStatus _VI_FUNC hp33120aAttrFMInternalDeviation_ReadCallback (ViSession vi,
                                       ViSession io,
                                       ViConstString channelName,
                                       ViAttr attributeId,
                                       ViReal64 *value);
static ViStatus _VI_FUNC hp33120aAttrFMInternalDeviation_CheckCallback (ViSession vi,
                                    ViConstString channelName,
                                    ViAttr attributeId,
                                    ViReal64 value);
static ViStatus _VI_FUNC hp33120aAttrFMInternalWaveform_WriteCallback (ViSession vi,
                                       ViSession io,
                                       ViConstString channelName,
                                       ViAttr attributeId,
                                       ViInt32 value);
static ViStatus _VI_FUNC hp33120aAttrFMInternalWaveform_ReadCallback (ViSession vi,
                                      ViSession io,
                                      ViConstString channelName,
                                      ViAttr attributeId,
                                      ViInt32 *value);
static ViStatus _VI_FUNC hp33120aAttrFMInternalFrequency_WriteCallback (ViSession vi,
                                    ViSession io,
                                    ViConstString channelName,
                                    ViAttr attributeId,
                                    ViReal64 value);
static ViStatus _VI_FUNC hp33120aAttrFMInternalFrequency_ReadCallback (ViSession vi,
                                       ViSession io,
                                       ViConstString channelName,
                                       ViAttr attributeId,
                                       ViReal64 *value);
static ViStatus _VI_FUNC hp33120aAttrFSKEnabled_WriteCallback (ViSession vi,
                                   ViSession io,
                                   ViConstString channelName,
                                   ViAttr attributeId,
                                   ViBoolean value);
static ViStatus _VI_FUNC hp33120aAttrFSKEnabled_ReadCallback (ViSession vi,
                                  ViSession io,
                                  ViConstString channelName,
                                  ViAttr attributeId,
                                  ViBoolean *value);
static ViStatus _VI_FUNC hp33120aAttrFSKEnabled_CheckCallback (ViSession vi,
                                   ViConstString channelName,
                                   ViAttr attributeId,
                                   ViBoolean value);
static ViStatus _VI_FUNC hp33120aAttrFSKSource_WriteCallback (ViSession vi,
                                  ViSession io,
                                  ViConstString channelName,
                                  ViAttr attributeId,
                                  ViInt32 value);
static ViStatus _VI_FUNC hp33120aAttrFSKSource_ReadCallback (ViSession vi,
                                 ViSession io,
                                 ViConstString channelName,
                                 ViAttr attributeId,
                                 ViInt32 *value);
static ViStatus _VI_FUNC hp33120aAttrFSKInternalRate_WriteCallback (ViSession vi,
                                    ViSession io,
                                    ViConstString channelName,
                                    ViAttr attributeId,
                                    ViReal64 value);
static ViStatus _VI_FUNC hp33120aAttrFSKInternalRate_ReadCallback (ViSession vi,
                                   ViSession io,
                                   ViConstString channelName,
                                   ViAttr attributeId,
                                   ViReal64 *value);
static ViStatus _VI_FUNC hp33120aAttrFSKHopFreq_WriteCallback (ViSession vi,
                                   ViSession io,
                                   ViConstString channelName,
                                   ViAttr attributeId,
                                   ViReal64 value);
static ViStatus _VI_FUNC hp33120aAttrFSKHopFreq_ReadCallback (ViSession vi,
                                  ViSession io,
                                  ViConstString channelName,
                                  ViAttr attributeId,
                                  ViReal64 *value);
static ViStatus _VI_FUNC hp33120aAttrFSKHopFreq_CheckCallback (ViSession vi,
                                   ViConstString channelName,
                                   ViAttr attributeId,
                                   ViReal64 value);
static ViStatus _VI_FUNC hp33120aAttrSweepEnabled_WriteCallback (ViSession vi,
                                 ViSession io,
                                 ViConstString channelName,
                                 ViAttr attributeId,
                                 ViBoolean value);
static ViStatus _VI_FUNC hp33120aAttrSweepEnabled_ReadCallback (ViSession vi,
                                ViSession io,
                                ViConstString channelName,
                                ViAttr attributeId,
                                ViBoolean *value);
static ViStatus _VI_FUNC hp33120aAttrSweepEnabled_CheckCallback (ViSession vi,
                                 ViConstString channelName,
                                 ViAttr attributeId,
                                 ViBoolean value);
static ViStatus _VI_FUNC hp33120aAttrSweepStartFreq_WriteCallback (ViSession vi,
                                   ViSession io,
                                   ViConstString channelName,
                                   ViAttr attributeId,
                                   ViReal64 value);
static ViStatus _VI_FUNC hp33120aAttrSweepStartFreq_ReadCallback (ViSession vi,
                                  ViSession io,
                                  ViConstString channelName,
                                  ViAttr attributeId,
                                  ViReal64 *value);
static ViStatus _VI_FUNC hp33120aAttrSweepStartFreq_CheckCallback (ViSession vi,
                                   ViConstString channelName,
                                   ViAttr attributeId,
                                   ViReal64 value);
static ViStatus _VI_FUNC hp33120aAttrSweepStopFreq_WriteCallback (ViSession vi,
                                  ViSession io,
                                  ViConstString channelName,
                                  ViAttr attributeId,
                                  ViReal64 value);
static ViStatus _VI_FUNC hp33120aAttrSweepStopFreq_ReadCallback (ViSession vi,
                                 ViSession io,
                                 ViConstString channelName,
                                 ViAttr attributeId,
                                 ViReal64 *value);
static ViStatus _VI_FUNC hp33120aAttrSweepStopFreq_CheckCallback (ViSession vi,
                                  ViConstString channelName,
                                  ViAttr attributeId,
                                  ViReal64 value);
static ViStatus _VI_FUNC hp33120aAttrSweepSpacing_WriteCallback (ViSession vi,
                                 ViSession io,
                                 ViConstString channelName,
                                 ViAttr attributeId,
                                 ViInt32 value);
static ViStatus _VI_FUNC hp33120aAttrSweepSpacing_ReadCallback (ViSession vi,
                                ViSession io,
                                ViConstString channelName,
                                ViAttr attributeId,
                                ViInt32 *value);
static ViStatus _VI_FUNC hp33120aAttrSweepTime_WriteCallback (ViSession vi,
                                  ViSession io,
                                  ViConstString channelName,
                                  ViAttr attributeId,
                                  ViReal64 value);
static ViStatus _VI_FUNC hp33120aAttrSweepTime_ReadCallback (ViSession vi,
                                 ViSession io,
                                 ViConstString channelName,
                                 ViAttr attributeId,
                                 ViReal64 *value);
static ViStatus _VI_FUNC hp33120aAttrSweepSource_WriteCallback (ViSession vi,
                                ViSession io,
                                ViConstString channelName,
                                ViAttr attributeId,
                                ViInt32 value);
static ViStatus _VI_FUNC hp33120aAttrSweepSource_ReadCallback (ViSession vi,
                                   ViSession io,
                                   ViConstString channelName,
                                   ViAttr attributeId,
                                   ViInt32 *value);
static ViStatus _VI_FUNC hp33120aAttrBurstEnabled_WriteCallback (ViSession vi,
                                 ViSession io,
                                 ViConstString channelName,
                                 ViAttr attributeId,
                                 ViBoolean value);
static ViStatus _VI_FUNC hp33120aAttrBurstEnabled_ReadCallback (ViSession vi,
                                ViSession io,
                                ViConstString channelName,
                                ViAttr attributeId,
                                ViBoolean *value);
static ViStatus _VI_FUNC hp33120aAttrBurstEnabled_CheckCallback (ViSession vi,
                                 ViConstString channelName,
                                 ViAttr attributeId,
                                 ViBoolean value);
static ViStatus _VI_FUNC hp33120aAttrBurstSource_WriteCallback (ViSession vi,
                                ViSession io,
                                ViConstString channelName,
                                ViAttr attributeId,
                                ViInt32 value);
static ViStatus _VI_FUNC hp33120aAttrBurstSource_ReadCallback (ViSession vi,
                                   ViSession io,
                                   ViConstString channelName,
                                   ViAttr attributeId,
                                   ViInt32 *value);
static ViStatus _VI_FUNC hp33120aAttrBurstRate_WriteCallback (ViSession vi,
                                  ViSession io,
                                  ViConstString channelName,
                                  ViAttr attributeId,
                                  ViReal64 value);
static ViStatus _VI_FUNC hp33120aAttrBurstRate_ReadCallback (ViSession vi,
                                 ViSession io,
                                 ViConstString channelName,
                                 ViAttr attributeId,
                                 ViReal64 *value);
static ViStatus _VI_FUNC hp33120aAttrBurstPhase_WriteCallback (ViSession vi,
                                   ViSession io,
                                   ViConstString channelName,
                                   ViAttr attributeId,
                                   ViReal64 value);
static ViStatus _VI_FUNC hp33120aAttrBurstPhase_ReadCallback (ViSession vi,
                                  ViSession io,
                                  ViConstString channelName,
                                  ViAttr attributeId,
                                  ViReal64 *value);
static ViStatus _VI_FUNC hp33120aAttrBurstCount_WriteCallback (ViSession vi,
                                   ViSession io,
                                   ViConstString channelName,
                                   ViAttr attributeId,
                                   ViInt32 value);
static ViStatus _VI_FUNC hp33120aAttrBurstCount_ReadCallback (ViSession vi,
                                  ViSession io,
                                  ViConstString channelName,
                                  ViAttr attributeId,
                                  ViInt32 *value);
static ViStatus _VI_FUNC hp33120aAttrBurstCount_CheckCallback (ViSession vi,
                                   ViConstString channelName,
                                   ViAttr attributeId,
                                   ViInt32 value);

static ViStatus _VI_FUNC hp33120aAttrFuncDutyCycleHigh_RangeTableCallback (ViSession vi,
                                       ViConstString channelName,
                                       ViAttr attributeId,
                                       IviRangeTablePtr *rangeTablePtr);

static ViStatus _VI_FUNC hp33120aAttrOutputMode_CheckCallback (ViSession vi,
                                   ViConstString channelName,
                                   ViAttr attributeId,
                                   ViInt32 value);

/*****************************************************************************
 *------------ User-Callable Functions (Exportable Functions) ---------------*
 *****************************************************************************/

/*****************************************************************************
 * Function: hp33120a_init   
 * Purpose:  VXIplug&play required function.  Calls the   
 *           hp33120a_InitWithOptions function.   
 *****************************************************************************/
ViStatus _VI_FUNC hp33120a_init (ViRsrc resourceName, ViBoolean IDQuery,
                 ViBoolean resetDevice, ViSession *newVi)
{   
    ViStatus    error = VI_SUCCESS;

    if (newVi == VI_NULL)
    {
    Ivi_SetErrorInfo (VI_NULL, VI_FALSE, IVI_ERROR_INVALID_PARAMETER, 
              VI_ERROR_PARAMETER4, "Null address for Instrument Handle");
    checkErr( IVI_ERROR_INVALID_PARAMETER);
    }

    checkErr( hp33120a_InitWithOptions (resourceName, IDQuery, resetDevice, 
                    "", newVi));

Error:  
    return error;
}

/*****************************************************************************
 * Function: hp33120a_InitWithOptions                                       
 * Purpose:  This function creates a new IVI session and calls the 
 *           IviInit function.                                     
 *****************************************************************************/
ViStatus _VI_FUNC hp33120a_InitWithOptions (ViRsrc resourceName, ViBoolean IDQuery,
                        ViBoolean resetDevice, ViString optionString, 
                        ViSession *newVi)
{   
    ViStatus    error = VI_SUCCESS;
    ViSession   vi = VI_NULL;
    
    if (newVi == VI_NULL)
    {
    Ivi_SetErrorInfo (VI_NULL, VI_FALSE, IVI_ERROR_INVALID_PARAMETER, 
              VI_ERROR_PARAMETER5, "Null address for Instrument Handle");
    checkErr( IVI_ERROR_INVALID_PARAMETER);
    }

    *newVi = VI_NULL;
    
    /* create a new interchangable driver */
    checkErr( Ivi_SpecificDriverNew ("hp33120a", optionString, &vi));  
    
    /* init the driver */
    checkErr( hp33120a_IviInit (resourceName, IDQuery, resetDevice, vi)); 
    *newVi = vi;
    
Error:
    
    if (error < VI_SUCCESS) 
    Ivi_Dispose (vi);
    
    return error;
}

/*****************************************************************************
 * Function: hp33120a_IviInit                                                       
 * Purpose:  This function is called by hp33120a_InitWithOptions  
 *           or by an IVI class driver.  This function initializes the I/O 
 *           interface, optionally resets the device, optionally performs an
 *           ID query, and sends a default setup to the instrument.                
 *****************************************************************************/
ViStatus _VI_FUNC hp33120a_IviInit (ViRsrc resourceName, ViBoolean IDQuery,
                    ViBoolean reset, ViSession vi)
{
    ViStatus    error = VI_SUCCESS;
    ViSession   io = VI_NULL;
    
    checkErr( Ivi_BuildChannelTable (vi, HP33120A_CHANNEL_LIST, VI_FALSE, VI_NULL));
    
     /* Add attributes */
    checkErr( hp33120a_InitAttributes (vi));

    if (!Ivi_Simulating(vi))    /* called only when session is locked */
    {
    ViSession   rmSession = VI_NULL;

    /* Open instrument session */
    checkErr( Ivi_GetAttributeViSession (vi, VI_NULL, IVI_ATTR_VISA_RM_SESSION, 0,
                         &rmSession)); 
    viCheckErr( viOpen (rmSession, resourceName, VI_NULL, VI_NULL, &io));
    /* io session owned by driver now */
    checkErr( Ivi_SetAttributeViSession (vi, VI_NULL, IVI_ATTR_IO_SESSION, 0, io));  

    /* Configure VISA Formatted I/O */
    viCheckErr( viSetAttribute (io, VI_ATTR_TMO_VALUE, 5000 ));
    viCheckErr( viSetBuf (io, VI_READ_BUF, 4000));
    viCheckErr( viSetBuf (io, VI_WRITE_BUF, 32100));
    viCheckErr( viSetAttribute (io, VI_ATTR_WR_BUF_OPER_MODE, VI_FLUSH_ON_ACCESS));
    viCheckErr( viSetAttribute (io, VI_ATTR_RD_BUF_OPER_MODE, VI_FLUSH_ON_ACCESS));
    }
    
    /*- Reset instrument ----------------------------------------------------*/
    if (reset) 
    checkErr( hp33120a_reset (vi));
    else  /*- Send Default Instrument Setup ---------------------------------*/
    checkErr( hp33120a_DefaultInstrSetup (vi));
               
    /* Identification Query */
    if (IDQuery)                               
    {
    ViChar rdBuffer[BUFFER_SIZE];
    
    #define VALID_RESPONSE_STRING_START        "HEWLETT-PACKARD,33120A"
    
    checkErr( Ivi_GetAttributeViString (vi, VI_NULL, HP33120A_ATTR_ID_QUERY_RESPONSE, 
                        0, BUFFER_SIZE, rdBuffer));
    
        
    if (strncmp (rdBuffer, VALID_RESPONSE_STRING_START, 
             strlen(VALID_RESPONSE_STRING_START)) != 0)
        {             
        viCheckErr( VI_ERROR_FAIL_ID_QUERY);
        }
    }

    checkErr( hp33120a_CheckStatus (vi));

Error:
    if (error < VI_SUCCESS)
    {
    if (!Ivi_Simulating (vi) && io)
        viClose (io);
    }
    return error;
}

/*****************************************************************************
 * Function: hp33120a_close                                                           
 * Purpose:  This function closes the instrument.                            
 *
 *           Note:  This function must unlock the session before calling
 *           Ivi_Dispose.
 *****************************************************************************/
ViStatus _VI_FUNC hp33120a_close (ViSession vi)
{
    ViStatus    error = VI_SUCCESS;
    
    
    checkErr( Ivi_LockSession (vi, VI_NULL));
    
    checkErr( hp33120a_IviClose (vi));

Error:    
    Ivi_UnlockSession (vi, VI_NULL);
    Ivi_Dispose (vi);  

    return error;
}

/*****************************************************************************
 * Function: hp33120a_IviClose                                                        
 * Purpose:  This function performs all of the drivers clean-up operations   
 *           except for closing the IVI session.  This function is called by 
 *           hp33120a_close or by an IVI class driver. 
 *
 *           Note:  This function must close the I/O session and set 
 *           IVI_ATTR_IO_SESSION to 0.
 *****************************************************************************/
ViStatus _VI_FUNC hp33120a_IviClose (ViSession vi)
{
    ViStatus error = VI_SUCCESS;
    ViSession io = VI_NULL;

    /* Do not lock here.  The caller manages the lock. */

    checkErr( Ivi_GetAttributeViSession (vi, VI_NULL, IVI_ATTR_IO_SESSION, 0, &io));

Error:
    Ivi_SetAttributeViSession (vi, VI_NULL, IVI_ATTR_IO_SESSION, 0, VI_NULL);
    if(io)                                                      
    {
    viClose (io);
    }
    return error;   
}


/*****************************************************************************
 * Function: hp33120a_reset                                                         
 * Purpose:  This function resets the instrument.                          
 *****************************************************************************/
ViStatus _VI_FUNC hp33120a_reset (ViSession vi)
{
    ViStatus    error = VI_SUCCESS;

    checkErr( Ivi_LockSession (vi, VI_NULL));
    
    if (!Ivi_Simulating(vi))                /* call only when locked */
    {
    ViSession   io = Ivi_IOSession(vi); /* call only when locked */
    
    checkErr( Ivi_SetNeedToCheckStatus (vi, VI_TRUE));
    viCheckErr( viPrintf (io, "*RST"));
    }
    
    checkErr( hp33120a_DefaultInstrSetup (vi)); 
    
    checkErr( hp33120a_CheckStatus (vi));

Error:
    Ivi_UnlockSession (vi, VI_NULL);
    return error;
}


/*****************************************************************************
 * Function: hp33120a_self_test                                                       
 * Purpose:  This function executes the instrument self-test and returns the 
 *           result.                                                         
 *****************************************************************************/
ViStatus _VI_FUNC hp33120a_self_test (ViSession vi, ViInt16 *testResult, 
                      ViChar testMessage[])
{
    ViStatus    error = VI_SUCCESS;

    checkErr( Ivi_LockSession (vi, VI_NULL));

    if (testResult == VI_NULL)
    viCheckParm( IVI_ERROR_INVALID_PARAMETER, 2, "Null address for Test Result");
    if (testMessage == VI_NULL)
    viCheckParm( IVI_ERROR_INVALID_PARAMETER, 3, "Null address for Test Message");

    if (!Ivi_Simulating(vi))                /* call only when locked */
    {
    ViSession   io = Ivi_IOSession(vi); /* call only when locked */
    
    checkErr( Ivi_SetNeedToCheckStatus (vi, VI_TRUE));
    viCheckErr( viPrintf (io, "*TST?"));
        
    viCheckErr( viScanf (io, "%hd,\"%256[^\"]", testResult, testMessage));
    
    }
    else if (Ivi_UseSpecificSimulation(vi)) /* call only when locked */
    {
        /* Simulate Self Test */
    *testResult = 0;
    strcpy (testMessage, "No error.");
    }
    
    checkErr( hp33120a_CheckStatus (vi));

Error:
    Ivi_UnlockSession(vi, VI_NULL);
    return error;
}

/*****************************************************************************
 * Function: hp33120a_error_query                                                     
 * Purpose:  This function queries the instrument error queue and returns   
 *           the result.                                                     
 *****************************************************************************/
ViStatus _VI_FUNC hp33120a_error_query (ViSession vi, ViInt32 *errCode, 
                    ViChar errMessage[])
{
    ViStatus    error = VI_SUCCESS;
    
    checkErr( Ivi_LockSession (vi, VI_NULL));
    
    if (errCode == VI_NULL)
    viCheckParm( IVI_ERROR_INVALID_PARAMETER, 2, "Null address for Error Code");
    if (errMessage == VI_NULL)
    viCheckParm( IVI_ERROR_INVALID_PARAMETER, 3, "Null address for Error Message");

    if (!Ivi_Simulating(vi))                /* call only when locked */
    {
    ViSession   io = Ivi_IOSession(vi); /* call only when locked */
    
    checkErr( Ivi_SetNeedToCheckStatus (vi, VI_TRUE));
    viCheckErr( viPrintf(io, ":SYST:ERR?"));
    
    viCheckErr( viScanf (io, "%ld,\"%256[^\"]", errCode, errMessage));
    
    }
    else if (Ivi_UseSpecificSimulation(vi)) /* call only when locked */
    {
        /* Simulate Error Query */
    *errCode = 0;
    strcpy (errMessage, "No error.");
    }
    

Error:
    Ivi_UnlockSession(vi, VI_NULL);
    return error;
}

/*****************************************************************************
 * Function: hp33120a_error_message                                                  
 * Purpose:  This function translates the error codes returned by this       
 *           instrument driver into user-readable strings.  
 *
 *           Note:  The caller can pass VI_NULL for the vi parameter.  This 
 *           is usefull if one of the init functions fail.
 *****************************************************************************/
ViStatus _VI_FUNC hp33120a_error_message (ViSession vi, ViStatus errorCode,
                      ViChar errorMessage[256])
{
    ViStatus    error = VI_SUCCESS;
    
    static      IviStringValueTable errorTable = 
    {
        IVIFGEN_ERROR_CODES_AND_MSGS,
        {VI_NULL,                               VI_NULL}
    };
    
    if (vi)
    Ivi_LockSession(vi, VI_NULL);

    /* all VISA and IVI error codes are handled as well as codes in the table */
    if (errorMessage == VI_NULL)
    viCheckParm( IVI_ERROR_INVALID_PARAMETER, 3, "Null address for Error Message");

    checkErr( Ivi_GetSpecificDriverStatusDesc(vi, errorCode, errorMessage, errorTable));

Error:
    if (vi)
    Ivi_UnlockSession(vi, VI_NULL);
    return error;
}

/*****************************************************************************
 * Function: hp33120a_revision_query                                                  
 * Purpose:  This function returns the driver and instrument revisions.      
 *****************************************************************************/
ViStatus _VI_FUNC hp33120a_revision_query (ViSession vi, ViChar driverRev[], 
                       ViChar instrRev[])
{
    ViChar      rdBuffer[BUFFER_SIZE];
    ViStatus    error = VI_SUCCESS;
    
    checkErr( Ivi_LockSession (vi, VI_NULL));

    if (driverRev == VI_NULL)
    viCheckParm( IVI_ERROR_INVALID_PARAMETER, 2, "Null address for Driver Revision");
    if (instrRev == VI_NULL)
    viCheckParm( IVI_ERROR_INVALID_PARAMETER, 3, "Null address for Instrument Revision");

    checkErr( Ivi_GetAttributeViString (vi, VI_NULL, IVI_ATTR_DRIVER_REVISION, 
                    0, -1, driverRev));

    if (!Ivi_Simulating(vi))                /* call only when locked */
    {
    ViSession   io = Ivi_IOSession(vi); /* call only when locked */
    
    checkErr( Ivi_SetNeedToCheckStatus (vi, VI_TRUE));
    viCheckErr( viPrintf (io, "*IDN?"));
    
    viCheckErr( viScanf (io, "%*[^,],%*[^,],%*[^,],%256[^\n]", instrRev));
    }
    else if (Ivi_UseSpecificSimulation(vi)) /* call only when locked */
    {
        /* Simulate Instrument Revision Query */
    strcpy (instrRev, "No revision information available while simulating.");
    }

    checkErr( hp33120a_CheckStatus (vi));

Error:    
    Ivi_UnlockSession(vi, VI_NULL);
    return error;
}
/*****************************************************************************
 * Function: hp33120a_ConfigureOutputMode
 * Purpose: This function configures the output mode of the function
 *          generator.  The output mode determines how the function
 *          generator produces waveforms.  For example, you can select to
 *          output a standard waveform, an arbitrary waveform, or a sequence
 *          of arbitrary waveforms.
 *****************************************************************************/
ViStatus _VI_FUNC hp33120a_ConfigureOutputMode (ViSession vi, ViInt32 outputMode)
{
    ViStatus   error = VI_SUCCESS;
    
    checkErr( Ivi_LockSession (vi, VI_NULL));
    
    /* Set attribute: */
    viCheckParm( Ivi_SetAttributeViInt32 (vi, VI_NULL, HP33120A_ATTR_OUTPUT_MODE,
                      IVI_VAL_DIRECT_USER_CALL, outputMode),
                      2, "Output Mode");
                      
    checkErr( hp33120a_CheckStatus (vi));
    
Error:
    Ivi_UnlockSession (vi, VI_NULL);
    return error;
}

/*****************************************************************************
 * Function: hp33120a_ConfigureOutputImpedance
 * Purpose:  This function configures the output impedance for the channel
 *           you specify.
 *****************************************************************************/
ViStatus _VI_FUNC hp33120a_ConfigureOutputImpedance (ViSession vi, ViConstString channelName,
                           ViReal64 outputImpedance)
{
    ViStatus   error = VI_SUCCESS;
    
    checkErr( Ivi_LockSession (vi, VI_NULL));
    
    /* Set attribute: */
    viCheckParm( Ivi_SetAttributeViReal64 (vi, channelName, HP33120A_ATTR_OUTPUT_IMPEDANCE,
                       IVI_VAL_DIRECT_USER_CALL, outputImpedance),
                       3, "Output Impedance");
                      
    checkErr( hp33120a_CheckStatus (vi));
    
Error:
    Ivi_UnlockSession (vi, VI_NULL);
    return error;
}
    
/*****************************************************************************
 * Function: hp33120a_ConfigureRefClockSource
 * Purpose: This function configures the function generator's reference
 *          clock source.  The function generator uses the reference clock
 *          to derive frequencies and sample rates when generating output.
 *****************************************************************************/
ViStatus _VI_FUNC hp33120a_ConfigureRefClockSource (ViSession vi, ViInt32 refClockSource)
{
    ViStatus error = VI_SUCCESS;
    
    checkErr( Ivi_LockSession (vi, VI_NULL));
    
    /* Set attribute: */
    viCheckParm( Ivi_SetAttributeViInt32 (vi, VI_NULL,
                      HP33120A_ATTR_REF_CLOCK_SOURCE,
                      IVI_VAL_DIRECT_USER_CALL,
                      refClockSource),
                      2, "Reference Clock Source");

    checkErr( hp33120a_CheckStatus (vi));
    
Error:
    Ivi_UnlockSession (vi, VI_NULL);
    return error;
}

/*****************************************************************************
 * Function: hp33120a_EnableOutput
 * Purpose:  This function configures the function generator so that the
 *           signal the function generator produces appears at the channel's
 *           output connector.  
 *****************************************************************************/
ViStatus _VI_FUNC  hp33120a_EnableOutput (ViSession vi, ViConstString channelName)
{
    ViStatus error = VI_SUCCESS;
    
    checkErr( Ivi_LockSession (vi, VI_NULL));
    
    /* Set attribute: */
    checkErr( Ivi_SetAttributeViBoolean (vi, channelName,
                     HP33120A_ATTR_OUTPUT_ENABLED,
                     IVI_VAL_DIRECT_USER_CALL, VI_TRUE));
                     
Error:
    Ivi_UnlockSession (vi, VI_NULL);
    return error;
}

/*****************************************************************************
 * Function: hp33120a_DisableOutput
 * Purpose: This function configures the function generator so that the
 *          signal the function generator produces does not appear at the
 *          channel's output connector.
 *****************************************************************************/
ViStatus _VI_FUNC  hp33120a_DisableOutput (ViSession vi, ViConstString channelName)
{
    ViStatus error = VI_SUCCESS;
    
    checkErr( Ivi_LockSession (vi, VI_NULL));
    
    /* Set attribute: */
    checkErr( Ivi_SetAttributeViBoolean (vi, channelName,
                     HP33120A_ATTR_OUTPUT_ENABLED,
                     IVI_VAL_DIRECT_USER_CALL, VI_FALSE));
                     
Error:
    Ivi_UnlockSession (vi, VI_NULL);
    return error;
}

/*****************************************************************************
 * Function: hp33120a_InitiateGeneration
 * Purpose: This function initiates signal generation.  This causes the
 *          function generator to leave its Configuration state and enter
 *          its Signal Generation state.  
 *****************************************************************************/
ViStatus _VI_FUNC  hp33120a_InitiateGeneration (ViSession vi)
{
    return VI_SUCCESS;
}
    
/*****************************************************************************
 * Function: hp33120a_AbortGeneration
 * Purpose:  This function aborts a previously initiated output generation.
 *           After you call this function, the function generator leaves its
 *           Signal Generation state and returns to its Configuration state.
 *****************************************************************************/
ViStatus _VI_FUNC  hp33120a_AbortGeneration (ViSession vi)
{
    return VI_SUCCESS;
}

/*****************************************************************************
 * Function: hp33120a_ConfigureStandardWaveform
 * Purpose:  This function configures the attributes of the function
 *           generator that affect standard waveform generation.  These
 *           attributes are the waveform, amplitude, DC offset, frequency,
 *           and start phase.
 *****************************************************************************/
ViStatus _VI_FUNC hp33120a_ConfigureStandardWaveform (ViSession vi,
                              ViConstString channelName,
                              ViInt32 waveform,
                              ViReal64 amplitude,
                              ViReal64 dcOffset,
                              ViReal64 frequency,
                              ViReal64 startPhase)
{
    ViStatus error = VI_SUCCESS;
    
    checkErr( Ivi_LockSession (vi, VI_NULL));
    
    /* Set attributes: */
    viCheckParm( Ivi_SetAttributeViInt32 (vi, channelName,
                     HP33120A_ATTR_FUNC_WAVEFORM, 0,
                     waveform), 3, "Standard Waveform");
    viCheckParm( Ivi_SetAttributeViReal64 (vi, channelName,
                       HP33120A_ATTR_FUNC_AMPLITUDE, 0,
                       amplitude), 4, "Amplitude");
    viCheckParm( Ivi_SetAttributeViReal64 (vi, channelName,
                       HP33120A_ATTR_FUNC_DC_OFFSET, 0,
                       dcOffset), 5, "DC Offset");
    viCheckParm( Ivi_SetAttributeViReal64 (vi, channelName,
                       HP33120A_ATTR_FUNC_FREQUENCY, 0,
                       frequency), 6, "Frequency");
    viCheckParm( Ivi_SetAttributeViReal64 (vi, channelName,
                       HP33120A_ATTR_FUNC_START_PHASE, 0,
                       startPhase), 7, "Start Phase");

    checkErr( hp33120a_CheckStatus (vi));
    
Error:
    Ivi_UnlockSession (vi, VI_NULL);
    return error;
}

/*****************************************************************************
 * Function: hp33120a_GenerateStandardWaveform
 * Purpose:  This application function configures the attributes of the 
 *           function generator that affect standard waveform generation,
 *           enables an output channel, and initiates signal generation.  This
 *           behavior is identical to calling the
 *           hp33120a_ConfigureStandardWaveform function.
 *****************************************************************************/
ViStatus _VI_FUNC  hp33120a_GenerateStandardWaveform (ViSession vi,
                              ViConstString channelName,
                              ViInt32 waveform,
                              ViReal64 amplitude,
                              ViReal64 dcOffset,
                              ViReal64 frequency,
                              ViReal64 startPhase)
{
    ViStatus error = VI_SUCCESS;
    
    checkErr( Ivi_LockSession (vi, VI_NULL));

    /* Call functions */
    checkErr( hp33120a_ConfigureStandardWaveform (vi, channelName, waveform,
                          amplitude, dcOffset, frequency,
                          startPhase));
Error:
    Ivi_UnlockSession (vi, VI_NULL);
    return error;
}

/*****************************************************************************
 * Function: hp33120a_QueryArbWfmCapabilities
 * Purpose:  This function returns the attributes of the function generator
 *           that are related to creating arbitrary waveforms.  These
 *           attributes are the maximum number of waveforms, waveform
 *           quantum, minimum waveform size, and maximum waveform size.
 *****************************************************************************/
ViStatus _VI_FUNC  hp33120a_QueryArbWfmCapabilities (ViSession vi,
                          ViInt32 *maxNumWfms,
                          ViInt32 *wfmQuantum,
                          ViInt32 *minWfmSize,
                          ViInt32 *maxWfmSize) 
{
    ViStatus error = VI_SUCCESS;
    
    checkErr( Ivi_LockSession (vi, VI_NULL));
    
    /* Get attributes */
    if (maxNumWfms)
    viCheckParm( Ivi_GetAttributeViInt32 (vi, VI_NULL,
                         HP33120A_ATTR_MAX_NUM_WAVEFORMS,
                         0, maxNumWfms), 2,
                         "Max Number of Waveforms");
    if (wfmQuantum)
    viCheckParm( Ivi_GetAttributeViInt32 (vi, VI_NULL,
                         HP33120A_ATTR_WAVEFORM_QUANTUM,
                         0, wfmQuantum), 3, "Waveform Quantum");
    if (minWfmSize)
    viCheckParm( Ivi_GetAttributeViInt32 (vi, VI_NULL,
                         HP33120A_ATTR_MIN_WAVEFORM_SIZE,
                         0, minWfmSize), 4, "Min Waveform Size");
    if (maxWfmSize)
    viCheckParm( Ivi_GetAttributeViInt32 (vi, VI_NULL,
                         HP33120A_ATTR_MAX_WAVEFORM_SIZE,
                         0, maxWfmSize), 5, "Max Waveform Size");

    checkErr( hp33120a_CheckStatus (vi));
    
Error:
    Ivi_UnlockSession (vi, VI_NULL);
    return error;
}

/*****************************************************************************
 * Function: hp33120a_CreateArbWaveform
 * Purpose:  This function creates an arbitrary waveform and returns a handle
 *           that identifies that waveform.  You use the handles this
 *           function returns to specify a sequence of arbitrary waveforms
 *           with the hp33120a_CreateArbSequence function.
 *****************************************************************************/
ViStatus _VI_FUNC hp33120a_CreateArbWaveform (ViSession vi, ViInt32 wfmSize,
                         ViReal64 wfmData[],
                         ViInt32 *wfmHandle)
{
    ViStatus error = VI_SUCCESS;
    ViInt32  newWfmHandle = VI_NULL;
    ViChar   newWfmName[HP33120A_VAL_NAME_LENGTH];
    ViChar  *writBuff = VI_NULL;
    ViInt16 *binData = VI_NULL;
    
    checkErr( Ivi_LockSession (vi, VI_NULL));
    
    if (!wfmData)
    viCheckParm( IVI_ERROR_INVALID_PARAMETER, 3, "NULL address for wfmData");
    if (!wfmHandle)
    viCheckParm( IVI_ERROR_INVALID_PARAMETER, 4, "NULL address for wfmHandle");
    
    checkErr( hp33120a_VerifyOutputMode (vi, HP33120A_VAL_OUTPUT_ARB));
    checkErr( hp33120a_VerifyWfmCreatable(vi, wfmSize, wfmData)); 
    checkErr( hp33120a_GetNewWfmInfo (vi, &newWfmHandle, newWfmName)); 
       
    if (!Ivi_Simulating(vi))                /* call only when locked */
    {
    ViSession   io = Ivi_IOSession(vi); /* call only when locked */
    ViInt32     index, x;
    checkErr( Ivi_SetNeedToCheckStatus (vi, VI_TRUE));
    
    checkErr( Ivi_Alloc (vi, (sizeof(ViInt16)*wfmSize), (ViAddr *)(&binData)));
    checkErr( Ivi_Alloc (vi, (sizeof(ViChar)*((wfmSize*2) + 28)), (ViAddr *)(&writBuff)));
    /* convert to range -2047 to 2047 */
    for (index = 0; index < wfmSize; index++)
        {
        binData[index] = (ViInt16)(floor (2047 * wfmData[index]));
        }
    /* put data in Big-Endian notation */
    Fmt (binData, "%*d[b2o10]<%*d[b2]",wfmSize,wfmSize,binData);
    /* build output buffer to write to instrument */
    x = sprintf(writBuff, ":DATA:DAC VOLATILE, #5%05d", (2*wfmSize));
    memcpy((writBuff + x), binData, (2*wfmSize));
    viCheckErr( viBufWrite (io, writBuff, (x + 2*wfmSize), VI_NULL));
    viCheckErr( viPrintf(io, ":DATA:COPY %s", newWfmName));
    
    }
    
    checkErr( hp33120a_CheckStatus (vi));
    checkErr( hp33120a_UpdateDriverWfmRecord (vi, newWfmHandle, wfmSize, newWfmName));
    
Error:
    if (binData)
    Ivi_Free (vi, binData);
    *wfmHandle = ((error < VI_SUCCESS) ? VI_NULL : newWfmHandle);
    Ivi_UnlockSession (vi, VI_NULL);
    return error;
}

/*****************************************************************************
 * Function: hp33120a_ConfigureSampleRate
 * Purpose:  This function configures the sample rate attribute, which
 *           determines the rate at which the function generator produces
 *           arbitrary waveforms.
 *****************************************************************************/
ViStatus _VI_FUNC hp33120a_ConfigureSampleRate (ViSession vi, ViReal64 sampleRate)
{
    ViStatus error = VI_SUCCESS;
    
    checkErr( Ivi_LockSession (vi, VI_NULL));
    
    /* Set attribute */
    viCheckParm( Ivi_SetAttributeViReal64 (vi, VI_NULL,
                       HP33120A_ATTR_ARB_SAMPLE_RATE,
                       0, sampleRate), 4, "Sample Rate");
    
    checkErr( hp33120a_CheckStatus (vi));
    
Error:
    Ivi_UnlockSession (vi, VI_NULL);
    return error;
}

/*****************************************************************************
 * Function: hp33120a_ConfigureArbWaveform
 * Purpose:  This function configures the attributes of the function
 *           generator that affect arbitrary waveform generation.  These
 *           attributes are the arbitrary waveform handle, gain, and offset.
 *****************************************************************************/
ViStatus _VI_FUNC  hp33120a_ConfigureArbWaveform (ViSession vi,
                          ViConstString channelName,
                          ViInt32 wfmHandle,
                          ViReal64 arbGain,
                          ViReal64 arbOffset)
{
    ViStatus error = VI_SUCCESS;
    
    checkErr( Ivi_LockSession (vi, VI_NULL));
    
    /* Set attributes */
    viCheckParm( Ivi_SetAttributeViInt32 (vi, channelName,
                      HP33120A_ATTR_ARB_WAVEFORM_HANDLE,
                      0, wfmHandle), 3, "Waveform Handle");
    viCheckParm( Ivi_SetAttributeViReal64 (vi, channelName,
                       HP33120A_ATTR_ARB_GAIN,
                       0, arbGain), 4, "Arbitrary Gain");
    viCheckParm( Ivi_SetAttributeViReal64 (vi, channelName, 
                       HP33120A_ATTR_ARB_OFFSET,
                       0, arbOffset), 5, "Arbitrary Offset");

    checkErr( hp33120a_CheckStatus (vi));
    
Error:
    Ivi_UnlockSession (vi, VI_NULL);
    return error;
}

/*****************************************************************************
 * Function: hp33120a_GenerateArbWaveform
 * Purpose: This function configures the attributes of the function generator
 *          that affect arbitrary waveform generation, enables an output
 *          channel, and initiates signal generation.  This
 *          behavior is identical to calling the
 *          hp33120a_ConfigureArbitraryWaveform function.
 *****************************************************************************/
ViStatus _VI_FUNC  hp33120a_GenerateArbWaveform (ViSession vi,
                         ViConstString channelName,
                         ViInt32 wfmHandle,
                         ViReal64 arbGain,
                         ViReal64 arbOffset)
{
    ViStatus error = VI_SUCCESS;
    ViBoolean outputActive = VI_FALSE;
    
    checkErr( Ivi_LockSession (vi, VI_NULL));
    
    /* Call functions */
    checkErr( hp33120a_ConfigureArbWaveform (vi, channelName, wfmHandle,
                         arbGain, arbOffset));

Error:
    Ivi_UnlockSession (vi, VI_NULL);
    return error;
}
    
/*****************************************************************************
 * Function: hp33120a_ClearArbWaveform
 * Purpose:  This function removes a previously created arbitrary waveform
 *           from the function generator's memory and invalidates the
 *           waveform's handle.  If the waveform handle's values is
 *           HP33120A_VAL_ALL_WAVEFORMS, this function 
 *           removes all user-specified arbitrary waveforms from memory.
 *****************************************************************************/
ViStatus _VI_FUNC  hp33120a_ClearArbWaveform (ViSession vi, ViInt32 wfmHandle)
{
    ViStatus  error = VI_SUCCESS;
    ViBoolean wfmExists;
    ViInt32   currentHandle;
    ViChar    wfmName[HP33120A_VAL_NAME_LENGTH];
    checkErr( Ivi_LockSession (vi, VI_NULL));
    
    checkErr( hp33120a_VerifyOutputMode (vi, HP33120A_VAL_OUTPUT_ARB));
    
    if (wfmHandle != HP33120A_VAL_ALL_WAVEFORMS)
    {
    checkErr( hp33120a_WfmExists (vi, wfmHandle, &wfmExists));
    if (!wfmExists)
        viCheckErrElab( HP33120A_ERROR_INVALID_WAVEFORM,
                "The waveform does not exist.");
    if (wfmHandle < (HP33120A_VAL_FIRST_WAVEFORM_HANDLE + 5) && (!Ivi_Simulating(vi)))
        viCheckErrElab( HP33120A_ERROR_INVALID_WAVEFORM,
                "The waveform is factory-installed and cannot be cleared.");
        
    checkErr( Ivi_GetAttributeViInt32 (vi, "1", HP33120A_ATTR_ARB_WAVEFORM_HANDLE, 
                       0, &currentHandle));
    if (currentHandle == wfmHandle)
        viCheckErrElab( HP33120A_ERROR_WFM_IN_USE,
                "Cannot clear waveform because it is configured for output");
        
    if (!Ivi_Simulating(vi))                /* call only when locked */
        {
        ViSession   io = Ivi_IOSession(vi); /* call only when locked */
        
        checkErr( Ivi_SetNeedToCheckStatus (vi, VI_TRUE));
        checkErr( hp33120a_GetWfmNameFromHandleUtil (vi, wfmHandle, 
                             HP33120A_VAL_NAME_LENGTH, wfmName));
        viCheckErr( viPrintf (io, "DATA:DEL %s;", wfmName));
        }
    }
    else
    {
    if (!Ivi_Simulating(vi))                /* call only when locked */
        {
        ViSession   io = Ivi_IOSession(vi); /* call only when locked */
        
        checkErr( Ivi_SetNeedToCheckStatus (vi, VI_TRUE));
        viCheckErr( viPrintf (io, "FUNC:USER EXP_RISE;:DATA:DEL:ALL;"));
        checkErr( Ivi_InvalidateAttribute (vi, IVI_VAL_ALL_CHANNELS,
              HP33120A_ATTR_ARB_WAVEFORM_HANDLE));
        }
    }
    
    checkErr( hp33120a_CheckStatus (vi));
    checkErr( hp33120a_ClearDriverWfmRecord (vi, wfmHandle));
    
Error:
    Ivi_UnlockSession (vi, VI_NULL);
    return error;
}


/*****************************************************************************
 * Function: hp33120a_GetWfmNameFromHandle
 * Purpose:  This function returns a waveform's name given its handle.
 *****************************************************************************/
ViStatus _VI_FUNC hp33120a_GetWfmNameFromHandle (ViSession vi, ViInt32 wfmHandle,
                         ViInt32 arraySize, ViChar wfmName[])
{
    ViStatus error;
    
    checkErr( Ivi_LockSession (vi, VI_NULL));
    
    if (arraySize < 1)
    viCheckParm (IVI_ERROR_INVALID_PARAMETER, 3, "Invalid Array Size");
    if (!wfmName)
    viCheckParm (IVI_ERROR_INVALID_PARAMETER, 4, "NULL value for wfmName");     
    
    error = (hp33120a_GetWfmNameFromHandleUtil (vi, wfmHandle,
                        arraySize, wfmName));

Error:
    Ivi_UnlockSession (vi, VI_NULL);
    return error;
}

/*****************************************************************************
 * Function: hp33120a_GetWfmHandleFromName
 * Purpose:  This function returns a waveform's handle given its name.
 *****************************************************************************/
ViStatus _VI_FUNC hp33120a_GetWfmHandleFromName (ViSession vi, ViConstString wfmName,
                         ViInt32 *wfmHandle)
{
    ViStatus error = VI_SUCCESS;
    
    checkErr( Ivi_LockSession (vi, VI_NULL));
    
    if (!wfmName)
    viCheckParm (IVI_ERROR_INVALID_PARAMETER, 2, "NULL value for wfmName");     
    if (!wfmHandle)
    viCheckParm (IVI_ERROR_INVALID_PARAMETER, 3, "NULL value for wfmHandle");

    checkErr (hp33120a_GetWfmHandleFromNameUtil (vi, wfmName, wfmHandle));

Error:
    Ivi_UnlockSession (vi, VI_NULL);
    return error;
}

/*****************************************************************************
 * Function: hp33120a_GetWfmNames
 * Purpose:  This function returns a comma delimited list of the names of all
 *           waveforms in the function generator's memory.  You can use this
 *           list to query the driver for the handles of individual arbitrary
 *           waveforms.
 *****************************************************************************/
ViStatus _VI_FUNC hp33120a_GetWfmNames (ViSession vi, ViInt32 arraySize, 
                    ViChar wfmNames[])
{
    ViStatus   error;
    ViInt32    numWfms, maxWfms, buffSize, index, indexScanned;
    wfmNodePtr wfmRecord;
    ViString   tmpNames = VI_NULL;
    
    checkErr( Ivi_LockSession (vi, VI_NULL));
    
    if (arraySize < 1)
    viCheckParm (IVI_ERROR_INVALID_PARAMETER, 2, "Invalid Array Size");
    if (!wfmNames)
    viCheckParm (IVI_ERROR_INVALID_PARAMETER, 3, "NULL value for wfmNames");
    
    checkErr( Ivi_GetAttributeViAddr (vi, VI_NULL, HP33120A_ATTR_WFM_STRUCTURE,
                      0, (ViAddr *)(&wfmRecord)));
    checkErr( Ivi_GetAttributeViInt32 (vi, VI_NULL, HP33120A_ATTR_MAX_NUM_WAVEFORMS,
                       0, &maxWfms));
    checkErr( hp33120a_GetNumDefinedWaveforms (vi, maxWfms, &numWfms));
    
    buffSize = ((numWfms * (HP33120A_VAL_NAME_LENGTH + 1)));
    checkErr( Ivi_Alloc (vi, buffSize, (ViAddr *)(&tmpNames)));
    
    if (arraySize > buffSize)
    arraySize = buffSize;
    else if (arraySize < buffSize)
    error = buffSize;
    
    indexScanned = 0;
    for (index = 0; (index < maxWfms) && (indexScanned < numWfms); index++)
    {
    if (wfmRecord[index].wfmSize)
        {
        strcat (tmpNames, wfmRecord[index].wfmName);
        if (indexScanned < numWfms)
        strcat (tmpNames, ",");
        indexScanned++;
        }
    }
    strncpy (wfmNames, tmpNames, arraySize);
    wfmNames[arraySize] = '\0';
    
Error:
    if (tmpNames)
    Ivi_Free (vi, (ViAddr)(tmpNames));
    
    Ivi_UnlockSession (vi, VI_NULL);
    return error;
}

/*****************************************************************************
 * Function: hp33120a_EnableAM
 * Purpose:  This function configures the function generator to apply
 *           amplitude modulation to a channel. It also specifies the source
 *           of the modulating signal the function generator uses for the
 *           channel.  The driver sets the
 *           HP33120A_ATTR_AM_ENABLED attribute to VI_TRUE.
 *****************************************************************************/
ViStatus _VI_FUNC hp33120a_EnableAM (ViSession vi, ViConstString channelName,
                     ViInt32 amSource)
{
    ViStatus error = VI_SUCCESS;

    checkErr( Ivi_LockSession (vi, VI_NULL));

    /* Set attributes */
    checkErr( Ivi_SetAttributeViBoolean (vi, channelName, HP33120A_ATTR_AM_ENABLED,
                       0, VI_TRUE));
    viCheckParm( Ivi_SetAttributeViInt32 (vi, channelName, HP33120A_ATTR_AM_SOURCE,
                      0, amSource), 3, "AM Source");

    checkErr( hp33120a_CheckStatus (vi));

Error:
    Ivi_UnlockSession (vi, VI_NULL);
    return error;
}
    
/*****************************************************************************
 * Function: hp33120a_DisableAM
 * Purpose:  This function disables amplitude modulation of the output signal
 *           on the channel.  The driver sets the
 *           HP33120A_ATTR_AM_ENABLED attribute to VI_FALSE.
 *****************************************************************************/
ViStatus _VI_FUNC hp33120a_DisableAM (ViSession vi, ViConstString channelName)
{
    ViStatus error = VI_SUCCESS;

    checkErr( Ivi_LockSession (vi, VI_NULL));

    /* Set attribute */
    checkErr( Ivi_SetAttributeViBoolean (vi, channelName, HP33120A_ATTR_AM_ENABLED,
                       0, VI_FALSE));

    checkErr( hp33120a_CheckStatus (vi));

Error:
    Ivi_UnlockSession (vi, VI_NULL);
    return error;
}
                       
/*****************************************************************************
 * Function: hp33120a_ConfigureAMInternal
 * Purpose:  This function configures the attributes that control the
 *           function generator's internal amplitude modulation source.
 *           These attributes are the modulation depth, waveform, and
 *           frequency.
 *****************************************************************************/
ViStatus _VI_FUNC  hp33120a_ConfigureAMInternal (ViSession vi, ViReal64 amDepth,
                         ViInt32 amWaveform, ViReal64 amFrequency)
{
    ViStatus error = VI_SUCCESS;
    
    checkErr( Ivi_LockSession (vi, VI_NULL));

    /* Set attributes */
    viCheckParm( Ivi_SetAttributeViInt32 (vi, VI_NULL,
                      HP33120A_ATTR_AM_INTERNAL_WAVEFORM, 0,
                      amWaveform), 3, "AM Waveform");
    viCheckParm( Ivi_SetAttributeViReal64 (vi, VI_NULL,
                       HP33120A_ATTR_AM_INTERNAL_FREQUENCY, 0,
                       amFrequency), 4, "AM Frequency");
    viCheckParm( Ivi_SetAttributeViReal64 (vi, VI_NULL,
                      HP33120A_ATTR_AM_INTERNAL_DEPTH, 0,
                      amDepth), 2, "AM Depth");

    checkErr( hp33120a_CheckStatus (vi));

Error:
    Ivi_UnlockSession (vi, VI_NULL);
    return error;
}

/*****************************************************************************
 * Function: hp33120a_EnableFM
 * Purpose:  This function configures the function generator to apply
 *           frequency modulation to a channel.  It also specifies the source
 *           of the modulating signal the function generator uses for the
 *           channel.  The driver sets the hp33120a_ATTR_FM_ENABLED
 *           attribute to VI_TRUE.
 *****************************************************************************/
ViStatus _VI_FUNC hp33120a_EnableFM (ViSession vi, ViConstString channelName, 
                         ViInt32 fmSource)
{
    ViStatus error = VI_SUCCESS;
    
    checkErr( Ivi_LockSession (vi, VI_NULL));
    
    /* Set attributes */
    checkErr( Ivi_SetAttributeViBoolean (vi, channelName, HP33120A_ATTR_FM_ENABLED,
                     0, VI_TRUE));
    viCheckParm( Ivi_SetAttributeViInt32 (vi, channelName, HP33120A_ATTR_FM_SOURCE,
                      0, fmSource), 3, "FM Source");

    checkErr( hp33120a_CheckStatus (vi));

Error:
    Ivi_UnlockSession (vi, VI_NULL);
    return error;
}        
    
/*****************************************************************************
 * Function: hp33120a_DisableFM
 * Purpose:  This function disables frequency modulation of the output signal
 *           on the channel.  The driver sets the
 *           HP33120A_ATTR_FM_ENABLED attribute to VI_FALSE.
 *****************************************************************************/
ViStatus _VI_FUNC hp33120a_DisableFM (ViSession vi, ViConstString channelName)
{
    ViStatus error = VI_SUCCESS;

    checkErr( Ivi_LockSession (vi, VI_NULL));

    /* Set attribute */
    checkErr( Ivi_SetAttributeViBoolean (vi, channelName, HP33120A_ATTR_FM_ENABLED,
                     0, VI_FALSE));

    checkErr( hp33120a_CheckStatus (vi));

Error:
    Ivi_UnlockSession (vi, VI_NULL);
    return error;
}
    
/*****************************************************************************
 * Function: hp33120a_ConfigureFMInternal
 * Purpose:  This function configures the attributes that control the
 *           function generator's internal frequency modulation source.
 *           These attributes are the peak deviation, waveform, and
 *           frequency.
 *****************************************************************************/
ViStatus _VI_FUNC  hp33120a_ConfigureFMInternal (ViSession vi, ViReal64 fmDeviance,
                         ViInt32 fmWaveform, ViReal64 fmFrequency)
{
    ViStatus error = VI_SUCCESS;
    
    checkErr( Ivi_LockSession (vi, VI_NULL));

    /* Set attributes */
    viCheckParm( Ivi_SetAttributeViReal64 (vi, VI_NULL,
                      HP33120A_ATTR_FM_INTERNAL_DEVIATION, 0,
                      fmDeviance), 2, "FM Peak Deviance");
    viCheckParm( Ivi_SetAttributeViInt32 (vi, VI_NULL,
                      HP33120A_ATTR_FM_INTERNAL_WAVEFORM, 0,
                      fmWaveform), 3, "FM Waveform");
    viCheckParm( Ivi_SetAttributeViReal64 (vi, VI_NULL,
                       HP33120A_ATTR_FM_INTERNAL_FREQUENCY, 0,
                       fmFrequency), 4, "FM Frequency");
    
    checkErr( hp33120a_CheckStatus (vi));
    
Error:
    Ivi_UnlockSession (vi, VI_NULL);
    return error;
} 

/*****************************************************************************
 * Function: hp33120a_EnableFSK                                                   
 * Purpose:  This function configures the function generator to apply
 *           frequency shift key modulation to a channel.  It also specifies
 *           the hop frequency and modulating trigger source the function
 *           generator uses for the channel.  The driver sets the 
 *           hp33120a_ATTR_FSK_ENABLED attribute to VI_TRUE.
 *****************************************************************************/
ViStatus _VI_FUNC hp33120a_EnableFSK (ViSession vi, ViInt32 fskSource,
                      ViReal64 hopFrequency)
{
    ViStatus error = VI_SUCCESS;
    
    checkErr( Ivi_LockSession(vi, VI_NULL));
    
    checkErr( Ivi_SetAttributeViBoolean (vi, VI_NULL, HP33120A_ATTR_FSK_ENABLED,
                     0, VI_TRUE));
    viCheckParm( Ivi_SetAttributeViInt32 (vi, VI_NULL, HP33120A_ATTR_FSK_SOURCE,
                      0, fskSource), 2, "FSK Source");
    viCheckParm( Ivi_SetAttributeViReal64 (vi, VI_NULL, HP33120A_ATTR_FSK_HOP_FREQUENCY,
                       0, hopFrequency), 3, "FSK Hop Frequency");
 
    checkErr( hp33120a_CheckStatus (vi));
    
Error:
    Ivi_UnlockSession(vi, VI_NULL);
    return error;
}

/*****************************************************************************
 * Function: hp33120a_DisableFSK
 * Purpose:  This function disables frequency shift key modulation of the 
 *           output signal on the channel.  The driver sets the
 *           HP33120A_ATTR_FSK_ENABLED attribute to VI_FALSE.
 *****************************************************************************/
ViStatus _VI_FUNC hp33120a_DisableFSK (ViSession vi)
{
    ViStatus error = VI_SUCCESS;

    checkErr( Ivi_LockSession (vi, VI_NULL));

    /* Set attribute */
    checkErr( Ivi_SetAttributeViBoolean (vi, VI_NULL, HP33120A_ATTR_FSK_ENABLED,
                       0, VI_FALSE));

    checkErr( hp33120a_CheckStatus (vi));
    
Error:
    Ivi_UnlockSession (vi, VI_NULL);
    return error;
}

/*****************************************************************************
 * Function: hp33120a_ConfigureFSKInternal
 * Purpose:  This function configures the attributes that control the
 *           function generator's internal FSK modulation trigger rate.  This
 *           is the rate at which the modulated signal's frequency shifts
 *           between the carrier signal's frequency and the FSK hop frequency.
 *****************************************************************************/
ViStatus _VI_FUNC  hp33120a_ConfigureFSKInternal (ViSession vi, ViReal64 fskRate)
{
    ViStatus error = VI_SUCCESS;
    
    checkErr( Ivi_LockSession (vi, VI_NULL));

    /* Set attributes */
    viCheckParm( Ivi_SetAttributeViReal64 (vi, VI_NULL,
                       HP33120A_ATTR_FSK_INTERNAL_RATE, 0,
                       fskRate), 2, "FSK Shift Rate");

    checkErr( hp33120a_CheckStatus (vi));

Error:
    Ivi_UnlockSession (vi, VI_NULL);
    return error;
}

/*****************************************************************************
 * Function: hp33120a_EnableSweep
 * Purpose:  This function configures the function generator to apply
 *           frequency sweep modulation to a channel.  It also specifies
 *           the sweep trigger source, spacing, start frequency, stop frequency,
 *           and sweep time the function generator uses to produce the sweep
 *           modulated signal.  The driver sets the 
 *           hp33120a_ATTR_SWEEP_ENABLED attribute to VI_TRUE.
 *****************************************************************************/
ViStatus _VI_FUNC hp33120a_EnableSweep (ViSession vi, ViInt32 sweepSource,
                    ViInt32 sweepSpacing, ViReal64 startFreq,
                    ViReal64 stopFreq, ViReal64 sweepTime)
{
    ViStatus error = VI_SUCCESS;
    
    checkErr( Ivi_LockSession (vi, VI_NULL));
    
    
    /* Set Attribute */
    checkErr( Ivi_SetAttributeViBoolean (vi, VI_NULL, HP33120A_ATTR_SWEEP_ENABLED,
                     0, VI_TRUE));
    viCheckParm( Ivi_SetAttributeViInt32 (vi, VI_NULL, HP33120A_ATTR_SWEEP_SOURCE,
                      0, sweepSource), 2, "Sweep Source");
    viCheckParm( Ivi_SetAttributeViInt32 (vi, VI_NULL, HP33120A_ATTR_SWEEP_SPACING,
                      0, sweepSpacing), 3, "Sweep Spacing");
    viCheckParm( Ivi_SetAttributeViReal64 (vi, VI_NULL, HP33120A_ATTR_SWEEP_START_FREQ,
                       0, startFreq), 4, "Start Frequency");
    viCheckParm( Ivi_SetAttributeViReal64 (vi, VI_NULL, HP33120A_ATTR_SWEEP_STOP_FREQ,
                       0, stopFreq), 5, "Stop Frequency");
    viCheckParm( Ivi_SetAttributeViReal64 (vi, VI_NULL, HP33120A_ATTR_SWEEP_TIME,
                       0, sweepTime), 6, "Sweep Time");

    checkErr( hp33120a_CheckStatus (vi));

Error:
    Ivi_UnlockSession (vi, VI_NULL);
    return error;
}

/*****************************************************************************
 * Function: hp33120a_DisableSweep
 * Purpose:  This function disables frequency sweep modulation of the 
 *           output signal on the channel.  The driver sets the
 *           HP33120A_ATTR_SWEEP_ENABLED attribute to VI_FALSE.
 *****************************************************************************/
ViStatus _VI_FUNC hp33120a_DisableSweep (ViSession vi)
{
    ViStatus error = VI_SUCCESS;

    checkErr( Ivi_LockSession (vi, VI_NULL));

    /* Set attribute */
    checkErr( Ivi_SetAttributeViBoolean (vi, VI_NULL, HP33120A_ATTR_SWEEP_ENABLED,
                       0, VI_FALSE));
    checkErr( hp33120a_CheckStatus (vi));

Error:
    Ivi_UnlockSession (vi, VI_NULL);
    return error;
}

/*****************************************************************************
 * Function: hp33120a_EnableBurst
 * Purpose:  This function configures the function generator to apply
 *           burst modulation to a channel.  It also specifies
 *           the burst trigger source the function generator uses to produce 
 *           the burst modulated signal.  The driver sets the 
 *           hp33120a_ATTR_BURST_ENABLED attribute to VI_TRUE.
 *****************************************************************************/
ViStatus _VI_FUNC hp33120a_EnableBurst (ViSession vi, ViInt32 burstSource,
                    ViInt32 burstCount, ViReal64 burstPhase)
{
    ViStatus error = VI_SUCCESS;
    
    checkErr( Ivi_LockSession (vi, VI_NULL));
    
    
    /* Set Attribute */
    checkErr( Ivi_SetAttributeViBoolean (vi, VI_NULL, HP33120A_ATTR_BURST_ENABLED,
                     0, VI_TRUE));
    viCheckParm( Ivi_SetAttributeViInt32 (vi, VI_NULL, HP33120A_ATTR_BURST_SOURCE,
                      0, burstSource), 2, "Burst Source");
    viCheckParm( Ivi_SetAttributeViInt32 (vi, VI_NULL, HP33120A_ATTR_BURST_COUNT,
                      0, burstCount), 3, "Burst Count");
    viCheckParm( Ivi_SetAttributeViReal64 (vi, VI_NULL, HP33120A_ATTR_BURST_PHASE,
                       0, burstPhase), 4, "Burst Phase");
                       
    checkErr( hp33120a_CheckStatus (vi));

Error:
    Ivi_UnlockSession (vi, VI_NULL);
    return error;
}

/*****************************************************************************
 * Function: hp33120a_DisableBurst
 * Purpose:  This function disables burst modulation of the 
 *           output signal on the channel.  The driver sets the
 *           HP33120A_ATTR_BURST_ENABLED attribute to VI_FALSE.
 *****************************************************************************/
ViStatus _VI_FUNC hp33120a_DisableBurst (ViSession vi)
{
    ViStatus error = VI_SUCCESS;

    checkErr( Ivi_LockSession (vi, VI_NULL));

    /* Set attribute */
    checkErr( Ivi_SetAttributeViBoolean (vi, VI_NULL, HP33120A_ATTR_BURST_ENABLED,
                     0, VI_FALSE));
    checkErr( hp33120a_CheckStatus (vi));

Error:
    Ivi_UnlockSession (vi, VI_NULL);
    return error;
}

/*****************************************************************************
 * Function: hp33120a_ConfigureBurstInternal
 * Purpose:  This function configures the attributes that control the
 *           function generator's internal burst modulation trigger rate.  This
 *           is the rate at which the function generator produces bursts of the
 *           carrier signal.
 *****************************************************************************/
ViStatus _VI_FUNC hp33120a_ConfigureBurstInternal (ViSession vi, ViReal64 burstRate)
{
    ViStatus error = VI_SUCCESS;
    
    checkErr( Ivi_LockSession (vi, VI_NULL));

    /* Set attribute */
    viCheckParm( Ivi_SetAttributeViReal64 (vi, VI_NULL, HP33120A_ATTR_BURST_INTERNAL_RATE,
                       0, burstRate), 2, "Internal Burst Rate");
    checkErr( hp33120a_CheckStatus (vi));

Error:
    Ivi_UnlockSession (vi, VI_NULL);
    return error;
}

/*****************************************************************************
 * Function: hp33120a_SendSWTrigger                                                   
 * Purpose:  Sends a command to trigger the instrument.                      
 *****************************************************************************/
ViStatus _VI_FUNC hp33120a_SendSWTrigger (ViSession vi)
{
    ViStatus    error = VI_SUCCESS;

    checkErr( Ivi_LockSession(vi, VI_NULL));

    if (!Ivi_Simulating(vi))                /* call only when locked */
    {
    ViSession   io = Ivi_IOSession(vi); /* call only when locked */
    
    checkErr( Ivi_SetNeedToCheckStatus (vi, VI_TRUE));

    viCheckErr( viWrite (io, "*TRG;", 5, VI_NULL));                     
    }
    
    /* 
        Do not invoke the hp33120a_CheckStatus function here.  
        After the user calls this function, the user can check 
        for errors by calling the hp33120a_error_query function.
    */   

Error:    
    Ivi_UnlockSession(vi, VI_NULL);
    return error;
}

/*****************************************************************************
 * Function: hp33120a_GetAttribute<type> Functions                                    
 * Purpose:  These functions enable the instrument driver user to get 
 *           attribute values directly.  There are typesafe versions for 
 *           ViInt32, ViReal64, ViString, ViBoolean, and ViSession attributes.                                         
 *****************************************************************************/
ViStatus _VI_FUNC hp33120a_GetAttributeViInt32 (ViSession vi, ViConstString channelName, 
                        ViAttr attributeId, ViInt32 *value)
{                                                                                                           
    return Ivi_GetAttributeViInt32 (vi, channelName, attributeId, IVI_VAL_DIRECT_USER_CALL, 
                    value);
}                                                                                                           
ViStatus _VI_FUNC hp33120a_GetAttributeViReal64 (ViSession vi, ViConstString channelName, 
                         ViAttr attributeId, ViReal64 *value)
{                                                                                                           
    return Ivi_GetAttributeViReal64 (vi, channelName, attributeId, IVI_VAL_DIRECT_USER_CALL, 
                     value);
}                                                                                                           
ViStatus _VI_FUNC hp33120a_GetAttributeViString (ViSession vi, ViConstString channelName, 
                         ViAttr attributeId, ViInt32 bufSize, 
                         ViChar value[]) 
{   
    return Ivi_GetAttributeViString (vi, channelName, attributeId, IVI_VAL_DIRECT_USER_CALL, 
                     bufSize, value);
}   
ViStatus _VI_FUNC hp33120a_GetAttributeViBoolean (ViSession vi, ViConstString channelName, 
                          ViAttr attributeId, ViBoolean *value)
{                                                                                                           
    return Ivi_GetAttributeViBoolean (vi, channelName, attributeId, IVI_VAL_DIRECT_USER_CALL, 
                      value);
}                                                                                                           
ViStatus _VI_FUNC hp33120a_GetAttributeViSession (ViSession vi, ViConstString channelName, 
                          ViAttr attributeId, ViSession *value)
{                                                                                                           
    return Ivi_GetAttributeViSession (vi, channelName, attributeId, IVI_VAL_DIRECT_USER_CALL, 
                      value);
}                                                                                                           

/*****************************************************************************
 * Function: hp33120a_SetAttribute<type> Functions                                    
 * Purpose:  These functions enable the instrument driver user to set 
 *           attribute values directly.  There are typesafe versions for 
 *           ViInt32, ViReal64, ViString, ViBoolean, and ViSession datatypes.                                         
 *****************************************************************************/
ViStatus _VI_FUNC hp33120a_SetAttributeViInt32 (ViSession vi, ViConstString channelName, 
                        ViAttr attributeId, ViInt32 value)
{                                                                                                           
    return Ivi_SetAttributeViInt32 (vi, channelName, attributeId, IVI_VAL_DIRECT_USER_CALL, 
                    value);
}                                                                                                           
ViStatus _VI_FUNC hp33120a_SetAttributeViReal64 (ViSession vi, ViConstString channelName, 
                         ViAttr attributeId, ViReal64 value)
{                                                                                                           
    return Ivi_SetAttributeViReal64 (vi, channelName, attributeId, IVI_VAL_DIRECT_USER_CALL, 
                     value);
}                                                                                                           
ViStatus _VI_FUNC hp33120a_SetAttributeViString (ViSession vi, ViConstString channelName, 
                         ViAttr attributeId, ViConstString value) 
{   
    return Ivi_SetAttributeViString (vi, channelName, attributeId, IVI_VAL_DIRECT_USER_CALL, 
                     value);
}   
ViStatus _VI_FUNC hp33120a_SetAttributeViBoolean (ViSession vi, ViConstString channelName, 
                          ViAttr attributeId, ViBoolean value)
{                                                                                                           
    return Ivi_SetAttributeViBoolean (vi, channelName, attributeId, IVI_VAL_DIRECT_USER_CALL, 
                      value);
}                                                                                                           
ViStatus _VI_FUNC hp33120a_SetAttributeViSession (ViSession vi, ViConstString channelName, 
                          ViAttr attributeId, ViSession value)
{                                                                                                           
    return Ivi_SetAttributeViSession (vi, channelName, attributeId, IVI_VAL_DIRECT_USER_CALL, 
                      value);
}                                                                                                           

/*****************************************************************************
 * Function: hp33120a_CheckAttribute<type> Functions                                  
 * Purpose:  These functions enable the instrument driver user to check  
 *           attribute values directly.  These functions check the value you
 *           specify even if you set the HP33120A_ATTR_RANGE_CHECK 
 *           attribute to VI_FALSE.  There are typesafe versions for ViInt32, 
 *           ViReal64, ViString, ViBoolean, and ViSession datatypes.                         
 *****************************************************************************/
ViStatus _VI_FUNC hp33120a_CheckAttributeViInt32 (ViSession vi, ViConstString channelName, 
                          ViAttr attributeId, ViInt32 value)
{                                                                                                           
    return Ivi_CheckAttributeViInt32 (vi, channelName, attributeId, IVI_VAL_DIRECT_USER_CALL, 
                      value);
}
ViStatus _VI_FUNC hp33120a_CheckAttributeViReal64 (ViSession vi, ViConstString channelName, 
                           ViAttr attributeId, ViReal64 value)
{                                                                                                           
    return Ivi_CheckAttributeViReal64 (vi, channelName, attributeId, IVI_VAL_DIRECT_USER_CALL, 
                       value);
}                                                                                                           
ViStatus _VI_FUNC hp33120a_CheckAttributeViString (ViSession vi, ViConstString channelName, 
                           ViAttr attributeId, ViConstString value)  
{   
    return Ivi_CheckAttributeViString (vi, channelName, attributeId, IVI_VAL_DIRECT_USER_CALL, 
                       value);
}   
ViStatus _VI_FUNC hp33120a_CheckAttributeViBoolean (ViSession vi, ViConstString channelName, 
                            ViAttr attributeId, ViBoolean value)
{                                                                                                           
    return Ivi_CheckAttributeViBoolean (vi, channelName, attributeId, IVI_VAL_DIRECT_USER_CALL, 
                    value);
}                                                                                                           
ViStatus _VI_FUNC hp33120a_CheckAttributeViSession (ViSession vi, ViConstString channelName, 
                            ViAttr attributeId, ViSession value)
{                                                                                                           
    return Ivi_CheckAttributeViSession (vi, channelName, attributeId, IVI_VAL_DIRECT_USER_CALL, 
                    value);
}                                                                                                           

/*****************************************************************************
 * Function: hp33120a_LockSession and hp33120a_UnlockSession Functions                        
 * Purpose:  These functions enable the instrument driver user to lock the 
 *           session around a sequence of driver calls during which other
 *           execution threads must not disturb the instrument state.
 *                                                                          
 *           NOTE:  The callerHasLock parameter must be a local variable 
 *           initialized to VI_FALSE and passed by reference, or you can pass 
 *           VI_NULL.                     
 *****************************************************************************/
ViStatus _VI_FUNC hp33120a_LockSession (ViSession vi, ViBoolean *callerHasLock)  
{                                              
    return Ivi_LockSession(vi,callerHasLock);      
}                                              
ViStatus _VI_FUNC hp33120a_UnlockSession (ViSession vi, ViBoolean *callerHasLock) 
{                                              
    return Ivi_UnlockSession(vi,callerHasLock);    
}   
/*****************************************************************************
 * Function: hp33120a_GetErrorInfo and hp33120a_ClearErrorInfo Functions                       
 * Purpose:  These functions enable the instrument driver user to  
 *           get or clear the error information the driver associates with the
 *           IVI session.                                                        
 *****************************************************************************/
ViStatus _VI_FUNC hp33120a_GetErrorInfo (ViSession vi, ViStatus *primaryError, 
                     ViStatus *secondaryError, ViChar errorElaboration[256])  
{                                                                                                           
    return Ivi_GetErrorInfo(vi, primaryError, secondaryError, errorElaboration);                                
}                                                                                                           
ViStatus _VI_FUNC hp33120a_ClearErrorInfo (ViSession vi)                                                        
{                                                                                                           
    return Ivi_ClearErrorInfo (vi);                                                                             
}

/*****************************************************************************
 * Function: WriteInstrData and ReadInstrData Functions                      
 * Purpose:  These functions enable the instrument driver user to  
 *           write and read commands directly to and from the instrument.            
 *                                                                           
 *           Note:  These functions bypass the IVI attribute state caching.  
 *                  WriteInstrData invalidates the cached values for all 
 *                  attributes.
 *****************************************************************************/
ViStatus _VI_FUNC hp33120a_WriteInstrData (ViSession vi, ViConstString writeBuffer)   
{   
    return Ivi_WriteInstrData (vi, writeBuffer);    
}   
ViStatus _VI_FUNC hp33120a_ReadInstrData (ViSession vi, ViInt32 numBytes, 
                      ViChar rdBuf[], ViInt32 *bytesRead)  
{   
    return Ivi_ReadInstrData (vi, numBytes, rdBuf, bytesRead);   
} 

/*****************************************************************************
 *-------------------- Utility Functions (Not Exported) ---------------------*
 *****************************************************************************/
/*****************************************************************************
 * Function: hp33120a_VerifyOutputMode
 * Purpose:  This utility function verifies that the instrument is currently
 *           in a specific output mode.  If the instrument is not in the,
 *           correct output mode, the function returns the error
 *           HP33120A_ERROR_NOT_CONFIGURABLE.
 *****************************************************************************/
static ViStatus hp33120a_VerifyOutputMode(ViSession vi, ViInt32 outputMode)
{
    ViStatus error = VI_SUCCESS;
    ViInt32 actualMode = VI_NULL;
    
    checkErr( Ivi_GetAttributeViInt32 (vi, VI_NULL, HP33120A_ATTR_OUTPUT_MODE,
                       0, &actualMode));
    if (outputMode != actualMode)
    viCheckErrElab( HP33120A_ERROR_INVALID_MODE,
            "The istrument is not in the correct output mode to complete this action.");
         
Error:
    return error;
}

/*****************************************************************************
 * Function: hp33120a_GetEquivalentFrequency
 * Purpose:  This utility function determines the instrument's output mode
 *           and returns the equivalent frequency of the channel's waveform.
 *           If the instrument is configured to produce an arbitrary waveform,
 *           the equivalent frequency is the waveform size multiplied by the
 *           sample rate.
 *****************************************************************************/
static ViStatus hp33120a_GetEquivalentFrequency (ViSession vi, ViConstString channelName,
                         ViReal64 *frequency, ViReal64 *maxFreq)
{
    ViStatus error = VI_SUCCESS;
    ViInt32  outputMode, wfmHandle, wfmSize;
    ViReal64 sampleRate, tmpFreq, tmpMax;
    
    checkErr( Ivi_GetAttributeViInt32 (vi, VI_NULL, HP33120A_ATTR_OUTPUT_MODE,
                       0, &outputMode));

    if (outputMode == HP33120A_VAL_OUTPUT_ARB)
    {
    checkErr( Ivi_GetAttributeViReal64 (vi, VI_NULL, HP33120A_ATTR_ARB_SAMPLE_RATE,
                       0, &sampleRate));
    checkErr( Ivi_GetAttributeViInt32 (vi, channelName, HP33120A_ATTR_ARB_WAVEFORM_HANDLE,
                       0, &wfmHandle));
    checkErr( hp33120a_GetWfmSize (vi, wfmHandle, &wfmSize));
      
    tmpFreq = sampleRate * wfmSize;
    
    if (wfmSize > 12287)
        tmpMax = 200.0e+3;
    else if (wfmSize > 8192)
        tmpMax = 2.5e+6;
    else
        tmpMax = 5.0e+6;
    }
    else
    {
    checkErr( Ivi_GetAttributeViReal64 (vi, channelName, HP33120A_ATTR_FUNC_FREQUENCY,
                        0, &tmpFreq));
    checkErr( Ivi_GetAttrMinMaxViReal64 (vi, channelName, HP33120A_ATTR_FUNC_FREQUENCY, VI_NULL,
                         &tmpMax, VI_NULL, VI_NULL));
    }                                     
Error:
    *frequency = ((error < VI_SUCCESS) ? 0.0 : tmpFreq);
    if (maxFreq)
    *maxFreq = ((error < VI_SUCCESS) ? 0.0 : tmpMax);
    return error;
}

/*****************************************************************************
 * Function: hp33120a_GetWfmSize
 * Purpose:  This utility function returnss the size of the specified arbitrary
 *           waveform.
 *****************************************************************************/
static ViStatus hp33120a_GetWfmSize (ViSession vi, ViInt32 wfmHandle, ViInt32 *wfmSize)
{
    ViStatus   error = VI_SUCCESS;
    ViInt32    index;
    
    wfmNodePtr wfmRecord = VI_NULL;
    
    checkErr( Ivi_GetAttributeViAddr (vi, VI_NULL, HP33120A_ATTR_WFM_STRUCTURE,
                      0, (ViAddr *)(&wfmRecord)));

    index = HP33120A_WFM_INDEX_FROM_HANDLE (wfmHandle);
    
    *wfmSize = wfmRecord[index].wfmSize;
    
Error:
    return error;
}

/*****************************************************************************
 * Function: hp33120a_AnyModulationEnabled
 * Purpose:  This utility function returnss a boolean value of VI_TRUE to the
 *           calling function if any form of modulation is enabled.
 *****************************************************************************/
static ViStatus hp33120a_AnyModulationEnabled (ViSession vi, ViConstString channelName,
                           ViInt32 modType, ViBoolean *modOn)
{
    ViStatus    error = VI_SUCCESS;
    ViBoolean   amOn = VI_FALSE, fmOn = VI_FALSE, fskOn = VI_FALSE, sweepOn = VI_FALSE,
        burstOn = VI_FALSE;
    
    if (modType != HP33120A_VAL_AM)
    checkErr( Ivi_GetAttributeViBoolean (vi, channelName, HP33120A_ATTR_AM_ENABLED,
                         0, &amOn));
    if (modType != HP33120A_VAL_FM)
    checkErr( Ivi_GetAttributeViBoolean (vi, channelName, HP33120A_ATTR_FM_ENABLED,
                         0, &fmOn));
    if (modType != HP33120A_VAL_FSK)
    checkErr( Ivi_GetAttributeViBoolean (vi, VI_NULL, HP33120A_ATTR_FSK_ENABLED,
                         0, &fskOn));
    if (modType != HP33120A_VAL_BURST)
    checkErr( Ivi_GetAttributeViBoolean (vi, VI_NULL, HP33120A_ATTR_BURST_ENABLED,
                         0, &sweepOn));
    if (modType != HP33120A_VAL_SWEEP)
    checkErr( Ivi_GetAttributeViBoolean (vi, VI_NULL, HP33120A_ATTR_SWEEP_ENABLED,
                         0, &burstOn));
    
    *modOn = ((amOn) || (fmOn) || (fskOn) || (burstOn) || (sweepOn));
                         
Error:
    if (error < VI_SUCCESS)
    *modOn = VI_FALSE;
    return error;
}

/*****************************************************************************
 * Function: hp33120a_VerifyWfmCreatable
 * Purpose:  This utility function verifies that a user-specified waveform is
 *           of valid size, that the data in the waveform is valid, and that
 *           there is memory available for the new waveform.
 *****************************************************************************/
static ViStatus hp33120a_VerifyWfmCreatable (ViSession vi, ViInt32 wfmSize,
                    ViReal64 wfmData[])
{
    ViStatus  error = VI_SUCCESS;
    ViBoolean posRange = VI_FALSE, negRange = VI_FALSE;
    
    ViInt32 maxWfms, currentWfms, wfmQuantum, minWfmSize, maxWfmSize;
    ViInt16 i;
    
    checkErr( Ivi_GetAttributeViInt32 (vi, VI_NULL, HP33120A_ATTR_MAX_NUM_WAVEFORMS,
                       0, &maxWfms));
    checkErr( Ivi_GetAttributeViInt32 (vi, VI_NULL, HP33120A_ATTR_WAVEFORM_QUANTUM,
                       0, &wfmQuantum));
    checkErr( Ivi_GetAttributeViInt32 (vi, VI_NULL, HP33120A_ATTR_MIN_WAVEFORM_SIZE,
                       0, &minWfmSize));
    checkErr( Ivi_GetAttributeViInt32 (vi, VI_NULL, HP33120A_ATTR_MAX_WAVEFORM_SIZE,
                       0, &maxWfmSize));
    checkErr( hp33120a_GetNumDefinedWaveforms (vi, maxWfms, &currentWfms));
    
    if (currentWfms >= maxWfms)
    viCheckErr( HP33120A_ERROR_NO_WFMS_AVAILABLE);
    
    if ((wfmSize < minWfmSize) || (wfmSize > maxWfmSize) || (wfmSize % wfmQuantum))
    viCheckErr( HP33120A_ERROR_INVALID_WFM_LENGTH);
       
    for (i = 0; i < wfmSize; i++)
    {
    if ((fabs(wfmData[i])) > 1.0)
        {
        ViChar errElab[BUFFER_SIZE];
        sprintf(errElab, "Element number %d of the waveform is not valid.",i);
        viCheckErrElab( HP33120A_ERROR_INVALID_WFM_ELEMENT, errElab);
        }
    posRange = (wfmData[i] ==  1.0 ? VI_TRUE : posRange);
    negRange = (wfmData[i] == -1.0 ? VI_TRUE : negRange);
    }
    if (!(posRange && negRange))
    {
    viCheckErrElab( HP33120A_ERROR_INVALID_WFM_ELEMENT,
            "The waveform's elements do not span the full range of -1.0 to +1.0.");
    }
Error:
    return error;
}

/*****************************************************************************
 * Function: hp33120a_GetNewWfmInfo
 * Purpose:  This utility function determines the handle for a new waveform
 *           from the number of current user-defined waveforms.
 *****************************************************************************/
static ViStatus hp33120a_GetNewWfmInfo (ViSession vi, ViInt32 *wfmHandle,
                    ViChar wfmName[])
{
    ViStatus error = VI_SUCCESS;
    ViInt32  tmpHandle, maxWfms;
    ViChar   tmpName[HP33120A_VAL_NAME_LENGTH];
    wfmNodePtr wfmRecord;
    
    checkErr( Ivi_GetAttributeViAddr (vi, VI_NULL, HP33120A_ATTR_WFM_STRUCTURE,
                      0, (ViAddr *)&wfmRecord));
    checkErr( Ivi_GetAttributeViInt32 (vi, VI_NULL, HP33120A_ATTR_MAX_NUM_WAVEFORMS,
                       0, &maxWfms));
    
    for (tmpHandle = 0; tmpHandle < maxWfms; tmpHandle++)
    {
    if (!(wfmRecord[tmpHandle].wfmSize))
        {
        break;
        }
    }
    
    tmpHandle = HP33120A_WFM_HANDLE_FROM_INDEX(tmpHandle);
    checkErr( hp33120a_GetNewWfmName (vi, wfmRecord, maxWfms, tmpHandle, tmpName));
    
Error:
    if (error < VI_SUCCESS)
    {
    *wfmHandle = VI_NULL;
    strcpy (wfmName, "\0");
    }
    else
    {
    *wfmHandle = tmpHandle;
    strncpy (wfmName, tmpName, HP33120A_VAL_NAME_LENGTH);
    }
    return error;
}

/*****************************************************************************
 * Function: hp33120a_GetNewWfmName
 * Purpose:  This utilitiy function determines the name for a new waveform.
 *****************************************************************************/
static ViStatus hp33120a_GetNewWfmName (ViSession vi, wfmNodePtr wfmRecord, ViInt32 maxWfms,
                    ViInt32 wfmHandle, ViChar wfmName[])
{
    ViStatus  error = VI_SUCCESS;
    ViInt32   index;
    ViBoolean nameExists = VI_TRUE;
    ViChar tmpName[HP33120A_VAL_NAME_LENGTH];
    
    while (nameExists)
    {
    nameExists = VI_FALSE;
    sprintf (tmpName, "ARB%d", wfmHandle);
    for (index = 0; index < maxWfms; index++)
        {
        if (!strncmp (tmpName, wfmRecord[index].wfmName, HP33120A_VAL_NAME_LENGTH))
        {
        nameExists = VI_TRUE;
        wfmHandle++;
        break;
        }
        }
        
    if (!nameExists)
        strcpy (wfmName, tmpName);
    }
    return VI_SUCCESS;
}

/*****************************************************************************
 * Function: hp33120a_GetWfmNameFromHandleUtil
 * Purpose:  This utilitiy function returns a waveform's name given its handle.
 *****************************************************************************/
static ViStatus hp33120a_GetWfmNameFromHandleUtil (ViSession vi, ViInt32 wfmHandle, 
                           ViInt32 arraySize, ViChar wfmName[])
{
    ViStatus  error = VI_SUCCESS;
    ViInt32   index;
    ViBoolean wfmExists;
    wfmNodePtr wfmRecord;
    
    checkErr( Ivi_GetAttributeViAddr (vi, VI_NULL, HP33120A_ATTR_WFM_STRUCTURE,
                      0, (ViAddr *)(&wfmRecord)));
    
    checkErr( hp33120a_WfmExists (vi, wfmHandle, &wfmExists));
    if (!wfmExists)
    viCheckErrElab( HP33120A_ERROR_INVALID_WAVEFORM,
            "The waveform handle you specified was not found.");
    
    index = HP33120A_WFM_INDEX_FROM_HANDLE (wfmHandle);
    
    if (arraySize > HP33120A_VAL_NAME_LENGTH)
    arraySize = HP33120A_VAL_NAME_LENGTH;
    else if (arraySize < HP33120A_VAL_NAME_LENGTH)
    error = HP33120A_VAL_NAME_LENGTH;
    
    strncpy (wfmName, wfmRecord[index].wfmName, arraySize);
    wfmName[arraySize] = '\0';

Error:
    return error;
}

/*****************************************************************************
 * Function: hp33120a_GetWfmHandleFromNameUtil
 * Purpose:  This utilitiy function returns a waveform's handle given its name.
 *****************************************************************************/
static ViStatus hp33120a_GetWfmHandleFromNameUtil (ViSession vi, ViConstString wfmName,
                           ViInt32 *wfmHandle)
{
    ViStatus error = VI_SUCCESS;
    ViInt32 index, maxWfms;
    wfmNodePtr wfmRecord;
    ViBoolean nameExists = VI_FALSE;
    
    checkErr( Ivi_GetAttributeViAddr (vi, VI_NULL, HP33120A_ATTR_WFM_STRUCTURE,
                      0, (ViAddr *)(&wfmRecord)));
    checkErr( Ivi_GetAttributeViInt32 (vi, VI_NULL, HP33120A_ATTR_MAX_NUM_WAVEFORMS,
                       0, &maxWfms));
                       
    for (index = 0; index < maxWfms; index++)
    {
    if (!strncmp (wfmName, wfmRecord[index].wfmName, HP33120A_VAL_NAME_LENGTH))
        {
        nameExists = VI_TRUE;
        *wfmHandle = HP33120A_WFM_HANDLE_FROM_INDEX(index);
        break;
        }
    }
    if (!nameExists)
    {
    *wfmHandle = VI_NULL;
    viCheckErrElab( HP33120A_ERROR_INVALID_WAVEFORM,
            "The waveform name you specified was not found.");
    }

Error:
    return error;
}

/*****************************************************************************
 * Function: hp33120a_GetNumDefinedWaveforms
 * Purpose:  This utilitiy function returns the number of arbitrary waveforms
 *           the user has created.
 *****************************************************************************/
static ViStatus hp33120a_GetNumDefinedWaveforms (ViSession vi, ViInt32 maxWfms, ViInt32 *numWfms)
{
    ViStatus   error = VI_SUCCESS;
    ViInt32    index, tmpNumWfms = 0;
    wfmNodePtr wfmRecord;
    
    /* Determine the current number of defined waveforms */
     checkErr( Ivi_GetAttributeViAddr (vi, VI_NULL, HP33120A_ATTR_WFM_STRUCTURE,
                       0, (ViAddr *)(&wfmRecord)));
    for (index = 0; index < maxWfms; index++)
    {
        if (wfmRecord[index].wfmSize)
        tmpNumWfms++;
    }
    
Error:
    if (error < VI_SUCCESS)
    *numWfms = 0;
    else
    *numWfms = tmpNumWfms;    
    return error;
}

/*****************************************************************************
 * Function: hp33120a_CreateDriverWfmRecord
 * Purpose:  This utilitiy function creates a new array of waveform structures
 *           and assigns a pointer to the array to the hidden attribute
 *           HP33120A_ATTR_WFM_STRUCTURE.  This function is
 *           called as from the hp33120a_InitAttributes function as
 *           part of the initialization of the driver.
 *****************************************************************************/
static ViStatus hp33120a_CreateDriverWfmRecord (ViSession vi)
{
    ViStatus error = VI_SUCCESS;
    ViInt32 maxWfms;
    wfmNodePtr newWfmStruct;
    
    checkErr( Ivi_GetAttributeViInt32 (vi, VI_NULL, HP33120A_ATTR_MAX_NUM_WAVEFORMS,
                       0, &maxWfms));
    checkErr( Ivi_GetAttributeViAddr (vi, VI_NULL, HP33120A_ATTR_WFM_STRUCTURE,
                      0, (ViAddr *)(&newWfmStruct)));
    if (newWfmStruct == VI_NULL)
    {
    checkErr( Ivi_Alloc (vi, sizeof(wfmNode)*maxWfms,  (ViAddr *)(&newWfmStruct)));
    checkErr( Ivi_SetAttributeViAddr (vi, VI_NULL, HP33120A_ATTR_WFM_STRUCTURE,
                      0, (ViAddr)newWfmStruct));
    }
    
Error:
    return error;
}

/*****************************************************************************
 * Function: hp33120a_ClearDriverWfmRecord
 * Purpose:  This utilitiy function clears data from an existing waveform
 *           data structure.  If the waveforms handle parameter equals
 *           HP33120A_VAL_ALL_WAVEFORMS, then the entire data
 *           structure is cleared.
 *****************************************************************************/
static ViStatus hp33120a_ClearDriverWfmRecord (ViSession vi, ViInt32 wfmHandle)
{
    ViStatus   error = VI_SUCCESS;
    ViInt32    index, maxWfms;
    wfmNodePtr wfmRecord;
    
    checkErr( Ivi_GetAttributeViInt32 (vi, VI_NULL, HP33120A_ATTR_MAX_NUM_WAVEFORMS,
                       0, &maxWfms));
    checkErr( Ivi_GetAttributeViAddr (vi, VI_NULL, HP33120A_ATTR_WFM_STRUCTURE,
                      0, (ViAddr *)&wfmRecord));
    
    if (wfmHandle == HP33120A_VAL_ALL_WAVEFORMS)
    {
    memset ((wfmRecord + 5), 0, (sizeof(wfmNode)*(maxWfms - 5)));
    }
    else
    {
    index = HP33120A_WFM_INDEX_FROM_HANDLE(wfmHandle);
    wfmRecord[index].wfmSize = 0;
    wfmRecord[index].wfmName[0] = '\0';
    }
       
Error:
    return error;
}

/*****************************************************************************
 * Function: hp33120a_UpdateDriverWfmRecord
 * Purpose:  This utilitiy function updates an element of an existing waveform
 *           data structure to reflect a newly created waveform.
 *****************************************************************************/
static ViStatus hp33120a_UpdateDriverWfmRecord (ViSession vi, ViInt32 wfmHandle,
                           ViInt32 wfmSize, ViChar wfmName[])
{
    ViStatus   error = VI_SUCCESS;
    ViInt32    index;
    wfmNodePtr wfmRecord;
    
    checkErr( Ivi_GetAttributeViAddr (vi, VI_NULL, HP33120A_ATTR_WFM_STRUCTURE,
                      0, (ViAddr *)&wfmRecord));
    index = HP33120A_WFM_INDEX_FROM_HANDLE(wfmHandle);
    wfmRecord[index].wfmSize = wfmSize;
    strcpy (wfmRecord[index].wfmName, wfmName);
    
Error:
    return error;
}

/*****************************************************************************
 * Function: hp33120a_WfmExists
 * Purpose:  This utilitiy function checks to see whether a waveform exists.
 *****************************************************************************/
static ViStatus hp33120a_WfmExists (ViSession vi, ViInt32 wfmHandle, ViBoolean *wfmExists)
{
    ViStatus   error = VI_SUCCESS;
    ViInt32    index, maxWfms;
    wfmNodePtr wfmStructPtr;
    
    checkErr( Ivi_GetAttributeViAddr (vi, VI_NULL, HP33120A_ATTR_WFM_STRUCTURE,
                      0, (ViAddr *)&wfmStructPtr));
    checkErr( Ivi_GetAttributeViInt32 (vi, VI_NULL, HP33120A_ATTR_MAX_NUM_WAVEFORMS,
                       0, &maxWfms));
    
    if ((wfmHandle >= (HP33120A_VAL_FIRST_WAVEFORM_HANDLE + maxWfms)) ||
    (wfmHandle < (HP33120A_VAL_FIRST_WAVEFORM_HANDLE)))
    *wfmExists = VI_FALSE;
    else
    {
    index =  HP33120A_WFM_INDEX_FROM_HANDLE(wfmHandle);
    *wfmExists = (!!(wfmStructPtr[index].wfmSize));
    }
Error:
    if (error < VI_SUCCESS)
    *wfmExists = VI_FALSE;
    return error;
}

/*****************************************************************************
 * Function: hp33120a_CheckStatus                                                 
 * Purpose:  This function checks the status of the instrument to detect 
 *           whether the instrument has encountered an error.  This function  
 *           is called at the end of most exported driver functions.  If the    
 *           instrument reports an error, this function returns      
 *           IVI_ERROR_INSTR_SPECIFIC.  The user can set the 
 *           IVI_ATTR_QUERY_INSTR_STATUS attribute to VI_FALSE to disable this 
 *           check and increase execution speed.                                   
 *
 *           Note:  Call this function only when the session is locked.
 *****************************************************************************/
static ViStatus hp33120a_CheckStatus (ViSession vi)
{
    ViStatus    error = VI_SUCCESS;

    if (Ivi_QueryInstrStatus (vi) && Ivi_NeedToCheckStatus (vi) && !Ivi_Simulating (vi))
    {
    checkErr( hp33120a_CheckStatusCallback (vi, Ivi_IOSession(vi)));
    checkErr( Ivi_SetNeedToCheckStatus (vi, VI_FALSE));
    }
    
Error:
    return error;
}

/*****************************************************************************
 * Function: hp33120a_WaitForOPC                                                  
 * Purpose:  This function waits for the instrument to complete the      
 *           execution of all pending operations.  This function is a        
 *           wrapper for the WaitForOPCCallback.  It can be called from 
 *           other instrument driver functions. 
 *
 *           The maxTime parameter specifies the maximum time to wait for
 *           operation complete in milliseconds.
 *
 *           Note:  Call this function only when the session locked.
 *****************************************************************************/
static ViStatus hp33120a_WaitForOPC (ViSession vi, ViInt32 maxTime) 
{
    ViStatus    error = VI_SUCCESS;

    if (!Ivi_Simulating(vi))
    {
    ViInt32  oldOPCTimeout; 
    
    checkErr( Ivi_GetAttributeViInt32 (vi, VI_NULL, HP33120A_ATTR_OPC_TIMEOUT, 
                       0, &oldOPCTimeout));
    Ivi_SetAttributeViInt32 (vi, VI_NULL, HP33120A_ATTR_OPC_TIMEOUT,        
                 0, maxTime);

    error = hp33120a_WaitForOPCCallback (vi, Ivi_IOSession(vi));

    Ivi_SetAttributeViInt32 (vi, VI_NULL, HP33120A_ATTR_OPC_TIMEOUT, 
                 0, oldOPCTimeout);
    viCheckErr( error);
    }
Error:
    return error;
}

/*****************************************************************************
 * Function: hp33120a_DefaultInstrSetup                                               
 * Purpose:  This function sends a default setup to the instrument.  The    
 *           hp33120a_reset function calls this function.  The 
 *           hp33120a_IviInit function calls this function when the
 *           user passes VI_FALSE for the reset parameter.  This function is 
 *           useful for configuring settings that other instrument driver 
 *           functions require.    
 *
 *           Note:  Call this function only when the session is locked.
 *****************************************************************************/
static ViStatus hp33120a_DefaultInstrSetup (ViSession vi)
{
    ViStatus error = VI_SUCCESS;
    ViInt32  index, wfmHandle, wfmSize, catBuffSize, scanCnt = 1;
    ViChar   wfmCatalog[BUFFER_SIZE], wfmName[HP33120A_VAL_NAME_LENGTH];
    
    /* Invalidate all attributes */
    checkErr( Ivi_InvalidateAllAttributes (vi));
    
    checkErr( hp33120a_CreateDriverWfmRecord (vi));
    
    if (!Ivi_Simulating(vi))
    {
    ViSession   io = Ivi_IOSession(vi); /* call only when locked */
    checkErr( Ivi_SetNeedToCheckStatus (vi, VI_TRUE));
    
    /* clear instrument */
    viCheckErr( viPrintf (io, "*CLS;*ESE 1;*SRE 32"));
    
    /* retrieve all existing arbitrary waveforms */
    viCheckErr( viPrintf (io, "DATA:CAT?"));
    viCheckErr( viScanf (io, "%#s", &catBuffSize, wfmCatalog));
    for (index = 0; scanCnt > 0; index++)
        {
        scanCnt = sscanf (wfmCatalog, "%*[,\"]%[^\"]%s", wfmName, wfmCatalog);
        if (scanCnt <= 0)
        break;
        viCheckErr( viPrintf (io, "DATA:ATTR:POIN? %s", wfmName));
        viCheckErr( viScanf  (io, "%d", &wfmSize));
        wfmHandle = HP33120A_WFM_HANDLE_FROM_INDEX(index);
        checkErr( hp33120a_UpdateDriverWfmRecord (vi, wfmHandle, wfmSize, wfmName));
        }
    /* set up default output */
    viCheckErr( viPrintf(io, ":FORM:BORD NORM;"));
    viCheckErr( viPrintf (io, "APPL:SIN 1000.0, 2.0, 0.00;"));
    }
Error:
    return error;
}

/*****************************************************************************
 * Function: ReadToFile and WriteFromFile Functions                          
 * Purpose:  Functions for instrument driver developers to read/write        
 *           instrument data to/from a file.                                 
 *****************************************************************************/
static ViStatus hp33120a_ReadToFile (ViSession vi, ViConstString filename, 
                     ViInt32 maxBytesToRead, ViInt32 fileAction, 
                     ViInt32 *totalBytesWritten)  
{   
    return Ivi_ReadToFile (vi, filename, maxBytesToRead, fileAction, totalBytesWritten);  
}   
static ViStatus hp33120a_WriteFromFile (ViSession vi, ViConstString filename, 
                    ViInt32 maxBytesToWrite, ViInt32 byteOffset, 
                    ViInt32 *totalBytesWritten) 
{   
    return Ivi_WriteFromFile (vi, filename, maxBytesToWrite, byteOffset, totalBytesWritten); 
}

/*****************************************************************************
 *------------------------ Global Session Callbacks -------------------------*
 *****************************************************************************/

/*****************************************************************************
 * Function: hp33120a_CheckStatusCallback                                               
 * Purpose:  This function queries the instrument to determine if it has 
 *           encountered an error.  If the instrument has encountered an 
 *           error, this function returns the IVI_ERROR_INSTRUMENT_SPECIFIC 
 *           error code.  This function is called by the 
 *           hp33120a_CheckStatus utility function.  The 
 *           Ivi_SetAttribute and Ivi_GetAttribute functions invoke this 
 *           function when the optionFlags parameter includes the
 *           IVI_VAL_DIRECT_USER_CALL flag.
 *           
 *           The user can disable calls to this function by setting the 
 *           IVI_ATTR_QUERY_INSTR_STATUS attribute to VI_FALSE.  The driver can 
 *           disable the check status callback for a particular attribute by 
 *           setting the IVI_VAL_DONT_CHECK_STATUS flag.
 *****************************************************************************/
static ViStatus _VI_FUNC hp33120a_CheckStatusCallback (ViSession vi, ViSession io)
{
    ViStatus    error = VI_SUCCESS;
    
    
    ViInt16     esr = 0; 
    viCheckErr( viWrite (io, "*ESR?", 5, VI_NULL));
    
    viCheckErr( viScanf (io, "%hd", &esr));
    
    if ((esr & IEEE_488_2_QUERY_ERROR_BIT) ||
    (esr & IEEE_488_2_DEVICE_DEPENDENT_ERROR_BIT) ||
    (esr & IEEE_488_2_EXECUTION_ERROR_BIT) ||
    (esr & IEEE_488_2_COMMAND_ERROR_BIT))
        {     
        viCheckErr( IVI_ERROR_INSTR_SPECIFIC);
        }
      
Error:
    return error;
}

/*****************************************************************************
 * Function: hp33120a_WaitForOPCCallback                                               
 * Purpose:  This function waits until the instrument has finished processing 
 *           all pending operations.  This function is called by the 
 *           hp33120a_WaitForOPC utility function.  The IVI engine invokes
 *           this function in the following two cases:
 *           - Before invoking the read callback for attributes for which the 
 *             IVI_VAL_WAIT_FOR_OPC_BEFORE_READS flag is set.
 *           - After invoking the write callback for attributes for which the 
 *             IVI_VAL_WAIT_FOR_OPC_AFTER_WRITES flag is set.
 *****************************************************************************/
static ViStatus _VI_FUNC hp33120a_WaitForOPCCallback (ViSession vi, ViSession io)
{
    ViStatus    error = VI_SUCCESS;
    ViInt32     opcTimeout;
    ViUInt16    statusByte;

    checkErr( Ivi_GetAttributeViInt32 (vi, VI_NULL, HP33120A_ATTR_OPC_TIMEOUT, 
                       0, &opcTimeout));

    viCheckErr( viEnableEvent (io, VI_EVENT_SERVICE_REQ, VI_QUEUE, VI_NULL));

    viCheckErr( viPrintf (io, "*OPC"));

    // wait for SRQ 
    viCheckErr( viWaitOnEvent (io, VI_EVENT_SERVICE_REQ, opcTimeout, VI_NULL, VI_NULL));
    viCheckErr( viDisableEvent (io, VI_EVENT_SERVICE_REQ, VI_QUEUE));

    // clean up after SRQ 
    viCheckErr( viBufWrite (io, "*CLS", 4, VI_NULL));
    viCheckErr( viReadSTB (io, &statusByte));
    
Error:
    viDiscardEvents (io, VI_EVENT_SERVICE_REQ, VI_QUEUE);

    return error;
}

/*****************************************************************************
 *----------------- Attribute Range Tables and Callbacks --------------------*
 *****************************************************************************/

/*- HP33120A_ATTR_ID_QUERY_RESPONSE -*/

static ViStatus _VI_FUNC hp33120aAttrIdQueryResponse_ReadCallback (ViSession vi, 
                                   ViSession io,
                                   ViConstString channelName, 
                                   ViAttr attributeId,
                                   const ViConstString cacheValue)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      rdBuffer[BUFFER_SIZE];
    ViUInt32    retCnt;
    
    viCheckErr( viPrintf (io, "*IDN?"));
    viCheckErr( viRead (io, rdBuffer, BUFFER_SIZE-1, &retCnt));
    rdBuffer[retCnt] = 0;

    checkErr( Ivi_SetValInStringCallback (vi, attributeId, rdBuffer));
    
Error:
    return error;
}
    
/*- HP33120A_ATTR_DRIVER_REVISION -*/

static ViStatus _VI_FUNC hp33120aAttrDriverRevision_ReadCallback (ViSession vi, 
                                  ViSession io,
                                  ViConstString channelName, 
                                  ViAttr attributeId,
                                  const ViConstString cacheValue)
{
    ViStatus    error = VI_NULL;
    ViChar      driverRevision[256];
    
    
    sprintf (driverRevision, 
         "Driver: hp33120a %.2f, Compiler: %s %3.2f, "
         "Components: IVIEngine %.2f, VISA-Spec %.2f",
         HP33120A_MAJOR_VERSION+HP33120A_MINOR_VERSION/1000.0,
         IVI_COMPILER_NAME, IVI_COMPILER_VER_NUM, IVI_ENGINE_MAJOR_VERSION+ 
         IVI_ENGINE_MINOR_VERSION/1000.0, Ivi_ConvertVISAVer(VI_SPEC_VERSION));

    checkErr( Ivi_SetValInStringCallback (vi, attributeId, driverRevision));    
Error:
    return error;
}

/*- HP33120A_ATTR_OUTPUT_MODE -*/

static IviRangeTableEntry attrOutputModeRangeTableEntries[] =
    {
    {HP33120A_VAL_OUTPUT_FUNC, 0,  0,  "", 0},
    {HP33120A_VAL_OUTPUT_ARB,  0,  0,  "",  0},
    {IVI_RANGE_TABLE_LAST_ENTRY}
    };
static IviRangeTable attrOutputModeRangeTable =
    {
    IVI_VAL_DISCRETE,
    VI_FALSE,
    VI_FALSE,
    VI_NULL,
    attrOutputModeRangeTableEntries,
    };

static ViStatus _VI_FUNC hp33120aAttrOutputMode_WriteCallback (ViSession vi,
                                   ViSession io,
                                   ViConstString channelName,
                                   ViAttr attributeId,
                                   ViInt32 value)
{
    ViStatus error = VI_SUCCESS;
    ViChar   *modeBuff[2] = {"SIN", "USER"};
    
    viCheckErr( viPrintf (io, "APPL:%s 1000.0, 2.0, 0.0;", modeBuff[(value == HP33120A_VAL_OUTPUT_ARB)]));
    
Error:
    return error;
}

static ViStatus _VI_FUNC hp33120aAttrOutputMode_CheckCallback (ViSession vi,
                                   ViConstString channelName,
                                   ViAttr attributeId,
                                   ViInt32 value)
{
    ViStatus error = VI_SUCCESS;
    ViBoolean modOn;
    
    checkErr( hp33120a_AnyModulationEnabled (vi, channelName, VI_NULL, &modOn));
    if (modOn)
    viCheckErrElab( IVI_ERROR_INVALID_CONFIGURATION,
            "Cannot change output mode while any modulation is enabled.");
    
    checkErr( Ivi_DefaultCheckCallbackViInt32 (vi, channelName, attributeId, value));
    
Error:
    return error;
}

/*- HP33120A_ATTR_REF_CLOCK_SOURCE -*/

static IviRangeTableEntry attrRefClockRangeTableEntries[] =
    {
    {HP33120A_VAL_INTERNAL,    0,  0,  "", 0},
    {IVI_RANGE_TABLE_LAST_ENTRY}
    };
static IviRangeTable attrRefClockRangeTable =
    {
    IVI_VAL_DISCRETE,
    VI_FALSE,
    VI_FALSE,
    VI_NULL,
    attrRefClockRangeTableEntries,
    };

/*- HP33120A_ATTR_OUTPUT_ENABLED -*/
static ViStatus _VI_FUNC hp33120aAttrOutputEnabled_CheckCallback (ViSession vi,
                                  ViConstString channelName,
                                  ViAttr attributeId,
                                  ViBoolean value)
{
    return (value ? VI_SUCCESS : IVI_ERROR_INVALID_VALUE);
}

/*- HP33120A_ATTR_OUTPUT_IMPEDANCE -*/
    
static IviRangeTableEntry attrOutputImpedanceRangeTableEntries[] =
    {
    {HP33120A_VAL_50_OHMS,   0,  0,  "MIN", 0},
    {HP33120A_VAL_INFINITE,  0,  0,  "MAX", 0},
    {IVI_RANGE_TABLE_LAST_ENTRY}  
    };
static IviRangeTable attrOutputImpedanceRangeTable = 
    {
    IVI_VAL_DISCRETE,
    VI_FALSE,
    VI_FALSE,
    VI_NULL,
    attrOutputImpedanceRangeTableEntries,
    };
    
static ViStatus _VI_FUNC hp33120aAttrOutputImpedance_WriteCallback (ViSession vi,
                                    ViSession io,
                                    ViConstString channelName,
                                    ViAttr attributeId,
                                    ViReal64 value)
{
    ViStatus error = VI_SUCCESS;
    ViString cmd;
    
    viCheckErr( Ivi_GetViReal64EntryFromValue (value, &attrOutputImpedanceRangeTable,
                           VI_NULL, VI_NULL, VI_NULL, VI_NULL,
                           &cmd, VI_NULL));

    viCheckErr (viPrintf (io, "OUTP:LOAD %s;", cmd));   
    
Error:
    return error;
}

static ViStatus _VI_FUNC hp33120aAttrOutputImpedance_ReadCallback (ViSession vi,
                                   ViSession io,
                                   ViConstString channelName,
                                   ViAttr attributeId,
                                   ViReal64 *value)
{
    ViStatus error = VI_SUCCESS;
    
    viCheckErr (viPrintf (io, "OUTP:LOAD?"));   
    viCheckErr (viScanf (io, "%Lf", value));   
    
Error:
    return error;
}

/*- HP33120A_ATTR_FUNC_WAVEFORM -*/

static IviRangeTableEntry attrFuncWaveformRangeTableEntries[] =
    {
    {HP33120A_VAL_WFM_SINE,        0,  0,  "SIN",       0},
    {HP33120A_VAL_WFM_SQUARE,      0,  0,  "SQU",       0},   
    {HP33120A_VAL_WFM_TRIANGLE,    0,  0,  "TRI",       0}, 
    {HP33120A_VAL_WFM_RAMP_UP,     0,  0,  "RAMP",      0},  
    {HP33120A_VAL_WFM_DC,          0,  0,  "DC",        0},
    {HP33120A_VAL_WFM_NOISE,       0,  0,  "NOISE",     0},
    {IVI_RANGE_TABLE_LAST_ENTRY}
    };
static IviRangeTable attrFuncWaveformRangeTable =
    {
    IVI_VAL_DISCRETE,
    VI_FALSE,
    VI_FALSE,
    VI_NULL,
    attrFuncWaveformRangeTableEntries,
    };
    
static ViStatus _VI_FUNC hp33120aAttrFuncWaveform_WriteCallback (ViSession vi,
                                 ViSession io,
                                 ViConstString channelName,
                                 ViAttr attributeId,
                                 ViInt32 value)
{
    ViStatus error = VI_SUCCESS;
    ViString    cmd;
    
    viCheckErr( Ivi_GetViInt32EntryFromValue (value, &attrFuncWaveformRangeTable,
                        VI_NULL, VI_NULL, VI_NULL, VI_NULL,
                        &cmd, VI_NULL));         

    checkErr( Ivi_SetAttributeViReal64 (vi, channelName, HP33120A_ATTR_FUNC_FREQUENCY,
                    IVI_VAL_DONT_MARK_AS_SET_BY_USER, 1000.00));
    checkErr( Ivi_SetAttributeViReal64 (vi, channelName, HP33120A_ATTR_FUNC_AMPLITUDE,
                    IVI_VAL_DONT_MARK_AS_SET_BY_USER, 2.0));
                        
    viCheckErr( viPrintf (io, "FUNC:SHAP %s;", cmd));
    
Error:
    return error;
}

static ViStatus _VI_FUNC hp33120aAttrFuncWaveform_ReadCallback (ViSession vi,
                                ViSession io,
                                ViConstString channelName,
                                ViAttr attributeId,
                                ViInt32 *value)
{
    ViStatus error = VI_SUCCESS;
    ViChar   rdBuffer[BUFFER_SIZE];
    ViInt32  rdBufferSize = sizeof(rdBuffer);

    checkErr (hp33120a_VerifyOutputMode (vi, HP33120A_VAL_OUTPUT_FUNC));
    
    viCheckErr( viPrintf (io, "FUNC:SHAP?", channelName));
    viCheckErr( viScanf (io, "%#s", &rdBufferSize, rdBuffer));

    viCheckErr( Ivi_GetViInt32EntryFromString (rdBuffer, &attrFuncWaveformRangeTable, 
                           value, VI_NULL, VI_NULL, VI_NULL, 
                           VI_NULL));

Error:
    return error;
}

static ViStatus _VI_FUNC hp33120aAttrFuncWaveform_CheckCallback (ViSession vi,
                                 ViConstString channelName,
                                 ViAttr attributeId,
                                 ViInt32 value)
{
    ViStatus         error = VI_SUCCESS;
    ViString         cmd;
    ViReal64         frequency;
    ViBoolean        modOn;
    IviRangeTablePtr tblPtr = VI_NULL;
    
    checkErr( hp33120a_VerifyOutputMode (vi, HP33120A_VAL_OUTPUT_FUNC));
    checkErr( hp33120a_AnyModulationEnabled (vi, channelName, VI_NULL, &modOn));
    if (modOn)
    viCheckErrElab( IVI_ERROR_INVALID_CONFIGURATION,
            "Cannot change waveform while any modulation is enabled.");
    
    checkErr( Ivi_DefaultCheckCallbackViInt32 (vi, channelName, attributeId, value));
    
Error:
    return error;
}

/*- HP33120A_ATTR_FUNC_AMPLITUDE -*/

static IviRangeTableEntry attrFuncAmpRangeTableEntries[] =
    {
    {0.05, 10.00, 0, "", 0},
    {IVI_RANGE_TABLE_LAST_ENTRY},
    };
static IviRangeTable attrFuncAmpRangeTable =
    {
    IVI_VAL_RANGED,
    VI_TRUE,
    VI_TRUE,
    VI_NULL,
    attrFuncAmpRangeTableEntries,
    };

static ViStatus _VI_FUNC hp33120aAttrFuncAmplitude_WriteCallback (ViSession vi,
                                  ViSession io,
                                  ViConstString channelName,
                                  ViAttr attributeId,
                                  ViReal64 value)
{
    ViStatus error = VI_SUCCESS;
    
    checkErr( Ivi_SetAttributeViReal64 (vi, channelName, HP33120A_ATTR_FUNC_DC_OFFSET,
                       IVI_VAL_DONT_MARK_AS_SET_BY_USER, 0.00));
    viCheckErr (viPrintf (io, "VOLT %Lf;", value));   
    
Error:
    return error;
}

static ViStatus _VI_FUNC hp33120aAttrFuncAmplitude_ReadCallback (ViSession vi,
                                 ViSession io,
                                 ViConstString channelName,
                                 ViAttr attributeId,
                                 ViReal64 *value)
{
    ViStatus  error = VI_SUCCESS;
    ViBoolean AMEnabled;
    ViReal64  amplitude, amAmpFix;
    
    checkErr( hp33120a_VerifyOutputMode (vi, HP33120A_VAL_OUTPUT_FUNC));
    checkErr( Ivi_GetAttributeViBoolean (vi, channelName, HP33120A_ATTR_AM_ENABLED,
                     0, &AMEnabled));
                         
    viCheckErr (viPrintf (io, "VOLT?"));   
    viCheckErr (viScanf (io, "%Lf", &amplitude));
    
    amAmpFix = (AMEnabled ? 2.0 : 1.0);
    * value = amplitude / amAmpFix;
    
Error:
    return error;
}

static ViStatus _VI_FUNC hp33120aAttrFuncAmplitude_CheckCallback (ViSession vi,
                                  ViConstString channelName,
                                  ViAttr attributeId,
                                  ViReal64 value)
{
    ViStatus  error = VI_SUCCESS;
    ViReal64  dcOffset, ampMax, outputImpedance, impScaler;
    ViBoolean modOn;
    
    checkErr( hp33120a_VerifyOutputMode (vi, HP33120A_VAL_OUTPUT_FUNC));
    checkErr( hp33120a_AnyModulationEnabled (vi, channelName, VI_NULL, &modOn));
    if (modOn)
    viCheckErrElab( IVI_ERROR_INVALID_CONFIGURATION,
            "Cannot change amplitude while any modulation is enabled.");
    
    checkErr( Ivi_GetAttributeViReal64 (vi, channelName, HP33120A_ATTR_OUTPUT_IMPEDANCE,
                    0, &outputImpedance));
    impScaler = ((outputImpedance == HP33120A_VAL_50_OHMS) ? 1.0 : 2.0);
    checkErr( Ivi_DefaultCheckCallbackViReal64 (vi, channelName, attributeId, (value / impScaler)));
                       
Error:
    return error;
}

/*- HP33120A_ATTR_FUNC_DC_OFFSET -*/

static IviRangeTableEntry attrFuncDCOffsetRangeTableEntries[] =
    {
    {-5.0, 5.0, 0, "", 0},
    {IVI_RANGE_TABLE_LAST_ENTRY}
    };
static IviRangeTable attrFuncDCOffsetRangeTable =
    {
    IVI_VAL_RANGED,
    VI_TRUE,
    VI_TRUE,
    VI_NULL,
    attrFuncDCOffsetRangeTableEntries,
    };

static ViStatus _VI_FUNC hp33120aAttrFuncDCOffset_WriteCallback (ViSession vi,
                                 ViSession io,
                                 ViConstString channelName,
                                 ViAttr attributeId,
                                 ViReal64 value)
{
    ViStatus error = VI_SUCCESS;
    
    viCheckErr (viPrintf (io, "VOLT:OFFS %Lf", value));   
    
Error:
    return error;
}

static ViStatus _VI_FUNC hp33120aAttrFuncDCOffset_ReadCallback (ViSession vi,
                                ViSession io,
                                ViConstString channelName,
                                ViAttr attributeId,
                                ViReal64 *value)
{
    ViStatus error = VI_SUCCESS;
    
    checkErr( hp33120a_VerifyOutputMode (vi, HP33120A_VAL_OUTPUT_FUNC));
    
    viCheckErr (viPrintf (io, "VOLT:OFFS?"));   
    viCheckErr (viScanf (io, "%Lf", value));   
    
Error:
    return error;
}

static ViStatus _VI_FUNC hp33120aAttrFuncDCOffset_CheckCallback (ViSession vi,
                                 ViConstString channelName,
                                 ViAttr attributeId,
                                 ViReal64 value)
{
    ViStatus error = VI_SUCCESS;
    ViReal64 amplitude, absMax, outputImpedance, impScaler;
    ViBoolean modOn;
    ViInt32 waveform;
    
    checkErr( hp33120a_VerifyOutputMode (vi, HP33120A_VAL_OUTPUT_FUNC));
    checkErr( hp33120a_AnyModulationEnabled (vi, channelName, VI_NULL, &modOn));
    if (modOn)
    viCheckErrElab( IVI_ERROR_INVALID_CONFIGURATION,
            "Cannot change DC offset while any modulation is enabled.");
    
    checkErr( Ivi_GetAttributeViReal64 (vi, VI_NULL, HP33120A_ATTR_OUTPUT_IMPEDANCE,
                    0, &outputImpedance));
    impScaler = ((outputImpedance == HP33120A_VAL_50_OHMS) ? 1.0 : 2.0);
    
    checkErr( Ivi_GetAttributeViInt32 (vi, channelName, HP33120A_ATTR_FUNC_WAVEFORM,
                       0, &waveform));
    checkErr( Ivi_DefaultCheckCallbackViReal64 (vi, channelName, attributeId, (value / impScaler)));
                        
    if (waveform != HP33120A_VAL_WFM_DC)
    {
    checkErr( Ivi_GetAttrMinMaxViReal64(vi, channelName, attributeId, VI_NULL,
                        &absMax, VI_NULL, VI_NULL));
    checkErr( Ivi_GetAttributeViReal64 (vi, channelName, HP33120A_ATTR_FUNC_AMPLITUDE,
                        0, &amplitude));
    absMax *= impScaler;
    
    if (((amplitude/2.0 + fabs(value)) > absMax) || (fabs(value) > (2*amplitude)))
        viCheckErrElab( IVI_ERROR_INVALID_CONFIGURATION,
               "The combination of DC Offset and Amplitude exceeds the output limit.");
    }
    
Error:
    return error;
}

/*- HP33120A_ATTR_FUNC_FREQUENCY -*/

static IviRangeTableEntry attrFuncDfltFreqRangeTableEntries[] = 
    {
    {100.00e-6, 15.00e+6, 0, "", 0},
    {IVI_RANGE_TABLE_LAST_ENTRY}
    };
static IviRangeTable attrFuncDfltFreqRangeTable =
    {
    IVI_VAL_RANGED,
    VI_TRUE,
    VI_TRUE,
    VI_NULL,
    attrFuncDfltFreqRangeTableEntries,
    };

static IviRangeTableEntry attrFuncTriRampFreqRangeTableEntries[] =
    {
    {100.00e-6, 100.00e+3, 0, "", 0},
    {IVI_RANGE_TABLE_LAST_ENTRY}
    };
static IviRangeTable attrFuncTriRampFreqRangeTable =
    {
    IVI_VAL_RANGED,
    VI_TRUE,
    VI_TRUE,
    VI_NULL,
    attrFuncTriRampFreqRangeTableEntries,
    };

static ViStatus _VI_FUNC hp33120aAttrFuncFrequency_WriteCallback (ViSession vi,
                                  ViSession io,
                                  ViConstString channelName,
                                  ViAttr attributeId,
                                  ViReal64 value)
{
    ViStatus error = VI_SUCCESS;
    
    checkErr( Ivi_SetAttributeViReal64 (vi, channelName, HP33120A_ATTR_FUNC_DUTY_CYCLE_HIGH,
                    IVI_VAL_DONT_MARK_AS_SET_BY_USER, 50.0));
    viCheckErr (viPrintf (io, "FREQ %Lf", value));   
    
Error:
    return error;
}

static ViStatus _VI_FUNC hp33120aAttrFuncFrequency_ReadCallback (ViSession vi,
                                 ViSession io,
                                 ViConstString channelName,
                                 ViAttr attributeId,
                                 ViReal64 *value)
{
    ViStatus error = VI_SUCCESS;
    
    checkErr( hp33120a_VerifyOutputMode (vi, HP33120A_VAL_OUTPUT_FUNC));

    viCheckErr (viPrintf (io, "FREQ?"));   
    viCheckErr (viScanf (io, "%Lf", value));   
    
Error:
    return error;
}

static ViStatus _VI_FUNC hp33120aAttrFuncFrequency_CheckCallback (ViSession vi,
                                  ViConstString channelName,
                                  ViAttr attributeId,
                                  ViReal64 value)
{
    ViStatus error = VI_SUCCESS;
    ViReal64 dutyCycleHigh;
    ViInt32  waveform;
    ViBoolean modOn;
    
    checkErr( hp33120a_VerifyOutputMode (vi, HP33120A_VAL_OUTPUT_FUNC));
    checkErr( hp33120a_AnyModulationEnabled (vi, channelName, VI_NULL, &modOn));
    if (modOn)
    viCheckErrElab( IVI_ERROR_INVALID_CONFIGURATION,
            "Cannot change frequency while any modulation is enabled.");
    
    checkErr( Ivi_DefaultCheckCallbackViReal64 (vi, channelName, attributeId, value));

Error:
    return error;
}

static ViStatus _VI_FUNC hp33120aAttrFuncFrequency_RangeTableCallback (ViSession vi,
                                       ViConstString channelName,
                                       ViAttr attributeId,
                                       IviRangeTablePtr *rangeTablePtr)    
{
    ViStatus         error = VI_SUCCESS;
    IviRangeTablePtr tblPtr = VI_NULL;
    ViInt32          waveform;
    
    checkErr( Ivi_GetAttributeViInt32 (vi, channelName, HP33120A_ATTR_FUNC_WAVEFORM,
                       0, &waveform));
                       
    switch (waveform)
    {
    case HP33120A_VAL_WFM_TRIANGLE:
    case HP33120A_VAL_WFM_RAMP_UP:
        tblPtr = &attrFuncTriRampFreqRangeTable;
        break;
    default:
        tblPtr = &attrFuncDfltFreqRangeTable;
        break;
    }
    
Error:
    *rangeTablePtr = tblPtr;
    return error;
}

/*- HP33120A_ATTR_FUNC_START_PHASE -*/

static IviRangeTableEntry attrFuncStartPhaseRangeTableEntries[] =
    {
    {0.00, 0.00, 0, "", 0},
    {IVI_RANGE_TABLE_LAST_ENTRY}
    };
static IviRangeTable attrFuncStartPhaseRangeTable =
    {
    IVI_VAL_RANGED,
    VI_TRUE,
    VI_TRUE,
    VI_NULL,
    attrFuncStartPhaseRangeTableEntries,
    };

/*- HP33120A_ATTR_FUNC_DUTY_CYCLE_HIGH -*/ 

static IviRangeTableEntry attrFuncDutyCycleLowFreqRangeTableEntries[] =
    {
    {20.00, 80.00, 0, "", 0},
    {IVI_RANGE_TABLE_LAST_ENTRY}
    };
static IviRangeTable attrFuncDutyCycleLowFreqRangeTable =
    {
    IVI_VAL_RANGED,
    VI_TRUE,
    VI_TRUE,
    VI_NULL,
    attrFuncDutyCycleLowFreqRangeTableEntries,
    };

static IviRangeTableEntry attrFuncDutyCycleHighFreqRangeTableEntries[] =
    {
    {40.00, 60.00, 0, "", 0},
    {IVI_RANGE_TABLE_LAST_ENTRY}
    };
static IviRangeTable attrFuncDutyCycleHighFreqRangeTable =
    {
    IVI_VAL_RANGED,
    VI_TRUE,
    VI_TRUE,
    VI_NULL,
    attrFuncDutyCycleHighFreqRangeTableEntries,
    };
    
static ViStatus _VI_FUNC hp33120aAttrFuncDutyCycleHigh_WriteCallback (ViSession vi,
                                      ViSession io,
                                      ViConstString channelName,
                                      ViAttr attributeId,
                                      ViReal64 value)
{
    ViStatus error = VI_SUCCESS;
    
    viCheckErr (viPrintf (io, "PULS:DCYCLE %Lf", value));   
    
Error:
    return error;
}

static ViStatus _VI_FUNC hp33120aAttrFuncDutyCycleHigh_ReadCallback (ViSession vi,
                                     ViSession io,
                                     ViConstString channelName,
                                     ViAttr attributeId,
                                     ViReal64 *value)
{
    ViStatus error = VI_SUCCESS;
    
    checkErr( hp33120a_VerifyOutputMode (vi, HP33120A_VAL_OUTPUT_FUNC));

    viCheckErr (viPrintf (io, "PULS:DCYCLE?"));   
    viCheckErr (viScanf (io, "%Lf", value));   
    
Error:
    return error;
}

static ViStatus _VI_FUNC hp33120aAttrFuncDutyCycleHigh_CheckCallback (ViSession vi,
                                      ViConstString channelName,
                                      ViAttr attributeId,
                                      ViReal64 value)
{
    ViStatus  error = VI_SUCCESS;
    ViBoolean modOn;
    
    checkErr( hp33120a_VerifyOutputMode (vi, HP33120A_VAL_OUTPUT_FUNC));
    checkErr( hp33120a_AnyModulationEnabled (vi, channelName, VI_NULL, &modOn));
    if (modOn)
    viCheckErrElab( IVI_ERROR_INVALID_CONFIGURATION,
            "Cannot change Duty Cycle while any modulation is enabled.");
    
    checkErr( Ivi_DefaultCheckCallbackViInt32 (vi, channelName, attributeId, value));
    
Error:
    return error;
}

static ViStatus _VI_FUNC hp33120aAttrFuncDutyCycleHigh_RangeTableCallback (ViSession vi,
                                       ViConstString channelName,
                                       ViAttr attributeId,
                                       IviRangeTablePtr *rangeTablePtr)
{
    ViStatus error = VI_SUCCESS;
    ViReal64 frequency;
    
    checkErr( Ivi_GetAttributeViReal64 (vi, channelName, HP33120A_ATTR_FUNC_FREQUENCY,
                    0, &frequency));
    if (frequency <= 5.0e+6)
    *rangeTablePtr = &attrFuncDutyCycleLowFreqRangeTable;
    else
    *rangeTablePtr = &attrFuncDutyCycleHighFreqRangeTable;
    
Error:
    return error;
}

/*- HP33120A_ATTR_ARB_WAVEFORM_HANDLE -*/

static ViStatus _VI_FUNC hp33120aAttrArbWaveformHandle_WriteCallback (ViSession vi,
                                      ViSession io,
                                      ViConstString channelName,
                                      ViAttr attributeId,
                                      ViInt32 value)
{
    ViStatus error = VI_SUCCESS;
    ViChar wfmName[HP33120A_VAL_NAME_LENGTH];
    
    checkErr( Ivi_SetAttributeViReal64 (vi, channelName, HP33120A_ATTR_ARB_GAIN,
                    IVI_VAL_DONT_MARK_AS_SET_BY_USER, 1.0));
    checkErr( hp33120a_GetWfmNameFromHandleUtil (vi, value, HP33120A_VAL_NAME_LENGTH, wfmName));
    viCheckErr( viPrintf (io, "FREQ 1000.0;:FUNC:USER %s", wfmName));

Error:
    return error;
}

static ViStatus _VI_FUNC hp33120aAttrArbWaveformHandle_ReadCallback (ViSession vi,
                                     ViSession io,
                                     ViConstString channelName,
                                     ViAttr attributeId,
                                     ViInt32 *value)
{
    ViStatus error = VI_SUCCESS;
    ViChar   rdBuffer[BUFFER_SIZE];
    ViInt32  rdBufferSize = sizeof(rdBuffer);
    
    checkErr( hp33120a_VerifyOutputMode (vi, HP33120A_VAL_OUTPUT_ARB));
    
    viCheckErr( viPrintf (io, "FUNC:USER?"));
    viCheckErr( viScanf (io, "%#s", &rdBufferSize, rdBuffer));
    checkErr( hp33120a_GetWfmHandleFromNameUtil (vi, rdBuffer, value));
    
Error:
    return error;
}

static ViStatus _VI_FUNC hp33120aAttrArbWaveformHandle_CheckCallback (ViSession vi,
                                      ViConstString channelName,
                                      ViAttr attributeId,
                                      ViInt32 value)
{
    ViStatus  error = VI_SUCCESS;
    ViBoolean modOn, wfmExists;
    ViReal64  equivFreq;
    ViInt32   wfmSize;
    
    checkErr( hp33120a_VerifyOutputMode (vi, HP33120A_VAL_OUTPUT_ARB));
    checkErr( hp33120a_AnyModulationEnabled (vi, channelName, VI_NULL, &modOn));
    if (modOn)
    viCheckErrElab( IVI_ERROR_INVALID_CONFIGURATION,
            "Cannot change arbitrary waveform handle while any modulation is enabled.");
    
    checkErr( hp33120a_WfmExists (vi, value, &wfmExists));
    if (!wfmExists)
    viCheckErr( HP33120A_ERROR_INVALID_WAVEFORM);
    
Error:
    return error;
}

/*- HP33120A_ATTR_ARB_WAVEFORM_GAIN -*/

static IviRangeTableEntry attrArbGainRangeTableEntries[] =
    {
    {0.025, 5.00, 0, "", 0},
    {IVI_RANGE_TABLE_LAST_ENTRY}
    };
static IviRangeTable attrArbGainRangeTable =
    {
    IVI_VAL_RANGED,
    VI_TRUE,
    VI_TRUE,
    VI_NULL,
    attrArbGainRangeTableEntries,
    };

static ViStatus _VI_FUNC hp33120aAttrArbGain_WriteCallback (ViSession vi,
                                ViSession io,
                                ViConstString channelName,
                                ViAttr attributeId,
                                ViReal64 value)
{   
    ViStatus error = VI_SUCCESS;
    
    checkErr( Ivi_SetAttributeViReal64 (vi, channelName, HP33120A_ATTR_ARB_OFFSET,
                    IVI_VAL_DONT_MARK_AS_SET_BY_USER, 0.00));

    viCheckErr( viPrintf (io, "VOLT %Lf;", (value * 2.0)));                                                                                              

Error:
    return error;
}

static ViStatus _VI_FUNC hp33120aAttrArbGain_ReadCallback (ViSession vi,
                               ViSession io,
                               ViConstString channelName,
                               ViAttr attributeId,
                               ViReal64 *value)
{
    ViStatus error = VI_SUCCESS;
    ViReal64 gain, amAmpFix;
    ViBoolean AMEnabled;
    
    checkErr( hp33120a_VerifyOutputMode (vi, HP33120A_VAL_OUTPUT_ARB));
    checkErr( Ivi_GetAttributeViBoolean (vi, channelName, HP33120A_ATTR_AM_ENABLED,
                    0, &AMEnabled));
                    
    viCheckErr( viPrintf (io, "VOLT?"));
    viCheckErr( viScanf (io, "%Lf", &gain));
    
    amAmpFix = (AMEnabled ? 4.0 : 2.0);
    *value = gain / amAmpFix;
    
Error:
    return error;
}

static ViStatus _VI_FUNC hp33120aAttrArbGain_CheckCallback (ViSession vi,
                                ViConstString channelName,
                                ViAttr attributeId,
                                ViReal64 value)
{
    ViStatus  error = VI_SUCCESS;
    ViReal64  arbOffset, ampMax, outputImpedance, impScaler;
    ViBoolean modOn;
    
    checkErr( hp33120a_VerifyOutputMode (vi, HP33120A_VAL_OUTPUT_ARB));
    checkErr( hp33120a_AnyModulationEnabled (vi, channelName, VI_NULL, &modOn));
    if (modOn)
    viCheckErrElab( IVI_ERROR_INVALID_CONFIGURATION,
            "Cannot change arbitrary gain while any modulation is enabled.");
    
    checkErr( Ivi_GetAttributeViReal64 (vi, channelName, HP33120A_ATTR_OUTPUT_IMPEDANCE,
                    0, &outputImpedance));
    impScaler = ((outputImpedance == HP33120A_VAL_50_OHMS) ? 1.0 : 2.0);
    
    checkErr( Ivi_DefaultCheckCallbackViReal64 (vi, channelName, attributeId, (value / impScaler)));

Error:
    return error;
}

/*- HP33120A_ATTR_ARB_WAVEFORM_OFFSET -*/

static IviRangeTableEntry attrArbOffsetRangeTableEntries[] = 
    {
    {-5.00, 5.00, 0, "", 0},
    {IVI_RANGE_TABLE_LAST_ENTRY}
    };
static IviRangeTable attrArbOffsetRangeTable = 
    {
    IVI_VAL_RANGED,
    VI_TRUE,
    VI_TRUE,
    VI_NULL,
    attrArbOffsetRangeTableEntries,
    };

static ViStatus _VI_FUNC hp33120aAttrArbOffset_WriteCallback (ViSession vi,
                                  ViSession io,
                                  ViConstString channelName,
                                  ViAttr attributeId,
                                  ViReal64 value)
{
    ViStatus error = VI_SUCCESS;
    
    viCheckErr( viPrintf (io, "VOLT:OFFS %Lf", value));                                                                                              

Error:
    return error;
}

static ViStatus _VI_FUNC hp33120aAttrArbOffset_ReadCallback (ViSession vi,
                                 ViSession io,
                                 ViConstString channelName,
                                 ViAttr attributeId,
                                 ViReal64 *value)
{
    ViStatus error = VI_SUCCESS;
    
    checkErr( hp33120a_VerifyOutputMode (vi, HP33120A_VAL_OUTPUT_ARB));
    
    viCheckErr( viPrintf (io, "VOLT:OFFS?"));
    viCheckErr( viScanf (io, "%Lf", value));
    
Error:
    return error;
}

static ViStatus _VI_FUNC hp33120aAttrArbOffset_CheckCallback (ViSession vi,
                                  ViConstString channelName,
                                  ViAttr attributeId,
                                  ViReal64 value)
{
    ViStatus error = VI_SUCCESS;
    ViReal64 arbGain, absMax, outputImpedance, impScaler;
    ViBoolean modOn;
    
    checkErr( hp33120a_VerifyOutputMode (vi, HP33120A_VAL_OUTPUT_ARB));
    checkErr( hp33120a_AnyModulationEnabled (vi, channelName, VI_NULL, &modOn));
    if (modOn)
    viCheckErrElab( IVI_ERROR_INVALID_CONFIGURATION,
            "Cannot change arbitrary offset while any modulation is enabled.");
    
    checkErr( Ivi_GetAttributeViReal64 (vi, channelName, HP33120A_ATTR_OUTPUT_IMPEDANCE,
                    0, &outputImpedance));
    impScaler = ((outputImpedance == HP33120A_VAL_50_OHMS) ? 1.0 : 2.0);
    
    checkErr( Ivi_DefaultCheckCallbackViReal64 (vi, channelName, attributeId, (value / impScaler)));
    
    checkErr( Ivi_GetAttrMinMaxViReal64(vi, channelName, attributeId, VI_NULL,
                    &absMax, VI_NULL, VI_NULL));
    checkErr( Ivi_GetAttributeViReal64 (vi, channelName, HP33120A_ATTR_ARB_GAIN,
                    0, &arbGain));
    absMax *= impScaler;
    
    if ((((arbGain + fabs(value)) > absMax)) || (fabs(arbGain) > (4*arbGain)))
    viCheckErrElab( IVI_ERROR_INVALID_CONFIGURATION,
               "The combination of Arbitrary Offset and Arbitrary Gain exceeds the output limits.");

Error:
    return error;
}

/*- HP33120A_ATTR_ARB_SAMPLE_RATE -*/

static ViStatus _VI_FUNC hp33120aAttrArbSampleRate_WriteCallback (ViSession vi,
                                  ViSession io,
                                  ViConstString channelName,
                                  ViAttr attributeId,
                                  ViReal64 value)
{
    ViStatus  error = VI_SUCCESS;
    ViInt32   arbWfmHandle, wfmSize;
    ViReal64  equivFreq;
    ViBoolean exists;    
    checkErr( Ivi_GetAttributeViInt32 (vi, channelName, HP33120A_ATTR_ARB_WAVEFORM_HANDLE,
                       0, &arbWfmHandle));
    
    checkErr( hp33120a_WfmExists (vi, arbWfmHandle, &exists));
    if (!exists)
    {
    viCheckErrElab( HP33120A_ERROR_INVALID_WAVEFORM,
            "You must specify an arbitrary waveform to generate"
            "before configuring the arbitrary waveform's sample rate.");
            
    }
    checkErr( hp33120a_GetWfmSize (vi, arbWfmHandle, &wfmSize));
    
    equivFreq = (value / (ViReal64)wfmSize);
    
    viCheckErr( viPrintf (io, "FREQ %Lf", equivFreq));
    
Error:
    return error;
}

static ViStatus _VI_FUNC hp33120aAttrArbSampleRate_ReadCallback (ViSession vi,
                                 ViSession io,
                                 ViConstString channelName,
                                 ViAttr attributeId,
                                 ViReal64 *value)
{
    ViStatus  error = VI_SUCCESS;
    ViInt32   arbWfmHandle, wfmSize;
    ViReal64  equivFreq;
    
    checkErr( hp33120a_VerifyOutputMode (vi, HP33120A_VAL_OUTPUT_ARB));
    
    viCheckErr( viPrintf(io, "FREQ?"));
    viCheckErr( viScanf(io, "%Lf", &equivFreq));
    
    checkErr( Ivi_GetAttributeViInt32 (vi, channelName, HP33120A_ATTR_ARB_WAVEFORM_HANDLE,
                       0, &arbWfmHandle));
    checkErr( hp33120a_GetWfmSize (vi, arbWfmHandle, &wfmSize));
    
    *value = (equivFreq * wfmSize);
    
Error:
    return error;
}

static ViStatus _VI_FUNC hp33120aAttrArbSampleRate_CheckCallback (ViSession vi,
                                  ViConstString channelName,
                                  ViAttr attributeId,
                                  ViReal64 value)
{
    ViStatus error = VI_SUCCESS;
    ViInt32 arbWfmHandle, wfmSize;
    ViReal64 equivFreq;
    ViBoolean modOn;
    
    checkErr( hp33120a_VerifyOutputMode (vi, HP33120A_VAL_OUTPUT_ARB));
    checkErr( hp33120a_AnyModulationEnabled (vi, channelName, VI_NULL, &modOn));
    if (modOn)
    viCheckErrElab( IVI_ERROR_INVALID_CONFIGURATION,
          "Cannot change sample rate while any modulation is enabled.");
    
    checkErr( Ivi_GetAttributeViInt32 (vi, channelName, HP33120A_ATTR_ARB_WAVEFORM_HANDLE,
                       0, &arbWfmHandle));
    checkErr( hp33120a_GetWfmSize (vi, arbWfmHandle, &wfmSize));
    
    equivFreq = (value / (ViReal64)wfmSize);
    
    if ((equivFreq < 100.0e-6) || (equivFreq > 5.0e+6))
    viCheckErr( IVI_ERROR_INVALID_VALUE);
    else if ((wfmSize > 12287) && (equivFreq > 200.0e+3))
    viCheckErr( IVI_ERROR_INVALID_VALUE);
    else if ((wfmSize > 8192) && (equivFreq > 2.5e+6)) 
    viCheckErr( IVI_ERROR_INVALID_VALUE);
        
Error:
    return error;
}

/*- HP33120A_ATTR_AM_ENABLED -*/

static ViStatus _VI_FUNC hp33120aAttrAMEnabled_WriteCallback (ViSession vi,
                                  ViSession io,
                                  ViConstString channelName,
                                  ViAttr attributeId,
                                  ViBoolean value)
{
    ViStatus error = VI_SUCCESS;

    viCheckErr( viPrintf (io, "AM:STAT %d", value));

Error:
    return error;
}

static ViStatus _VI_FUNC hp33120aAttrAMEnabled_ReadCallback (ViSession vi,
                                 ViSession io,
                                 ViConstString channelName,
                                 ViAttr attributeId,
                                 ViBoolean *value)
{
    ViStatus error = VI_SUCCESS;
    
    viCheckErr( viPrintf (io, "AM:STAT?"));
    viCheckErr( viScanf (io, "%d", value));

Error:
    return error;
}

static ViStatus _VI_FUNC hp33120aAttrAMEnabled_CheckCallback (ViSession vi,
                                  ViConstString channelName,
                                  ViAttr attributeId,
                                  ViBoolean value)
{
    ViStatus error = VI_SUCCESS;
    ViInt32  waveform;
    ViInt32  outputMode;
    ViBoolean modOn;
    
    if (value)
    {
    checkErr( hp33120a_AnyModulationEnabled (vi, channelName, HP33120A_VAL_AM,
                         &modOn));
    if (modOn)
        viCheckErrElab( IVI_ERROR_INVALID_CONFIGURATION, "Cannot Enable AM while any other modulation is enabled.");
    
    checkErr( Ivi_GetAttributeViInt32 (vi, VI_NULL, HP33120A_ATTR_OUTPUT_MODE,
                       0, &outputMode));
    if (outputMode == HP33120A_VAL_OUTPUT_FUNC)
        {
        checkErr( Ivi_GetAttributeViInt32 (vi, channelName, HP33120A_ATTR_FUNC_WAVEFORM,
                           0, &waveform));
        if ((waveform == HP33120A_VAL_WFM_DC) || (waveform == HP33120A_VAL_WFM_NOISE))
        viCheckErrElab( IVI_ERROR_INVALID_CONFIGURATION,
                "Cannot use current waveform as AM carrier signal.");
        }
    }
    
Error:
    return error;
}

/*- HP33120A_ATTR_AM_SOURCE -*/

static IviRangeTableEntry attrAmSourceRangeTableEntries[] = 
    {
    {HP33120A_VAL_BOTH,     0, 0, "BOTH", 0},
    {HP33120A_VAL_EXTERNAL, 0, 0, "EXT",  0},
    {IVI_RANGE_TABLE_LAST_ENTRY}
    };
static IviRangeTable attrAmSourceRangeTable = 
    {
    IVI_VAL_DISCRETE,
    VI_FALSE,
    VI_FALSE,
    VI_NULL,
    attrAmSourceRangeTableEntries,
    };

static ViStatus _VI_FUNC hp33120aAttrAMSource_WriteCallback (ViSession vi,
                                 ViSession io,
                                 ViConstString channelName,
                                 ViAttr attributeId,
                                 ViInt32 value)
{
    ViStatus error = VI_SUCCESS;

    ViString cmd;
    
    viCheckErr( Ivi_GetViInt32EntryFromValue (value, &attrAmSourceRangeTable,
                        VI_NULL, VI_NULL, VI_NULL, VI_NULL,
                        &cmd, VI_NULL));         
    viCheckErr( viPrintf (io, "AM:SOUR %s", cmd));
    
Error:
    return error;
}

static ViStatus _VI_FUNC hp33120aAttrAMSource_ReadCallback (ViSession vi,
                                ViSession io,
                                ViConstString channelName,
                                ViAttr attributeId,
                                ViInt32 *value)
{
    ViStatus error = VI_SUCCESS;
    ViChar   rdBuffer[BUFFER_SIZE];
    ViInt32  rdBufferSize = sizeof(rdBuffer);
    
    viCheckErr( viPrintf (io, "AM:SOUR?"));
    viCheckErr( viScanf (io, "%#s", &rdBufferSize, rdBuffer));

    viCheckErr( Ivi_GetViInt32EntryFromString (rdBuffer, &attrAmSourceRangeTable, 
                           value, VI_NULL, VI_NULL, VI_NULL, 
                           VI_NULL));

Error:
    return error;
}

/*- HP33120A_ATTR_AM_INTERNAL_DEPTH -*/

static IviRangeTableEntry attrAmInternalDepthRangeTableEntries[] =
    {
    {0.00, 120.00, 0, "", 0},
    {IVI_RANGE_TABLE_LAST_ENTRY}
    };
static IviRangeTable attrAmInternalDepthRangeTable =
    {
    IVI_VAL_RANGED,
    VI_TRUE,
    VI_TRUE,
    VI_NULL,
    attrAmInternalDepthRangeTableEntries,
    };

static ViStatus _VI_FUNC hp33120aAttrAMInternalDepth_WriteCallback (ViSession vi,
                                    ViSession io,
                                    ViConstString channelName,
                                    ViAttr attributeId,
                                    ViReal64 value)
{
    ViStatus error = VI_SUCCESS;
    
    viCheckErr( viPrintf (io, "AM:DEPT %Lf", value));
    
Error:
    return error;
}

static ViStatus _VI_FUNC hp33120aAttrAMInternalDepth_ReadCallback (ViSession vi,
                                   ViSession io,
                                   ViConstString channelName,
                                   ViAttr attributeId,
                                   ViReal64 *value)
{
    ViStatus error = VI_SUCCESS;
    
    viCheckErr( viPrintf (io, "AM:DEPT?"));
    viCheckErr( viScanf (io, "%Lf", value));
    
Error:
    return error;
}

/*- HP33120A_ATTR_AM_INTERNAL_WAVEFORM -*/

static IviRangeTableEntry attrAMInternalWaveformRangeTableEntries[] =
    {
    {HP33120A_VAL_WFM_SINE,         0, 0, "SIN",  0},
    {HP33120A_VAL_WFM_SQUARE,       0, 0, "SQU",  0},
    {HP33120A_VAL_WFM_TRIANGLE,     0, 0, "TRI",  0},
    {HP33120A_VAL_WFM_RAMP_UP,      0, 0, "RAMP", 0},
    {HP33120A_VAL_WFM_NOISE,        0, 0, "NOISE",  0},
    {HP33120A_VAL_WFM_ARBITRARY,    0, 0, "USER",  0},
    {IVI_RANGE_TABLE_LAST_ENTRY}
    };
static IviRangeTable attrAMInternalWaveformRangeTable =
    {
    IVI_VAL_DISCRETE,
    VI_FALSE,
    VI_FALSE,
    VI_NULL,
    attrAMInternalWaveformRangeTableEntries,
    };
    
static ViStatus _VI_FUNC hp33120aAttrAMInternalWaveform_WriteCallback (ViSession vi,
                                       ViSession io,
                                       ViConstString channelName,
                                       ViAttr attributeId,
                                       ViInt32 value)
{
    ViStatus error = VI_SUCCESS;
    ViString cmd;
    
    viCheckErr( Ivi_GetViInt32EntryFromValue (value, &attrAMInternalWaveformRangeTable,
                        VI_NULL, VI_NULL, VI_NULL, VI_NULL,
                        &cmd, VI_NULL));         
    viCheckErr( viPrintf (io, "AM:INT:FUNC %s", cmd));
    
Error:
    return error;
}

static ViStatus _VI_FUNC hp33120aAttrAMInternalWaveform_ReadCallback (ViSession vi,
                                      ViSession io,
                                      ViConstString channelName,
                                      ViAttr attributeId,
                                      ViInt32 *value)
{
    ViStatus  error = VI_SUCCESS;
    ViChar    rdBuffer[BUFFER_SIZE];
    ViInt32   rdBufferSize = sizeof(rdBuffer);

    viCheckErr( viPrintf (io, "AM:INT:FUNC?"));
    viCheckErr( viScanf (io, "%#s", &rdBufferSize, rdBuffer));

    viCheckErr( Ivi_GetViInt32EntryFromString (rdBuffer, &attrAMInternalWaveformRangeTable, 
                           value, VI_NULL, VI_NULL, VI_NULL, 
                           VI_NULL));    

Error:
    return error;
}

/*- HP33120A_ATTR_AM_INTERNAL_FREQUENCY -*/

static IviRangeTableEntry attrAmInternalFrequencyRangeTableEntries[] =
    {
    {10.0e-3, 20.0e+3, 0, "", 0},
    {IVI_RANGE_TABLE_LAST_ENTRY}
    };
static IviRangeTable attrAmInternalFrequencyRangeTable =
    {
    IVI_VAL_RANGED,
    VI_TRUE,
    VI_TRUE,
    VI_NULL,
    attrAmInternalFrequencyRangeTableEntries,
    };
    
static ViStatus _VI_FUNC hp33120aAttrAMInternalFrequency_WriteCallback (ViSession vi,
                                    ViSession io,
                                    ViConstString channelName,
                                    ViAttr attributeId,
                                    ViReal64 value)
{
    ViStatus error = VI_SUCCESS;
    
    viCheckErr( viPrintf (io, "AM:INT:FREQ %Lf", value));
    
Error:
    return error;
}

static ViStatus _VI_FUNC hp33120aAttrAMInternalFrequency_ReadCallback (ViSession vi,
                                       ViSession io,
                                       ViConstString channelName,
                                       ViAttr attributeId,
                                       ViReal64 *value)
{
    ViStatus error = VI_SUCCESS;
    
    viCheckErr( viPrintf (io, "AM:INT:FREQ?"));
    viCheckErr( viScanf (io, "%Lf", value));
    
Error:
    return error;
}

/*- HP33120A_ATTR_FM_ENABLED -*/

static ViStatus _VI_FUNC hp33120aAttrFMEnabled_WriteCallback (ViSession vi,
                                  ViSession io,
                                  ViConstString channelName,
                                  ViAttr attributeId,
                                  ViBoolean value)
{
    ViStatus error = VI_SUCCESS;

    if (value)
    {
    checkErr( Ivi_SetAttributeViReal64 (vi, VI_NULL, HP33120A_ATTR_FM_INTERNAL_DEVIATION,
                        IVI_VAL_DONT_MARK_AS_SET_BY_USER, 1000.00));
    }
    
    viCheckErr( viPrintf (io, "FM:STAT %d", value));
    
Error:
    return error;
}

static ViStatus _VI_FUNC hp33120aAttrFMEnabled_ReadCallback (ViSession vi,
                                 ViSession io,
                                 ViConstString channelName,
                                 ViAttr attributeId,
                                 ViBoolean *value)
{
    ViStatus error = VI_SUCCESS;
    
    viCheckErr( viPrintf (io, "FM:STAT?", channelName));
    viCheckErr( viScanf (io, "%d", value));

Error:
    return error;
}

static ViStatus _VI_FUNC hp33120aAttrFMEnabled_CheckCallback (ViSession vi,
                                  ViConstString channelName,
                                  ViAttr attributeId,
                                  ViBoolean value)
{
    ViStatus error = VI_SUCCESS;
    ViInt32  waveform, outputMode;
    ViReal64 frequency;
    ViBoolean modOn;
    
    if (value)
    {
    checkErr( hp33120a_AnyModulationEnabled (vi, channelName, HP33120A_VAL_FM,
                         &modOn));
    if (modOn)
        viCheckErrElab( IVI_ERROR_INVALID_CONFIGURATION, 
                "Cannot Enable FM while any other modulation is enabled.");
    
    checkErr( Ivi_GetAttributeViInt32 (vi, VI_NULL, HP33120A_ATTR_OUTPUT_MODE,
                       0, &outputMode));
    if (outputMode == HP33120A_VAL_OUTPUT_FUNC)
        {
        checkErr( Ivi_GetAttributeViInt32 (vi, channelName, HP33120A_ATTR_FUNC_WAVEFORM,
                           0, &waveform));
        if ((waveform == HP33120A_VAL_WFM_DC) || (waveform == HP33120A_VAL_WFM_NOISE))
        viCheckErrElab( IVI_ERROR_INVALID_CONFIGURATION,
                "Cannot use current waveform as FM carrier signal.");
        }
    }

Error:
    return error;
}

/*- HP33120A_ATTR_FM_SOURCE -*/

static IviRangeTableEntry attrFmSourceRangeTableEntries[] = 
    {
    {HP33120A_VAL_INTERNAL, 0, 0, "", 0},
    {IVI_RANGE_TABLE_LAST_ENTRY}
    };
static IviRangeTable attrFmSourceRangeTable = 
    {
    IVI_VAL_DISCRETE,
    VI_FALSE,
    VI_FALSE,
    VI_NULL,
    attrFmSourceRangeTableEntries,
    };

/*- HP33120A_ATTR_FM_INTERNAL_DEVIATION -*/

static ViStatus _VI_FUNC hp33120aAttrFMInternalDeviation_WriteCallback (ViSession vi,
                                    ViSession io,
                                    ViConstString channelName,
                                    ViAttr attributeId,
                                    ViReal64 value)
{
    ViStatus error = VI_SUCCESS;

    viCheckErr( viPrintf (io, "FM:DEV %Lf", value));
    
Error:
    return error;
}

static ViStatus _VI_FUNC hp33120aAttrFMInternalDeviation_ReadCallback (ViSession vi,
                                       ViSession io,
                                       ViConstString channelName,
                                       ViAttr attributeId,
                                       ViReal64 *value)
{
    ViStatus error = VI_SUCCESS;
    
    viCheckErr( viPrintf (io, "FM:DEV?"));
    viCheckErr( viScanf (io, "%Lf", value));
    
Error:
    return error;
}

static ViStatus _VI_FUNC hp33120aAttrFMInternalDeviation_CheckCallback (ViSession vi,
                                    ViConstString channelName,
                                    ViAttr attributeId,
                                    ViReal64 value)
{
    ViStatus error = VI_SUCCESS;
    ViReal64 frequency, maxFreq;
    ViBoolean FMEnabled;
    
    checkErr( Ivi_GetAttributeViBoolean (vi, channelName, HP33120A_ATTR_FM_ENABLED,
                     0, &FMEnabled));
    if (!FMEnabled)
    viCheckErrElab( IVI_ERROR_INVALID_CONFIGURATION,
            "Cannot set deviation frequency unless FM is enabled.");
    
    checkErr( hp33120a_GetEquivalentFrequency (vi, channelName, &frequency, &maxFreq));
    
    if ((value < 10.0e-3) || (value > 7.5e+6)) 
    viCheckErr( IVI_ERROR_INVALID_VALUE);
    if ((value > frequency) || ((frequency + value) > (maxFreq + 100.0e+3)))
    viCheckErrElab( IVI_ERROR_INVALID_CONFIGURATION,
            "Combination of deviation frequency and carrier signal frequency exceeds frequency limits.");
            
Error:
    return error;
}

/*- HP33120A_ATTR_FM_INTERNAL_WAVEFORM -*/

static IviRangeTableEntry attrFMInternalWaveformRangeTableEntries[] =
    {
    {HP33120A_VAL_WFM_SINE,         0, 0, "SIN",  0},
    {HP33120A_VAL_WFM_SQUARE,       0, 0, "SQU",  0},
    {HP33120A_VAL_WFM_TRIANGLE,     0, 0, "TRI",  0},
    {HP33120A_VAL_WFM_RAMP_UP,      0, 0, "RAMP", 0},
    {HP33120A_VAL_WFM_NOISE,        0, 0, "NOISE",  0},
    {HP33120A_VAL_WFM_ARBITRARY,    0, 0, "USER",  0},
    {IVI_RANGE_TABLE_LAST_ENTRY}
    };
static IviRangeTable attrFMInternalWaveformRangeTable =
    {
    IVI_VAL_DISCRETE,
    VI_FALSE,
    VI_FALSE,
    VI_NULL,
    attrFMInternalWaveformRangeTableEntries,
    };
    

static ViStatus _VI_FUNC hp33120aAttrFMInternalWaveform_WriteCallback (ViSession vi,
                                       ViSession io,
                                       ViConstString channelName,
                                       ViAttr attributeId,
                                       ViInt32 value)
{
    ViStatus error = VI_SUCCESS;
    ViString cmd;
    
    viCheckErr( Ivi_GetViInt32EntryFromValue (value, &attrFMInternalWaveformRangeTable,
                        VI_NULL, VI_NULL, VI_NULL, VI_NULL,
                        &cmd, VI_NULL));         
    checkErr( Ivi_SetAttributeViReal64 (vi, VI_NULL, HP33120A_ATTR_FM_INTERNAL_DEVIATION,
                    IVI_VAL_DONT_MARK_AS_SET_BY_USER, 1000.00));
                    
    viCheckErr( viPrintf (io, "FM:INT:FUNC %s", cmd));
    
Error:
    return error;
}
    
static ViStatus _VI_FUNC hp33120aAttrFMInternalWaveform_ReadCallback (ViSession vi,
                                      ViSession io,
                                      ViConstString channelName,
                                      ViAttr attributeId,
                                      ViInt32 *value)
{
    ViStatus  error = VI_SUCCESS;
    ViChar    rdBuffer[BUFFER_SIZE];
    ViInt32   rdBufferSize = sizeof(rdBuffer);

    viCheckErr( viPrintf (io, "FM:INT:FUNC?"));
    viCheckErr( viScanf (io, "%#s", &rdBufferSize, rdBuffer));

    viCheckErr( Ivi_GetViInt32EntryFromString (rdBuffer, &attrFMInternalWaveformRangeTable, 
                           value, VI_NULL, VI_NULL, VI_NULL, 
                           VI_NULL));    

Error:
    return error;
}

/*- HP33120A_ATTR_FM_INTERNAL_FREQUENCY -*/

static IviRangeTableEntry attrFmInternalFrequencyRangeTableEntries[] =
    {
    {10.00e-3, 10.0e+3, 0, "", 0},
    {IVI_RANGE_TABLE_LAST_ENTRY}
    };
static IviRangeTable attrFmInternalFrequencyRangeTable =
    {
    IVI_VAL_RANGED,
    VI_TRUE,
    VI_TRUE,
    VI_NULL,
    attrFmInternalFrequencyRangeTableEntries,
    };

static ViStatus _VI_FUNC hp33120aAttrFMInternalFrequency_WriteCallback (ViSession vi,
                                    ViSession io,
                                    ViConstString channelName,
                                    ViAttr attributeId,
                                    ViReal64 value)
{
    ViStatus error = VI_SUCCESS;
    
    viCheckErr( viPrintf (io, "FM:INT:FREQ %Lf", value));
    
Error:
    return error;
}

static ViStatus _VI_FUNC hp33120aAttrFMInternalFrequency_ReadCallback (ViSession vi,
                                       ViSession io,
                                       ViConstString channelName,
                                       ViAttr attributeId,
                                       ViReal64 *value)
{
    ViStatus error = VI_SUCCESS;
    
    viCheckErr( viPrintf (io, "FM:INT:FREQ?"));
    viCheckErr( viScanf (io, "%Lf", value));
    
Error:
    return error;
}

/*- HP33120A_ATTR_FSK_ENABLED -*/

static ViStatus _VI_FUNC hp33120aAttrFSKEnabled_WriteCallback (ViSession vi,
                                   ViSession io,
                                   ViConstString channelName,
                                   ViAttr attributeId,
                                   ViBoolean value)
{
    ViStatus error = VI_SUCCESS;
    
    viCheckErr( viPrintf (io, "FSK:INT:RATE 1000.00;:FSK:STAT %d", value));
    
Error:
    return error;
}

static ViStatus _VI_FUNC hp33120aAttrFSKEnabled_ReadCallback (ViSession vi,
                                  ViSession io,
                                  ViConstString channelName,
                                  ViAttr attributeId,
                                  ViBoolean *value)
{
    ViStatus error = VI_SUCCESS;
    
    viCheckErr( viPrintf (io, "FSK:STAT?"));
    viCheckErr( viScanf (io, "%d", value));

Error:
    return error;
}

static ViStatus _VI_FUNC hp33120aAttrFSKEnabled_CheckCallback (ViSession vi,
                                   ViConstString channelName,
                                   ViAttr attributeId,
                                   ViBoolean value)
{
    ViStatus error = VI_SUCCESS;
    ViInt32  waveform;
    ViInt32  outputMode;
    ViBoolean modOn;
    
    if (value)
    {
    checkErr( hp33120a_AnyModulationEnabled (vi, channelName, HP33120A_VAL_FSK,
                         &modOn));
    if (modOn)
        viCheckErrElab( IVI_ERROR_INVALID_CONFIGURATION,
            "Cannot Enable FSK while any other modulation is enabled.");
    
    checkErr( Ivi_GetAttributeViInt32 (vi, VI_NULL, HP33120A_ATTR_OUTPUT_MODE,
                       0, &outputMode));
    if (outputMode == HP33120A_VAL_OUTPUT_FUNC)
        {
        checkErr( Ivi_GetAttributeViInt32 (vi, channelName, HP33120A_ATTR_FUNC_WAVEFORM,
                           0, &waveform));
        if ((waveform == HP33120A_VAL_WFM_DC) || (waveform == HP33120A_VAL_WFM_NOISE))
        viCheckErrElab( IVI_ERROR_INVALID_CONFIGURATION,
                "Cannot use current waveform as FSK carrier signal.");
        }
    }
    
Error:
    return error;
}

/*- HP33120A_ATTR_FSK_SOURCE -*/

static IviRangeTableEntry attrFSKSourceRangeTableEntries[] = 
    {
    {HP33120A_VAL_INTERNAL, 0, 0, "INT", 0},
    {HP33120A_VAL_EXTERNAL, 0, 0, "EXT",  0},
    {IVI_RANGE_TABLE_LAST_ENTRY}
    };
static IviRangeTable attrFSKSourceRangeTable = 
    {
    IVI_VAL_DISCRETE,
    VI_FALSE,
    VI_FALSE,
    VI_NULL,
    attrFSKSourceRangeTableEntries,
    };

static ViStatus _VI_FUNC hp33120aAttrFSKSource_WriteCallback (ViSession vi,
                                  ViSession io,
                                  ViConstString channelName,
                                  ViAttr attributeId,
                                  ViInt32 value)
{
    ViStatus error = VI_SUCCESS;
    ViString cmd;
    
    viCheckErr( Ivi_GetViInt32EntryFromValue (value, &attrFSKSourceRangeTable,
                        VI_NULL, VI_NULL, VI_NULL, VI_NULL,
                        &cmd, VI_NULL));         
    viCheckErr( viPrintf (io, "FSK:SOUR %s", cmd));
    
Error:
    return error;
}

static ViStatus _VI_FUNC hp33120aAttrFSKSource_ReadCallback (ViSession vi,
                                 ViSession io,
                                 ViConstString channelName,
                                 ViAttr attributeId,
                                 ViInt32 *value)
{
    ViStatus error = VI_SUCCESS;
    ViChar   rdBuffer[BUFFER_SIZE];
    ViInt32  rdBufferSize = sizeof(rdBuffer);
    
    viCheckErr( viPrintf (io, "FSK:SOUR?"));
    viCheckErr( viScanf (io, "%#s", &rdBufferSize, rdBuffer));

    viCheckErr( Ivi_GetViInt32EntryFromString (rdBuffer, &attrAmSourceRangeTable, 
                           value, VI_NULL, VI_NULL, VI_NULL, 
                           VI_NULL));

Error:
    return error;
}

/*- HP33120A_ATTR_FSK_INTERNAL_RATE -*/

static IviRangeTableEntry attrFSKInternalRateRangeTableEntries[] =
    {
    {0.01, 50.0e+3, 0, "", 0},
    {IVI_RANGE_TABLE_LAST_ENTRY}
    };
static IviRangeTable attrFSKInternalRateRangeTable =
    {
    IVI_VAL_RANGED,
    VI_TRUE,
    VI_TRUE,
    VI_NULL,
    attrFSKInternalRateRangeTableEntries,
    };
    
static ViStatus _VI_FUNC hp33120aAttrFSKInternalRate_WriteCallback (ViSession vi,
                                    ViSession io,
                                    ViConstString channelName,
                                    ViAttr attributeId,
                                    ViReal64 value)
{
    ViStatus error = VI_SUCCESS;
    
    viCheckErr( viPrintf (io, "FSK:INT:RATE %Lf", value));
    
Error:
    return error;
}

static ViStatus _VI_FUNC hp33120aAttrFSKInternalRate_ReadCallback (ViSession vi,
                                   ViSession io,
                                   ViConstString channelName,
                                   ViAttr attributeId,
                                   ViReal64 *value)
{
    ViStatus error = VI_SUCCESS;
    
    viCheckErr( viPrintf (io, "FSK:INT:RATE?"));
    viCheckErr( viScanf (io, "%Lf", value));
    
Error:
    return error;
}

/*- HP33120A_ATTR_FSK_HOP_FREQUENCY -*/

static ViStatus _VI_FUNC hp33120aAttrFSKHopFreq_WriteCallback (ViSession vi,
                                   ViSession io,
                                   ViConstString channelName,
                                   ViAttr attributeId,
                                   ViReal64 value)
{
    ViStatus error = VI_SUCCESS;
    
    viCheckErr( viPrintf (io, "FSK:FREQ %Lf", value));
    
Error:
    return error;
}

static ViStatus _VI_FUNC hp33120aAttrFSKHopFreq_ReadCallback (ViSession vi,
                                  ViSession io,
                                  ViConstString channelName,
                                  ViAttr attributeId,
                                  ViReal64 *value)
{
    ViStatus error = VI_SUCCESS;
    
    viCheckErr( viPrintf (io, "FSK:FREQ?"));
    viCheckErr( viScanf (io, "%Lf", value));
    
Error:
    return error;
}

static ViStatus _VI_FUNC hp33120aAttrFSKHopFreq_CheckCallback (ViSession vi,
                                   ViConstString channelName,
                                   ViAttr attributeId,
                                   ViReal64 value)
{
    ViStatus error = VI_SUCCESS;
    ViBoolean FSKEnabled;
    ViReal64 frequency, maxFreq;
    
    checkErr( Ivi_GetAttributeViBoolean (vi, channelName, HP33120A_ATTR_FSK_ENABLED,
                     0, &FSKEnabled));
    if (!FSKEnabled)
    viCheckErrElab( IVI_ERROR_INVALID_CONFIGURATION,
            "Cannot set FSK hop frequency unless FSK modulation is enabled.");
    
    checkErr( hp33120a_GetEquivalentFrequency (vi, channelName, &frequency, &maxFreq));
    if ((value > maxFreq) || (value < 0.01))
    viCheckErr (IVI_ERROR_INVALID_VALUE);
    
Error:
    return error;
}

/*- HP33120A_ATTR_SWEEP_ENABLED -*/
static ViStatus _VI_FUNC hp33120aAttrSweepEnabled_WriteCallback (ViSession vi,
                                 ViSession io,
                                 ViConstString channelName,
                                 ViAttr attributeId,
                                 ViBoolean value)
{
    ViStatus error = VI_SUCCESS;

    viCheckErr( viPrintf (io, "FREQ:STAR 1000;:FREQ:STOP 1000;:SWE:STAT %d", value));

Error:
    return error;
}

static ViStatus _VI_FUNC hp33120aAttrSweepEnabled_ReadCallback (ViSession vi,
                                ViSession io,
                                ViConstString channelName,
                                ViAttr attributeId,
                                ViBoolean *value)
{
    ViStatus error = VI_SUCCESS;
    
    viCheckErr( viPrintf (io, "SWE:STAT?"));
    viCheckErr( viScanf (io, "%d", value));

Error:
    return error;
}

static ViStatus _VI_FUNC hp33120aAttrSweepEnabled_CheckCallback (ViSession vi,
                                 ViConstString channelName,
                                 ViAttr attributeId,
                                 ViBoolean value)
{
    ViStatus error = VI_SUCCESS;
    ViInt32  waveform;
    ViInt32  outputMode;
    ViBoolean modOn;
    
    if (value)
    {
    checkErr( hp33120a_AnyModulationEnabled (vi, channelName, HP33120A_VAL_SWEEP,
                         &modOn));
    if (modOn)
        viCheckErrElab( IVI_ERROR_INVALID_CONFIGURATION,
            "Cannot Enable Sweep while any other modulation is enabled.");
    
    checkErr( Ivi_GetAttributeViInt32 (vi, VI_NULL, HP33120A_ATTR_OUTPUT_MODE,
                       0, &outputMode));
    if (outputMode == HP33120A_VAL_OUTPUT_FUNC)
        {
        checkErr( Ivi_GetAttributeViInt32 (vi, channelName, HP33120A_ATTR_FUNC_WAVEFORM,
                           0, &waveform));
        if ((waveform == HP33120A_VAL_WFM_DC) || (waveform == HP33120A_VAL_WFM_NOISE))
        viCheckErrElab( IVI_ERROR_INVALID_CONFIGURATION,
                "Cannot use current waveform as Sweep modulation carrier signal.");
        }
    }
    
Error:
    return error;
}

/*- HP33120A_ATTR_SWEEP_START_FREQ -*/

static ViStatus _VI_FUNC hp33120aAttrSweepStartFreq_WriteCallback (ViSession vi,
                                   ViSession io,
                                   ViConstString channelName,
                                   ViAttr attributeId,
                                   ViReal64 value)
{
    ViStatus error = VI_SUCCESS;
    
    viCheckErr( viPrintf (io, "FREQ:STAR %Lf", value));
    
Error:
    return error;
}

static ViStatus _VI_FUNC hp33120aAttrSweepStartFreq_ReadCallback (ViSession vi,
                                  ViSession io,
                                  ViConstString channelName,
                                  ViAttr attributeId,
                                  ViReal64 *value)
{
    ViStatus error = VI_SUCCESS;
    
    viCheckErr( viPrintf (io, "FREQ:STAR?"));
    viCheckErr( viScanf  (io, "%Lf"));
    
Error:
    return error;
}

static ViStatus _VI_FUNC hp33120aAttrSweepStartFreq_CheckCallback (ViSession vi,
                                   ViConstString channelName,
                                   ViAttr attributeId,
                                   ViReal64 value)
{
    ViStatus error = VI_SUCCESS;
    ViBoolean sweepEnabled;
    ViReal64 frequency, maxFreq;
    
    checkErr( Ivi_GetAttributeViBoolean (vi, channelName, HP33120A_ATTR_SWEEP_ENABLED,
                     0, &sweepEnabled));
    if (!sweepEnabled)
    viCheckErrElab( IVI_ERROR_INVALID_CONFIGURATION,
            "Cannot set start frequency unless sweep modulation is enabled.");
    
    checkErr( hp33120a_GetEquivalentFrequency (vi, channelName, &frequency, &maxFreq));
    if ((value > maxFreq) || (value < 0.01))
    viCheckErr (IVI_ERROR_INVALID_VALUE);
    
Error:
    return error;
}

/*- HP33120A_ATTR_SWEEP_STOP_FREQ -*/

static ViStatus _VI_FUNC hp33120aAttrSweepStopFreq_WriteCallback (ViSession vi,
                                  ViSession io,
                                  ViConstString channelName,
                                  ViAttr attributeId,
                                  ViReal64 value)
{
    ViStatus error = VI_SUCCESS;
    
    viCheckErr( viPrintf (io, "FREQ:STOP %Lf", value));
    
Error:
    return error;
}

static ViStatus _VI_FUNC hp33120aAttrSweepStopFreq_ReadCallback (ViSession vi,
                                 ViSession io,
                                 ViConstString channelName,
                                 ViAttr attributeId,
                                 ViReal64 *value)
{
    ViStatus error = VI_SUCCESS;
    
    viCheckErr( viPrintf (io, "FREQ:STOP?"));
    viCheckErr( viScanf  (io, "%Lf"));
    
Error:
    return error;
}

static ViStatus _VI_FUNC hp33120aAttrSweepStopFreq_CheckCallback (ViSession vi,
                                  ViConstString channelName,
                                  ViAttr attributeId,
                                  ViReal64 value)
{
    ViStatus error = VI_SUCCESS;
    ViBoolean sweepEnabled;
    ViReal64 frequency, maxFreq;
    
    checkErr( Ivi_GetAttributeViBoolean (vi, channelName, HP33120A_ATTR_SWEEP_ENABLED,
                     0, &sweepEnabled));
    if (!sweepEnabled)
    viCheckErrElab( IVI_ERROR_INVALID_CONFIGURATION,
            "Cannot set stop frequency unless sweep modulation is enabled.");
    
    checkErr( hp33120a_GetEquivalentFrequency (vi, channelName, &frequency, &maxFreq));
    if ((value > maxFreq) || (value < 0.01))
    viCheckErr (IVI_ERROR_INVALID_VALUE);
    
Error:
    return error;
}

/*- HP33120A_ATTR_SWEEP_SPACING -*/     

static IviRangeTableEntry attrSweepSpacingRangeTableEntries[] =
    {
    {HP33120A_VAL_LINEAR,      0, 0, "LIN", 0},
    {HP33120A_VAL_LOGARITHMIC, 0, 0, "LOG", 0}
    };
static IviRangeTable attrSweepSpacingRangeTable =
    {
    IVI_VAL_DISCRETE,
    VI_FALSE,
    VI_FALSE,
    VI_NULL,
    attrSweepSpacingRangeTableEntries,
    };
    
static ViStatus _VI_FUNC hp33120aAttrSweepSpacing_WriteCallback (ViSession vi,
                                 ViSession io,
                                 ViConstString channelName,
                                 ViAttr attributeId,
                                 ViInt32 value)
{
    ViStatus error = VI_SUCCESS;
    ViString cmd;
    
    viCheckErr( Ivi_GetViInt32EntryFromValue (value, &attrSweepSpacingRangeTable,
                        VI_NULL, VI_NULL, VI_NULL, VI_NULL,
                        &cmd, VI_NULL));
    viCheckErr( viPrintf (io, "SWE:SPAC %s", cmd));
    
Error:
    return error;
}

static ViStatus _VI_FUNC hp33120aAttrSweepSpacing_ReadCallback (ViSession vi,
                                ViSession io,
                                ViConstString channelName,
                                ViAttr attributeId,
                                ViInt32 *value)
{
    ViStatus error = VI_SUCCESS;
    ViChar   rdBuffer[BUFFER_SIZE];
    ViInt32  rdBufferSize = sizeof (rdBuffer);
    
    viCheckErr( viPrintf (io, "SWE:SPAC?"));
    viCheckErr( viScanf (io, "%#s", &rdBufferSize, rdBuffer));
    
    viCheckErr( Ivi_GetViInt32EntryFromString (rdBuffer, &attrSweepSpacingRangeTable,
                           value, VI_NULL, VI_NULL, VI_NULL,
                           VI_NULL));

Error:
    return error;
}

/*- HP33120A_ATTR_SWEEP_TIME -*/     
static IviRangeTableEntry attrSweepTimeRangeTableEntries[] =
    {
    {0.1, 500.0, 0, "", 0},
    {IVI_RANGE_TABLE_LAST_ENTRY},
    };
static IviRangeTable attrSweepTimeRangeTable =
    {
    IVI_VAL_RANGED,
    VI_TRUE,
    VI_TRUE,
    VI_NULL,
    attrSweepTimeRangeTableEntries,
    };

static ViStatus _VI_FUNC hp33120aAttrSweepTime_WriteCallback (ViSession vi,
                                  ViSession io,
                                  ViConstString channelName,
                                  ViAttr attributeId,
                                  ViReal64 value)
{
    ViStatus error = VI_SUCCESS;
    
    viCheckErr( viPrintf (io, "SWE:TIME %Lf", value));

Error:
    return error;
}

static ViStatus _VI_FUNC hp33120aAttrSweepTime_ReadCallback (ViSession vi,
                                 ViSession io,
                                 ViConstString channelName,
                                 ViAttr attributeId,
                                 ViReal64 *value)
{
    ViStatus error = VI_SUCCESS;
    
    viCheckErr( viPrintf (io, "SWE:TIME?"));
    viCheckErr( viScanf (io, "%Lf", value));
    
Error:
    return error;
}

/*- HP33120A_ATTR_SWEEP_SOURCE -*/    

static IviRangeTableEntry attrSweepSourceRangeTableEntries[] =
    {
    {HP33120A_VAL_IMMEDIATE,    0, 0, "IMM", 0},
    {HP33120A_VAL_EXTERNAL,     0, 0, "EXT", 0},
    {HP33120A_VAL_SW_TRIG_FUNC, 0, 0, "BUS", 0}
    };
static IviRangeTable attrSweepSourceRangeTable =
    {
    IVI_VAL_DISCRETE,
    VI_FALSE,
    VI_FALSE,
    VI_NULL,
    attrSweepSourceRangeTableEntries,
    };
    
static ViStatus _VI_FUNC hp33120aAttrSweepSource_WriteCallback (ViSession vi,
                                ViSession io,
                                ViConstString channelName,
                                ViAttr attributeId,
                                ViInt32 value)
{
    ViStatus error = VI_SUCCESS;
    ViString cmd;
    
    viCheckErr( Ivi_GetViInt32EntryFromValue (value, &attrSweepSourceRangeTable,
                        VI_NULL, VI_NULL, VI_NULL, VI_NULL,
                        &cmd, VI_NULL));
    viCheckErr( viPrintf (io, "TRIG:SOUR %s", cmd));
    
Error:
    return error;
}

static ViStatus _VI_FUNC hp33120aAttrSweepSource_ReadCallback (ViSession vi,
                                   ViSession io,
                                   ViConstString channelName,
                                   ViAttr attributeId,
                                   ViInt32 *value)
{
    ViStatus error = VI_SUCCESS;
    ViChar   rdBuffer[BUFFER_SIZE];
    ViInt32  rdBufferSize = sizeof(rdBuffer);
    
    viCheckErr( viPrintf (io, "TRIG:SOUR?"));
    viCheckErr( viScanf (io, "%#s", &rdBufferSize, rdBuffer));
    
    viCheckErr( Ivi_GetViInt32EntryFromString (rdBuffer, &attrSweepSpacingRangeTable,
                           value, VI_NULL, VI_NULL, VI_NULL,
                           VI_NULL));

Error:
    return error;
}
    
/*- HP33120A_ATTR_BURST_ENABLED -*/

static ViStatus _VI_FUNC hp33120aAttrBurstEnabled_WriteCallback (ViSession vi,
                                 ViSession io,
                                 ViConstString channelName,
                                 ViAttr attributeId,
                                 ViBoolean value)
{
    ViStatus error = VI_SUCCESS;
    
    viCheckErr( viPrintf( io, "BM:INT:RATE 1000.0;:BM:STAT %d", value));
    
Error:
    return error;
}

static ViStatus _VI_FUNC hp33120aAttrBurstEnabled_ReadCallback (ViSession vi,
                                ViSession io,
                                ViConstString channelName,
                                ViAttr attributeId,
                                ViBoolean *value)
{
    ViStatus error = VI_SUCCESS;
    
    viCheckErr( viPrintf( io, "BM:STAT?"));
    viCheckErr( viScanf (io, "%d", value));
    
Error:
    return error;
}

static ViStatus _VI_FUNC hp33120aAttrBurstEnabled_CheckCallback (ViSession vi,
                                 ViConstString channelName,
                                 ViAttr attributeId,
                                 ViBoolean value)
{
    ViStatus error = VI_SUCCESS;
    ViInt32 outputMode, waveform;
    ViBoolean modOn;
    
    if (value)
    {
    checkErr( hp33120a_AnyModulationEnabled (vi, channelName, HP33120A_VAL_BURST,
                         &modOn));
    if (modOn)
        viCheckErrElab( IVI_ERROR_INVALID_CONFIGURATION,
                "Cannot Enable Burst while any other modulation is enabled.");
    
    checkErr( Ivi_GetAttributeViInt32 (vi, VI_NULL, HP33120A_ATTR_OUTPUT_MODE,
                       0, &outputMode));
    if (outputMode == HP33120A_VAL_OUTPUT_FUNC)
        {
        checkErr( Ivi_GetAttributeViInt32 (vi, channelName, HP33120A_ATTR_FUNC_WAVEFORM,
                           0, &waveform));
        if ((waveform == HP33120A_VAL_WFM_DC) || (waveform == HP33120A_VAL_WFM_NOISE))
        viCheckErrElab( IVI_ERROR_INVALID_CONFIGURATION,
                "Cannot use current waveform as carrier signal for burst modulation.");
        }
    }

Error:
    return error;
}

/*- HP33120A_ATTR_BURST_TRIGGER -*/

static IviRangeTableEntry attrBurstSourceRangeTableEntries[] =
    {
    {HP33120A_VAL_INTERNAL,      0, 0, "IMM:INT", 0},
    {HP33120A_VAL_EXTERNAL,      0, 0, "IMM:EXT", 0},
    {HP33120A_VAL_EXTERNAL_GATE, 0, 0, "EXT:INT", 0}
    };
static IviRangeTable attrBurstSourceRangeTable =
    {
    IVI_VAL_DISCRETE,
    VI_FALSE,
    VI_FALSE,
    VI_NULL,
    attrBurstSourceRangeTableEntries,
    };
    
static ViStatus _VI_FUNC hp33120aAttrBurstSource_WriteCallback (ViSession vi,
                                ViSession io,
                                ViConstString channelName,
                                ViAttr attributeId,
                                ViInt32 value)
{
    ViStatus error = VI_SUCCESS;
    ViString cmd;
    ViChar   cmd1[4], cmd2[4];
    
    viCheckErr( Ivi_GetViInt32EntryFromValue (value, &attrBurstSourceRangeTable, VI_NULL,
                        VI_NULL, VI_NULL, VI_NULL, &cmd,
                        VI_NULL));
    sscanf(cmd, "%3s:%3s", cmd1, cmd2);
    
    viCheckErr (viPrintf (io, "TRIG:SOUR %s;:BM:SOUR %s", cmd1, cmd2));
    
Error:
    return error;
}

static ViStatus _VI_FUNC hp33120aAttrBurstSource_ReadCallback (ViSession vi,
                                   ViSession io,
                                   ViConstString channelName,
                                   ViAttr attributeId,
                                   ViInt32 *value)
{
    ViStatus error = VI_SUCCESS;
    ViChar   rdBuffer[BUFFER_SIZE], cmd1[4], cmd2[4];
    ViChar   *charPoint;
    
    ViInt32  rdBufferSize = sizeof(rdBuffer);
    
    viCheckErr( viPrintf (io, "TRIG:SOUR?;:BM:SOUR?"));
    viCheckErr( viScanf (io, "%#s", rdBuffer));
    charPoint = memchr (rdBuffer, ';', rdBufferSize);
    *charPoint = ':';
    
    viCheckErr( Ivi_GetViInt32EntryFromString (rdBuffer, &attrBurstSourceRangeTable,
                           value, VI_NULL, VI_NULL, VI_NULL,
                           VI_NULL));

Error:
    return error;
}

/*- HP33120A_ATTR_BURST_INTERNAL_RATE -*/

static IviRangeTableEntry attrBurstRateRangeTableEntries[] =
    {
    {0.01, 50.0e+3, 0, "", 0},
    {IVI_RANGE_TABLE_LAST_ENTRY},
    };
static IviRangeTable attrBurstRateRangeTable =
    {
    IVI_VAL_RANGED,
    VI_TRUE,
    VI_TRUE,
    VI_NULL,
    attrBurstRateRangeTableEntries
    };
    
static ViStatus _VI_FUNC hp33120aAttrBurstRate_WriteCallback (ViSession vi,
                                  ViSession io,
                                  ViConstString channelName,
                                  ViAttr attributeId,
                                  ViReal64 value)
{
    ViStatus error = VI_SUCCESS;
    
    viCheckErr( viPrintf (io, "BM:INT:RATE %Lf", value));
    
Error:
    return error;
}

static ViStatus _VI_FUNC hp33120aAttrBurstRate_ReadCallback (ViSession vi,
                                 ViSession io,
                                 ViConstString channelName,
                                 ViAttr attributeId,
                                 ViReal64 *value)
{
    ViStatus error = VI_SUCCESS;
    
    viCheckErr( viPrintf (io, "BM:INT:RATE?"));
    viCheckErr( viPrintf (io, "%Lf", value));
    
Error:
    return error;
}

/*- HP33120A_ATTR_BURST_PHASE -*/

static IviRangeTableEntry attrBurstPhaseRangeTableEntries [] =
    {
    {-360.00, 360.00, 0, "", 0},
    {IVI_RANGE_TABLE_LAST_ENTRY},
    };
static IviRangeTable attrBurstPhaseRangeTable =
    {
    IVI_VAL_RANGED,
    VI_TRUE,
    VI_TRUE,
    VI_NULL,
    attrBurstPhaseRangeTableEntries
    };
    
static ViStatus _VI_FUNC hp33120aAttrBurstPhase_WriteCallback (ViSession vi,
                                   ViSession io,
                                   ViConstString channelName,
                                   ViAttr attributeId,
                                   ViReal64 value)
{
    ViStatus error = VI_SUCCESS;
    
    viCheckErr( viPrintf (io, "BM:PHAS %Lf", value));
  
Error:
    return error;
}

static ViStatus _VI_FUNC hp33120aAttrBurstPhase_ReadCallback (ViSession vi,
                                  ViSession io,
                                  ViConstString channelName,
                                  ViAttr attributeId,
                                  ViReal64 *value)
{
    ViStatus error = VI_SUCCESS;
    
    viCheckErr( viPrintf (io, "BM:PHAS?"));
    viCheckErr( viScanf (io, "%Lf", value));
    
Error:
    return error;
}

/*- HP33120A_ATTR_BURST_COUNT -*/

static ViStatus _VI_FUNC hp33120aAttrBurstCount_WriteCallback (ViSession vi,
                                   ViSession io,
                                   ViConstString channelName,
                                   ViAttr attributeId,
                                   ViInt32 value)
{
    ViStatus error = VI_SUCCESS;
    
    if (value == HP33120A_VAL_GENERATE_CONTINUOUS)
    viCheckErr( viPrintf (io, "BM:NCYC INF"));
    else
    viCheckErr( viPrintf (io, "BM:NCYC %d", value));
 
Error:
    return error;
}

static ViStatus _VI_FUNC hp33120aAttrBurstCount_ReadCallback (ViSession vi,
                                  ViSession io,
                                  ViConstString channelName,
                                  ViAttr attributeId,
                                  ViInt32 *value)
{
    ViStatus error = VI_SUCCESS;
    
    viCheckErr( viPrintf (io, "BM:NCYC?"));
    viCheckErr( viScanf (io, "%d", value));
    
Error:
    return error;
}

static ViStatus _VI_FUNC hp33120aAttrBurstCount_CheckCallback (ViSession vi,
                                   ViConstString channelName,
                                   ViAttr attributeId,
                                   ViInt32 value)
{
    ViStatus error = VI_SUCCESS;
    ViBoolean burstEnabled;
    ViReal64 frequency;
    ViInt32  minCnt, maxCnt;
    
    checkErr( Ivi_GetAttributeViBoolean (vi, channelName, HP33120A_ATTR_BURST_ENABLED,
                     0, &burstEnabled));
    if (!burstEnabled)
    viCheckErrElab( IVI_ERROR_INVALID_CONFIGURATION,
            "Cannot set burst count unless burst modulation is enabled.");
    
    if (value != HP33120A_VAL_GENERATE_CONTINUOUS)
    {
    checkErr( hp33120a_GetEquivalentFrequency (vi, channelName, &frequency, VI_NULL));
    
    minCnt = (ViInt32 )(ceil(frequency / 1.0e+6));
    if (frequency < 100.0)
        maxCnt = (ViInt32)(floor (frequency * 500));
    else
        maxCnt = 50000;
    
    if ((value < minCnt) || (value > maxCnt))
        viCheckErr( IVI_ERROR_INVALID_VALUE);
    }
Error:
    return error;
}

    
/*****************************************************************************
 * Function: hp33120a_InitAttributes                                                  
 * Purpose:  This function adds attributes to the IVI session, initializes   
 *           instrument attributes, and sets attribute invalidation          
 *           dependencies.                                                   
 *****************************************************************************/
static ViStatus hp33120a_InitAttributes (ViSession vi)
{
    ViStatus    error = 0;

    /*- Initialize instrument attributes --------------------------------*/            

    checkErr( Ivi_SetAttributeViInt32 (vi, VI_NULL, HP33120A_ATTR_DRIVER_MAJOR_VERSION, 
                       0, HP33120A_MAJOR_VERSION));                
    checkErr( Ivi_SetAttributeViInt32 (vi, VI_NULL, HP33120A_ATTR_DRIVER_MINOR_VERSION, 
                       0, HP33120A_MINOR_VERSION));                
    checkErr( Ivi_SetAttrReadCallbackViString (vi, HP33120A_ATTR_DRIVER_REVISION,          
                           hp33120aAttrDriverRevision_ReadCallback));

    checkErr( Ivi_SetAttributeViAddr (vi, VI_NULL, IVI_ATTR_OPC_CALLBACK, 0,
                      hp33120a_WaitForOPCCallback));
    checkErr( Ivi_SetAttributeViAddr (vi, VI_NULL, IVI_ATTR_CHECK_STATUS_CALLBACK, 0,
                      hp33120a_CheckStatusCallback));
    checkErr( Ivi_SetAttributeViBoolean (vi, VI_NULL, IVI_ATTR_SUPPORTS_WR_BUF_OPER_MODE, 
                     0, VI_TRUE));

    /*- Add instrument-specific attributes ------------------------------*/            
    checkErr( Ivi_AddAttributeViInt32   (vi, HP33120A_ATTR_OUTPUT_MODE,
                     "HP33120A_ATTR_OUTPUT_MODE",
                     HP33120A_VAL_OUTPUT_FUNC, 0,
                     VI_NULL,
                     hp33120aAttrOutputMode_WriteCallback,
                     &attrOutputModeRangeTable));
    checkErr( Ivi_SetAttrCheckCallbackViInt32 (vi, HP33120A_ATTR_OUTPUT_MODE,
                           hp33120aAttrOutputMode_CheckCallback));
                           
    checkErr( Ivi_AddAttributeViInt32   (vi, HP33120A_ATTR_REF_CLOCK_SOURCE,
                     "HP33120A_ATTR_REF_CLOCK_SOURCE",
                     HP33120A_VAL_INTERNAL, 0,
                     VI_NULL, VI_NULL,
                     &attrRefClockRangeTable));
    
    checkErr( Ivi_AddAttributeViBoolean (vi, HP33120A_ATTR_OUTPUT_ENABLED,
                     "HP33120A_ATTR_OUTPUT_ENABLED",
                     VI_TRUE, IVI_VAL_MULTI_CHANNEL,
                     VI_NULL, VI_NULL));
    checkErr( Ivi_SetAttrCheckCallbackViBoolean (vi, HP33120A_ATTR_OUTPUT_ENABLED,
                         hp33120aAttrOutputEnabled_CheckCallback));
                         
    checkErr( Ivi_AddAttributeViReal64 (vi, HP33120A_ATTR_OUTPUT_IMPEDANCE,
                    "HP33120A_ATTR_OUTPUT_IMPEDANCE",
                    HP33120A_VAL_50_OHMS, IVI_VAL_MULTI_CHANNEL,
                    hp33120aAttrOutputImpedance_ReadCallback,
                    hp33120aAttrOutputImpedance_WriteCallback,
                    &attrOutputImpedanceRangeTable, 0));
    
    checkErr( Ivi_AddAttributeViInt32  (vi, HP33120A_ATTR_FUNC_WAVEFORM,
                    "HP33120A_ATTR_FUNC_WAVEFORM",
                    HP33120A_VAL_WFM_SINE, IVI_VAL_MULTI_CHANNEL,
                    hp33120aAttrFuncWaveform_ReadCallback,
                    hp33120aAttrFuncWaveform_WriteCallback,
                    &attrFuncWaveformRangeTable));
    checkErr( Ivi_SetAttrCheckCallbackViInt32 (vi, HP33120A_ATTR_FUNC_WAVEFORM,
                           hp33120aAttrFuncWaveform_CheckCallback));
    
    checkErr( Ivi_AddAttributeViReal64 (vi, HP33120A_ATTR_FUNC_AMPLITUDE,
                    "HP33120A_ATTR_FUNC_AMPLITUDE",
                    2.00, IVI_VAL_MULTI_CHANNEL,
                    hp33120aAttrFuncAmplitude_ReadCallback,
                    hp33120aAttrFuncAmplitude_WriteCallback,
                    &attrFuncAmpRangeTable, 0));
    checkErr( Ivi_SetAttrCheckCallbackViReal64 (vi, HP33120A_ATTR_FUNC_AMPLITUDE,
                        hp33120aAttrFuncAmplitude_CheckCallback));
    
    checkErr( Ivi_AddAttributeViReal64 (vi, HP33120A_ATTR_FUNC_DC_OFFSET,
                    "HP33120A_ATTR_FUNC_DC_OFFSET",
                    0.00, IVI_VAL_MULTI_CHANNEL,
                    hp33120aAttrFuncDCOffset_ReadCallback,
                    hp33120aAttrFuncDCOffset_WriteCallback,
                    &attrFuncDCOffsetRangeTable, 0));
    checkErr( Ivi_SetAttrCheckCallbackViReal64 (vi, HP33120A_ATTR_FUNC_DC_OFFSET,
                        hp33120aAttrFuncDCOffset_CheckCallback));
                         
    checkErr( Ivi_AddAttributeViReal64 (vi, HP33120A_ATTR_FUNC_FREQUENCY,
                    "HP33120A_ATTR_FUNC_FREQUENCY",
                    1000.00, IVI_VAL_MULTI_CHANNEL,
                    hp33120aAttrFuncFrequency_ReadCallback,
                    hp33120aAttrFuncFrequency_WriteCallback,
                    VI_NULL, 0));
    checkErr( Ivi_SetAttrCheckCallbackViReal64 (vi, HP33120A_ATTR_FUNC_FREQUENCY,
                        hp33120aAttrFuncFrequency_CheckCallback));
    checkErr( Ivi_SetAttrRangeTableCallback (vi, HP33120A_ATTR_FUNC_FREQUENCY,
                             hp33120aAttrFuncFrequency_RangeTableCallback));
    
    checkErr( Ivi_AddAttributeViReal64 (vi, HP33120A_ATTR_FUNC_START_PHASE,
                    "HP33120A_ATTR_FUNC_START_PHASE",
                    0.00, IVI_VAL_MULTI_CHANNEL,
                    VI_NULL, VI_NULL, 
                    &attrFuncStartPhaseRangeTable, 0));
    checkErr( Ivi_AddAttributeViReal64 (vi, HP33120A_ATTR_FUNC_DUTY_CYCLE_HIGH,
                    "HP33120A_ATTR_FUNC_DUTY_CYCLE_HIGH",
                    50.00, IVI_VAL_MULTI_CHANNEL,
                    hp33120aAttrFuncDutyCycleHigh_ReadCallback,
                    hp33120aAttrFuncDutyCycleHigh_WriteCallback,
                    VI_NULL, 0));
    checkErr( Ivi_SetAttrCheckCallbackViReal64 (vi, HP33120A_ATTR_FUNC_DUTY_CYCLE_HIGH,
                        hp33120aAttrFuncDutyCycleHigh_CheckCallback));
    checkErr( Ivi_SetAttrRangeTableCallback (vi, HP33120A_ATTR_FUNC_DUTY_CYCLE_HIGH,
                         hp33120aAttrFuncDutyCycleHigh_RangeTableCallback));
    checkErr( Ivi_AddAttributeViInt32 (vi, HP33120A_ATTR_ARB_WAVEFORM_HANDLE,
                       "HP33120A_ATTR_ARB_WAVEFORM_HANDLE",
                       VI_NULL, IVI_VAL_MULTI_CHANNEL,
                       hp33120aAttrArbWaveformHandle_ReadCallback,
                       hp33120aAttrArbWaveformHandle_WriteCallback,
                       VI_NULL));
    checkErr( Ivi_SetAttrCheckCallbackViInt32 (vi, HP33120A_ATTR_ARB_WAVEFORM_HANDLE,
                           hp33120aAttrArbWaveformHandle_CheckCallback));
                           
    checkErr( Ivi_AddAttributeViReal64 (vi, HP33120A_ATTR_ARB_GAIN,
                    "HP33120A_ATTR_ARB_GAIN",
                    0.00, IVI_VAL_MULTI_CHANNEL,
                    hp33120aAttrArbGain_ReadCallback,
                    hp33120aAttrArbGain_WriteCallback,
                    &attrArbGainRangeTable, 0));
    checkErr( Ivi_SetAttrCheckCallbackViReal64 (vi, HP33120A_ATTR_ARB_GAIN,
                        hp33120aAttrArbGain_CheckCallback));
                         
    checkErr( Ivi_AddAttributeViReal64 (vi, HP33120A_ATTR_ARB_OFFSET,
                    "HP33120A_ATTR_ARB_OFFSET",
                    1.00, IVI_VAL_MULTI_CHANNEL,
                    hp33120aAttrArbOffset_ReadCallback,
                    hp33120aAttrArbOffset_WriteCallback,
                    &attrArbOffsetRangeTable, 0));
    checkErr( Ivi_SetAttrCheckCallbackViReal64 (vi, HP33120A_ATTR_ARB_OFFSET,
                        hp33120aAttrArbOffset_CheckCallback));
                    
    checkErr( Ivi_AddAttributeViReal64 (vi, HP33120A_ATTR_ARB_SAMPLE_RATE,
                    "HP33120A_ATTR_ARB_SAMPLE_RATE",
                    0.00, 0,
                    hp33120aAttrArbSampleRate_ReadCallback,
                    hp33120aAttrArbSampleRate_WriteCallback,
                    VI_NULL, 0));
    checkErr( Ivi_SetAttrCheckCallbackViReal64 (vi, HP33120A_ATTR_ARB_SAMPLE_RATE,
                        hp33120aAttrArbSampleRate_CheckCallback));
                         
    checkErr( Ivi_AddAttributeViInt32 (vi, HP33120A_ATTR_MAX_NUM_WAVEFORMS,
                       "HP33120A_ATTR_MAX_NUM_WAVEFORMS",
                       9, IVI_VAL_NOT_WRITABLE, VI_NULL,
                       VI_NULL, VI_NULL));
                       
    checkErr( Ivi_AddAttributeViInt32 (vi, HP33120A_ATTR_WAVEFORM_QUANTUM,
                       "HP33120A_ATTR_WAVEFORM_QUANTUM",
                       1, IVI_VAL_NOT_WRITABLE, VI_NULL,
                       VI_NULL, VI_NULL));
                       
    checkErr( Ivi_AddAttributeViInt32 (vi, HP33120A_ATTR_MIN_WAVEFORM_SIZE,
                       "HP33120A_ATTR_MIN_WAVEFORM_SIZE",
                       8, IVI_VAL_NOT_WRITABLE, VI_NULL,
                       VI_NULL, VI_NULL));
                       
    checkErr( Ivi_AddAttributeViInt32 (vi, HP33120A_ATTR_MAX_WAVEFORM_SIZE,
                       "HP33120A_ATTR_MAX_WAVEFORM_SIZE",
                       16000, IVI_VAL_NOT_WRITABLE, VI_NULL,
                       VI_NULL, VI_NULL));
                       
    checkErr( Ivi_AddAttributeViBoolean (vi, HP33120A_ATTR_AM_ENABLED,
                     "HP33120A_ATTR_AM_ENABLED",
                     VI_FALSE, IVI_VAL_MULTI_CHANNEL,
                     hp33120aAttrAMEnabled_ReadCallback,
                     hp33120aAttrAMEnabled_WriteCallback));
    checkErr( Ivi_SetAttrCheckCallbackViBoolean (vi, HP33120A_ATTR_AM_ENABLED,
                         hp33120aAttrAMEnabled_CheckCallback));
    
    checkErr( Ivi_AddAttributeViInt32 (vi, HP33120A_ATTR_AM_SOURCE,
                       "HP33120A_ATTR_AM_SOURCE",
                       HP33120A_VAL_INTERNAL,
                       IVI_VAL_MULTI_CHANNEL,
                       hp33120aAttrAMSource_ReadCallback,
                       hp33120aAttrAMSource_WriteCallback,
                       &attrAmSourceRangeTable));
                           
    checkErr( Ivi_AddAttributeViReal64 (vi, HP33120A_ATTR_AM_INTERNAL_DEPTH,
                    "HP33120A_ATTR_AM_INTERNAL_DEPTH",
                    100.00, 0,
                    hp33120aAttrAMInternalDepth_ReadCallback,
                    hp33120aAttrAMInternalDepth_WriteCallback,
                    &attrAmInternalDepthRangeTable, 0));
                        
    checkErr( Ivi_AddAttributeViInt32 (vi, HP33120A_ATTR_AM_INTERNAL_WAVEFORM,
                       "HP33120A_ATTR_AM_INTERNAL_WAVEFORM",
                       HP33120A_VAL_WFM_SINE, 0,
                       hp33120aAttrAMInternalWaveform_ReadCallback,
                       hp33120aAttrAMInternalWaveform_WriteCallback,
                       &attrAMInternalWaveformRangeTable));
                           
    checkErr( Ivi_AddAttributeViReal64 (vi, HP33120A_ATTR_AM_INTERNAL_FREQUENCY,
                    "HP33120A_ATTR_AM_INTERNAL_FREQUENCY",
                    1000.00, 0,
                    hp33120aAttrAMInternalFrequency_ReadCallback,
                    hp33120aAttrAMInternalFrequency_WriteCallback,
                    &attrAmInternalFrequencyRangeTable, 0));
    checkErr( Ivi_AddAttributeViBoolean (vi, HP33120A_ATTR_FM_ENABLED,
                     "HP33120A_ATTR_FM_ENABLED",
                     VI_FALSE, IVI_VAL_MULTI_CHANNEL,
                     hp33120aAttrFMEnabled_ReadCallback,
                     hp33120aAttrFMEnabled_WriteCallback));
    checkErr( Ivi_SetAttrCheckCallbackViBoolean (vi, HP33120A_ATTR_FM_ENABLED,
                         hp33120aAttrFMEnabled_CheckCallback));
    
    checkErr( Ivi_AddAttributeViInt32 (vi, HP33120A_ATTR_FM_SOURCE,
                       "HP33120A_ATTR_FM_SOURCE",
                       HP33120A_VAL_INTERNAL, IVI_VAL_MULTI_CHANNEL,
                       VI_NULL, VI_NULL, &attrFmSourceRangeTable));
                           
    checkErr (Ivi_AddAttributeViReal64 (vi, HP33120A_ATTR_FM_INTERNAL_DEVIATION,
                    "HP33120A_ATTR_FM_INTERNAL_DEVIATION", 0.00,
                    IVI_VAL_MULTI_CHANNEL,
                    hp33120aAttrFMInternalDeviation_ReadCallback,
                    hp33120aAttrFMInternalDeviation_WriteCallback,
                    VI_NULL, 0));
    checkErr( Ivi_SetAttrCheckCallbackViReal64 (vi, HP33120A_ATTR_FM_INTERNAL_DEVIATION,
                        hp33120aAttrFMInternalDeviation_CheckCallback));
                        
    checkErr( Ivi_AddAttributeViInt32 (vi, HP33120A_ATTR_FM_INTERNAL_WAVEFORM,
                       "HP33120A_ATTR_FM_INTERNAL_WAVEFORM",
                       HP33120A_VAL_WFM_SINE, 0,
                       hp33120aAttrFMInternalWaveform_ReadCallback,
                       hp33120aAttrFMInternalWaveform_WriteCallback,
                       VI_NULL));
    
    checkErr( Ivi_AddAttributeViReal64 (vi, HP33120A_ATTR_FM_INTERNAL_FREQUENCY,
                    "HP33120A_ATTR_FM_INTERNAL_FREQUENCY",
                    1000.00, 0,
                    hp33120aAttrFMInternalFrequency_ReadCallback,
                    hp33120aAttrFMInternalFrequency_WriteCallback,
                    &attrFmInternalFrequencyRangeTable, 0));
    
    checkErr (Ivi_AddAttributeViBoolean (vi, HP33120A_ATTR_FSK_ENABLED,
                     "HP33120A_ATTR_FSK_ENABLED", VI_FALSE, 0,
                     hp33120aAttrFSKEnabled_ReadCallback,
                     hp33120aAttrFSKEnabled_WriteCallback));
    checkErr( Ivi_SetAttrCheckCallbackViBoolean (vi, HP33120A_ATTR_FSK_ENABLED,
                         hp33120aAttrFSKEnabled_CheckCallback));
                         
    checkErr (Ivi_AddAttributeViInt32 (vi, HP33120A_ATTR_FSK_SOURCE,
                       "HP33120A_ATTR_FSK_SOURCE",
                       HP33120A_VAL_INTERNAL, 0,
                       hp33120aAttrFSKSource_ReadCallback,
                       hp33120aAttrFSKSource_WriteCallback,
                       &attrFSKSourceRangeTable));
                            
    checkErr( Ivi_AddAttributeViReal64 (vi, HP33120A_ATTR_FSK_INTERNAL_RATE,
                    "HP33120A_ATTR_FSK_INTERNAL_RATE",
                    1000.00, 0,
                    hp33120aAttrFSKInternalRate_ReadCallback,
                    hp33120aAttrFSKInternalRate_WriteCallback,
                    &attrFSKInternalRateRangeTable, 0));
                         
    checkErr( Ivi_AddAttributeViReal64 (vi, HP33120A_ATTR_FSK_HOP_FREQUENCY,
                    "HP33120A_ATTR_FSK_HOP_FREQUENCY",
                    1000.00, 0,
                    hp33120aAttrFSKHopFreq_ReadCallback,
                    hp33120aAttrFSKHopFreq_WriteCallback,
                    &attrFSKInternalRateRangeTable, 0));
    checkErr( Ivi_SetAttrCheckCallbackViReal64 (vi, HP33120A_ATTR_FSK_HOP_FREQUENCY,
                        hp33120aAttrFSKHopFreq_CheckCallback));
    checkErr (Ivi_AddAttributeViBoolean (vi, HP33120A_ATTR_SWEEP_ENABLED,
                     "HP33120A_ATTR_SWEEP_ENABLED", VI_FALSE, 0,
                     hp33120aAttrSweepEnabled_ReadCallback,
                     hp33120aAttrSweepEnabled_WriteCallback));
    checkErr( Ivi_SetAttrCheckCallbackViBoolean (vi, HP33120A_ATTR_SWEEP_ENABLED,
                         hp33120aAttrSweepEnabled_CheckCallback));
                         
    checkErr (Ivi_AddAttributeViReal64 (vi, HP33120A_ATTR_SWEEP_START_FREQ,
                    "HP33120A_ATTR_SWEEP_START_FREQ", 1000.0, 0,
                    hp33120aAttrSweepStartFreq_ReadCallback,
                    hp33120aAttrSweepStartFreq_WriteCallback,
                    VI_NULL, 0));
    checkErr( Ivi_SetAttrCheckCallbackViReal64 (vi, HP33120A_ATTR_SWEEP_START_FREQ,
                        hp33120aAttrSweepStartFreq_CheckCallback));
                        
    checkErr (Ivi_AddAttributeViReal64 (vi, HP33120A_ATTR_SWEEP_STOP_FREQ,
                    "HP33120A_ATTR_SWEEP_STOP_FREQ", 1000.0, 0,
                    hp33120aAttrSweepStopFreq_ReadCallback,
                    hp33120aAttrSweepStopFreq_WriteCallback,
                    VI_NULL, 0));
    checkErr( Ivi_SetAttrCheckCallbackViReal64 (vi, HP33120A_ATTR_SWEEP_STOP_FREQ,
                        hp33120aAttrSweepStopFreq_CheckCallback));
    
    checkErr (Ivi_AddAttributeViInt32 (vi, HP33120A_ATTR_SWEEP_SPACING,
                       "HP33120A_ATTR_SWEEP_SPACING",
                       HP33120A_VAL_LINEAR, 0,
                       hp33120aAttrSweepSpacing_ReadCallback,
                       hp33120aAttrSweepSpacing_WriteCallback,
                       &attrSweepSpacingRangeTable));
    
    checkErr (Ivi_AddAttributeViReal64 (vi, HP33120A_ATTR_SWEEP_TIME,
                    "HP33120A_ATTR_SWEEP_TIME",
                    HP33120A_VAL_LINEAR, 0,
                    hp33120aAttrSweepTime_ReadCallback,
                    hp33120aAttrSweepTime_WriteCallback,
                    &attrSweepTimeRangeTable, 0));
                           
    checkErr (Ivi_AddAttributeViInt32 (vi, HP33120A_ATTR_SWEEP_SOURCE,
                       "HP33120A_ATTR_SWEEP_SOURCE",
                       HP33120A_VAL_IMMEDIATE, 0,
                       hp33120aAttrSweepSource_ReadCallback,
                       hp33120aAttrSweepSource_WriteCallback,
                       &attrSweepSourceRangeTable));
    
    checkErr (Ivi_AddAttributeViBoolean (vi, HP33120A_ATTR_BURST_ENABLED,
                     "HP33120A_ATTR_BURST_ENABLED", VI_FALSE, 0,
                     hp33120aAttrBurstEnabled_ReadCallback,
                     hp33120aAttrBurstEnabled_WriteCallback));
    checkErr( Ivi_SetAttrCheckCallbackViBoolean (vi, HP33120A_ATTR_BURST_ENABLED,
                    hp33120aAttrBurstEnabled_CheckCallback));
    
    checkErr (Ivi_AddAttributeViInt32 (vi, HP33120A_ATTR_BURST_SOURCE,
                       "HP33120A_ATTR_BURST_SOURCE",
                       HP33120A_VAL_INTERNAL, 0,
                       hp33120aAttrBurstSource_ReadCallback,
                       hp33120aAttrBurstSource_WriteCallback,
                       &attrBurstSourceRangeTable));
                       
    checkErr (Ivi_AddAttributeViReal64 (vi, HP33120A_ATTR_BURST_INTERNAL_RATE,
                    "HP33120A_ATTR_BURST_INTERNAL_RATE", 1000.0,
                    0, hp33120aAttrBurstRate_ReadCallback,
                    hp33120aAttrBurstRate_WriteCallback,
                    &attrBurstRateRangeTable, 0));
    
    checkErr (Ivi_AddAttributeViReal64 (vi, HP33120A_ATTR_BURST_PHASE,
                    "HP33120A_ATTR_BURST_PHASE", 1000.0, 0,
                    hp33120aAttrBurstPhase_ReadCallback,
                    hp33120aAttrBurstPhase_WriteCallback,
                    &attrBurstPhaseRangeTable, 0));
    
    checkErr (Ivi_AddAttributeViInt32 (vi, HP33120A_ATTR_BURST_COUNT,
                       "HP33120A_ATTR_BURST_COUNT",
                       HP33120A_VAL_INTERNAL, 0,
                       hp33120aAttrBurstCount_ReadCallback,
                       hp33120aAttrBurstCount_WriteCallback, VI_NULL));
                       
    checkErr( Ivi_SetAttrCheckCallbackViInt32 (vi, HP33120A_ATTR_BURST_COUNT,
                           hp33120aAttrBurstCount_CheckCallback));
                           
    checkErr( Ivi_AddAttributeViAddr (vi, HP33120A_ATTR_WFM_STRUCTURE,
                      "HP33120A_ATTR_WFM_STRUCTURE",
                      VI_NULL, IVI_VAL_HIDDEN, VI_NULL, VI_NULL));
    
    checkErr( Ivi_AddAttributeViString (vi, HP33120A_ATTR_ID_QUERY_RESPONSE,
                    "HP33120A_ATTR_ID_QUERY_RESPONSE", 
                    "HEWLETT-PACKARD,33120A",
                    IVI_VAL_NOT_USER_WRITABLE,
                    hp33120aAttrIdQueryResponse_ReadCallback,
                    VI_NULL));
                           

    checkErr( Ivi_AddAttributeViInt32 (vi, HP33120A_ATTR_OPC_TIMEOUT,
                       "HP33120A_ATTR_OPC_TIMEOUT",
                       5000, IVI_VAL_HIDDEN | IVI_VAL_DONT_CHECK_STATUS,
                       VI_NULL, VI_NULL, VI_NULL)); 
    /*- Add Invalidations -*/
    checkErr( Ivi_AddAttributeInvalidation (vi, HP33120A_ATTR_OUTPUT_IMPEDANCE,
                        HP33120A_ATTR_FUNC_AMPLITUDE, VI_TRUE));
    checkErr( Ivi_AddAttributeInvalidation (vi, HP33120A_ATTR_OUTPUT_IMPEDANCE,
                        HP33120A_ATTR_FUNC_DC_OFFSET, VI_TRUE));
    checkErr( Ivi_AddAttributeInvalidation (vi, HP33120A_ATTR_OUTPUT_IMPEDANCE,
                        HP33120A_ATTR_ARB_GAIN, VI_TRUE));
    checkErr( Ivi_AddAttributeInvalidation (vi, HP33120A_ATTR_OUTPUT_IMPEDANCE,
                        HP33120A_ATTR_ARB_OFFSET, VI_TRUE));
    
    checkErr( Ivi_AddAttributeInvalidation (vi, HP33120A_ATTR_OUTPUT_MODE,
                        HP33120A_ATTR_FUNC_WAVEFORM, VI_TRUE));
    checkErr( Ivi_AddAttributeInvalidation (vi, HP33120A_ATTR_OUTPUT_MODE,
                        HP33120A_ATTR_FUNC_AMPLITUDE, VI_TRUE));
    checkErr( Ivi_AddAttributeInvalidation (vi, HP33120A_ATTR_OUTPUT_MODE,
                        HP33120A_ATTR_FUNC_DC_OFFSET, VI_TRUE));
    checkErr( Ivi_AddAttributeInvalidation (vi, HP33120A_ATTR_OUTPUT_MODE,
                        HP33120A_ATTR_FUNC_FREQUENCY, VI_TRUE));
    checkErr( Ivi_AddAttributeInvalidation (vi, HP33120A_ATTR_OUTPUT_MODE,
                        HP33120A_ATTR_FUNC_START_PHASE, VI_TRUE));
    checkErr( Ivi_AddAttributeInvalidation (vi, HP33120A_ATTR_OUTPUT_MODE,
                        HP33120A_ATTR_FUNC_DUTY_CYCLE_HIGH, VI_TRUE));
    checkErr( Ivi_AddAttributeInvalidation (vi, HP33120A_ATTR_OUTPUT_MODE,
                        HP33120A_ATTR_ARB_WAVEFORM_HANDLE, VI_TRUE));
    checkErr( Ivi_AddAttributeInvalidation (vi, HP33120A_ATTR_OUTPUT_MODE,
                        HP33120A_ATTR_ARB_GAIN, VI_TRUE));
    checkErr( Ivi_AddAttributeInvalidation (vi, HP33120A_ATTR_OUTPUT_MODE,
                        HP33120A_ATTR_ARB_OFFSET, VI_TRUE));
    checkErr( Ivi_AddAttributeInvalidation (vi, HP33120A_ATTR_OUTPUT_MODE,
                        HP33120A_ATTR_ARB_SAMPLE_RATE, VI_TRUE));
    
    checkErr( Ivi_AddAttributeInvalidation (vi, HP33120A_ATTR_FUNC_WAVEFORM,
                        HP33120A_ATTR_FUNC_AMPLITUDE, VI_FALSE));
    checkErr( Ivi_AddAttributeInvalidation (vi, HP33120A_ATTR_FUNC_WAVEFORM,
                        HP33120A_ATTR_FUNC_DC_OFFSET, VI_FALSE));
    checkErr( Ivi_AddAttributeInvalidation (vi, HP33120A_ATTR_FUNC_WAVEFORM,
                        HP33120A_ATTR_FUNC_FREQUENCY, VI_FALSE));
    
    checkErr( Ivi_AddAttributeInvalidation (vi, HP33120A_ATTR_FUNC_AMPLITUDE,
                        HP33120A_ATTR_FUNC_DC_OFFSET, VI_FALSE));
                        
    checkErr( Ivi_AddAttributeInvalidation (vi, HP33120A_ATTR_ARB_WAVEFORM_HANDLE,
                        HP33120A_ATTR_ARB_SAMPLE_RATE, VI_FALSE));
    checkErr( Ivi_AddAttributeInvalidation (vi, HP33120A_ATTR_ARB_WAVEFORM_HANDLE,
                        HP33120A_ATTR_ARB_GAIN, VI_FALSE));
    checkErr( Ivi_AddAttributeInvalidation (vi, HP33120A_ATTR_ARB_WAVEFORM_HANDLE,
                        HP33120A_ATTR_ARB_OFFSET, VI_FALSE));

    checkErr( Ivi_AddAttributeInvalidation (vi, HP33120A_ATTR_ARB_GAIN,
                        HP33120A_ATTR_ARB_OFFSET, VI_FALSE));
                        
    checkErr( Ivi_AddAttributeInvalidation (vi, HP33120A_ATTR_AM_ENABLED,
                        HP33120A_ATTR_FUNC_AMPLITUDE, VI_TRUE));
    checkErr( Ivi_AddAttributeInvalidation (vi, HP33120A_ATTR_AM_ENABLED,
                        HP33120A_ATTR_ARB_GAIN, VI_TRUE));
    checkErr (Ivi_AddAttributeInvalidation (vi, HP33120A_ATTR_SWEEP_ENABLED,
                        HP33120A_ATTR_SWEEP_START_FREQ, VI_TRUE));
    checkErr (Ivi_AddAttributeInvalidation (vi, HP33120A_ATTR_SWEEP_ENABLED,
                        HP33120A_ATTR_SWEEP_STOP_FREQ, VI_TRUE));
    checkErr (Ivi_AddAttributeInvalidation (vi, HP33120A_ATTR_FUNC_WAVEFORM,
                        HP33120A_ATTR_FUNC_DUTY_CYCLE_HIGH,
                        VI_FALSE));
    checkErr (Ivi_AddAttributeInvalidation (vi, HP33120A_ATTR_FUNC_FREQUENCY,
                        HP33120A_ATTR_FUNC_DUTY_CYCLE_HIGH,
                        VI_FALSE));
    checkErr (Ivi_AddAttributeInvalidation (vi, HP33120A_ATTR_FM_ENABLED,
                        HP33120A_ATTR_FM_INTERNAL_DEVIATION,
                        VI_FALSE));
    checkErr (Ivi_AddAttributeInvalidation (vi, HP33120A_ATTR_BURST_ENABLED,
                        HP33120A_ATTR_BURST_INTERNAL_RATE,
                        VI_TRUE));
Error:
    return error;
}
    
/***************************************************************************** 
 *------------------- End Instrument Driver Source Code ---------------------* 
 *****************************************************************************/
